@@ -26,6 +26,11 @@
 #include "graphics.h"
 #include "backends/rendering.h"
 
+#if LIBAVUTIL_VERSION_MAJOR < 51
+#define AVMEDIA_TYPE_VIDEO CODEC_TYPE_VIDEO
+#define AVMEDIA_TYPE_AUDIO CODEC_TYPE_AUDIO
+#endif
+
 using namespace lightspark;
 using namespace std;
 
@@ -35,7 +40,7 @@ bool VideoDecoder::setSize(uint32_t w, uint32_t h)
 	{
 		frameWidth=w;
 		frameHeight=h;
-		LOG(LOG_NO_INFO,_("VIDEO DEC: Video frame size ") << frameWidth << 'x' << frameHeight);
+		LOG(LOG_INFO,_("VIDEO DEC: Video frame size ") << frameWidth << 'x' << frameHeight);
 		resizeGLBuffers=true;
 		videoTexture=sys->getRenderThread()->allocateTexture(frameWidth, frameHeight, true);
 		return true;
@@ -203,7 +208,7 @@ void FFMpegVideoDecoder::setSize(uint32_t w, uint32_t h)
 		while(discardFrame());
 
 		//As the size chaged, reset the buffer
-		uint32_t bufferSize=frameWidth*frameHeight;
+		uint32_t bufferSize=frameWidth*frameHeight/**4*/;
 		buffers.regen(YUVBufferGenerator(bufferSize));
 	}
 }
@@ -260,7 +265,7 @@ bool FFMpegVideoDecoder::decodeData(uint8_t* data, uint32_t datalen, uint32_t ti
 		if(status==INIT && fillDataAndCheckValidity())
 			status=VALID;
 
-		assert(frameIn->pts==AV_NOPTS_VALUE || frameIn->pts==0);
+		assert(frameIn->pts==(int64_t)AV_NOPTS_VALUE || frameIn->pts==0);
 
 		copyFrameToBuffers(frameIn, time);
 	}
@@ -270,7 +275,13 @@ bool FFMpegVideoDecoder::decodeData(uint8_t* data, uint32_t datalen, uint32_t ti
 bool FFMpegVideoDecoder::decodePacket(AVPacket* pkt, uint32_t time)
 {
 	int frameOk=0;
+
+#if HAVE_AVCODEC_DECODE_VIDEO2
 	int ret=avcodec_decode_video2(codecContext, frameIn, &frameOk, pkt);
+#else
+	int ret=avcodec_decode_video(codecContext, frameIn, &frameOk, pkt->data, pkt->size);
+#endif
+
 	assert_and_throw(ret==(int)pkt->size);
 	if(frameOk)
 	{
@@ -279,7 +290,7 @@ bool FFMpegVideoDecoder::decodePacket(AVPacket* pkt, uint32_t time)
 		if(status==INIT && fillDataAndCheckValidity())
 			status=VALID;
 
-		assert(frameIn->pts==AV_NOPTS_VALUE || frameIn->pts==0);
+		assert(frameIn->pts==(int64_t)AV_NOPTS_VALUE || frameIn->pts==0);
 
 		copyFrameToBuffers(frameIn, time);
 	}
@@ -497,7 +508,7 @@ bool FFMpegAudioDecoder::fillDataAndCheckValidity()
 {
 	if(codecContext->sample_rate!=0)
 	{
-		LOG(LOG_NO_INFO,_("AUDIO DEC: Audio sample rate ") << codecContext->sample_rate);
+		LOG(LOG_INFO,_("AUDIO DEC: Audio sample rate ") << codecContext->sample_rate);
 		sampleRate=codecContext->sample_rate;
 	}
 	else
@@ -505,12 +516,20 @@ bool FFMpegAudioDecoder::fillDataAndCheckValidity()
 
 	if(codecContext->channels!=0)
 	{
-		LOG(LOG_NO_INFO, _("AUDIO DEC: Audio channels ") << codecContext->channels);
+		LOG(LOG_INFO, _("AUDIO DEC: Audio channels ") << codecContext->channels);
 		channelCount=codecContext->channels;
 	}
 	else
 		return false;
 
+	if(initialTime==(uint32_t)-1 && !samplesBuffer.isEmpty())
+	{
+		initialTime=getFrontTime();
+		LOG(LOG_INFO,_("AUDIO DEC: Initial timestamp ") << initialTime);
+	}
+	else
+		return false;
+
 	return true;
 }
 
@@ -545,7 +564,24 @@ uint32_t FFMpegAudioDecoder::decodePacket(AVPacket* pkt, uint32_t time)
 {
 	FrameSamples& curTail=samplesBuffer.acquireLast();
 	int maxLen=AVCODEC_MAX_AUDIO_FRAME_SIZE;
-	uint32_t ret=avcodec_decode_audio3(codecContext, curTail.samples, &maxLen, pkt);
+
+#if HAVE_AVCODEC_DECODE_AUDIO3
+	int ret=avcodec_decode_audio3(codecContext, curTail.samples, &maxLen, pkt);
+#else
+	int ret=avcodec_decode_audio2(codecContext, curTail.samples, &maxLen, pkt->data, pkt->size);
+#endif
+
+	if(ret==-1)
+	{
+		//A decoding error occurred, create an empty sample buffer
+		LOG(LOG_ERROR,_("Malformed audio packet"));
+		curTail.len=0;
+		curTail.current=curTail.samples;
+		curTail.time=time;
+		samplesBuffer.commitLast();
+		return maxLen;
+	}
+
 	assert_and_throw(ret==pkt->size);
 
 	if(status==INIT && fillDataAndCheckValidity())
@@ -575,16 +611,21 @@ FFMpegStreamDecoder::FFMpegStreamDecoder(std::istream& s):stream(s),formatCtx(NU
 	if(avioContext==NULL)
 		return;
 
+#if LIBAVFORMAT_VERSION_MAJOR > 52 || (LIBAVFORMAT_VERSION_MAJOR == 52  && LIBAVFORMAT_VERSION_MINOR > 64)
+ 	avioContext->seekable = 0;
+#else
 	avioContext->is_streamed=1;
+#endif
+	
 	//Probe the stream format.
 	//NOTE: in FFMpeg 0.7 there is av_probe_input_buffer
 	AVProbeData probeData;
 	probeData.filename="lightspark_stream";
-	probeData.buf=new uint8_t[64+AVPROBE_PADDING_SIZE];
-	memset(probeData.buf,0,64+AVPROBE_PADDING_SIZE);
-	stream.read((char*)probeData.buf,64);
+	probeData.buf=new uint8_t[8192+AVPROBE_PADDING_SIZE];
+	memset(probeData.buf,0,8192+AVPROBE_PADDING_SIZE);
+	stream.read((char*)probeData.buf,8192);
 	int read=stream.gcount();
-	if(read!=64)
+	if(read!=8192)
 		LOG(LOG_ERROR,_("Not sufficient data is available from the stream"));
 	probeData.buf_size=read;
 
@@ -596,7 +637,7 @@ FFMpegStreamDecoder::FFMpegStreamDecoder(std::istream& s):stream(s),formatCtx(NU
 		return;
 
 	int ret=av_open_input_stream(&formatCtx, avioContext, "lightspark_stream", fmt, NULL);
-	if(ret!=0)
+	if(ret<0)
 		return;
 	
 	ret=av_find_stream_info(formatCtx);
@@ -609,12 +650,12 @@ FFMpegStreamDecoder::FFMpegStreamDecoder(std::istream& s):stream(s),formatCtx(NU
 		if(formatCtx->streams[i]->codec->codec_type==AVMEDIA_TYPE_VIDEO && videoFound==false)
 		{
 			videoFound=true;
-			videoIndex=i;
+			videoIndex=(int32_t)i;
 		}
 		else if(formatCtx->streams[i]->codec->codec_type==AVMEDIA_TYPE_AUDIO && audioFound==false)
 		{
 			audioFound=true;
-			audioIndex=i;
+			audioIndex=(int32_t)i;
 		}
 
 	}
@@ -637,6 +678,11 @@ FFMpegStreamDecoder::FFMpegStreamDecoder(std::istream& s):stream(s),formatCtx(NU
 
 FFMpegStreamDecoder::~FFMpegStreamDecoder()
 {
+	//Delete the decoders before deleting the input stream to avoid a crash in ffmpeg code
+	delete audioDecoder;
+	delete videoDecoder;
+	audioDecoder=NULL;
+	videoDecoder=NULL;
 	if(formatCtx)
 		av_close_input_stream(formatCtx);
 	if(avioContext)
@@ -653,7 +699,7 @@ bool FFMpegStreamDecoder::decodeNextFrame()
 	//Should use dts
 	uint32_t mtime=pkt.dts*1000*time_base.num/time_base.den;
 
-	if(pkt.stream_index==audioIndex)
+	if(pkt.stream_index==(int)audioIndex)
 		customAudioDecoder->decodePacket(&pkt, mtime);
 	else
 		customVideoDecoder->decodePacket(&pkt, mtime);
@@ -668,6 +714,11 @@ bool FFMpegStreamDecoder::getMetadataInteger(const char* name, uint32_t& ret) co
 
 bool FFMpegStreamDecoder::getMetadataDouble(const char* name, double& ret) const
 {
+	if( string(name) == "duration" )
+	{
+		ret = double(formatCtx->duration) / double(AV_TIME_BASE);
+		return true;
+	}
 	return false;
 }
 
