@@ -24,7 +24,9 @@
 #include "logger.h"
 #include "exceptions.h"
 #include "backends/rendering.h"
+#include "glmatrices.h"
 #include "compat.h"
+#include "scripting/flash/text/flashtext.h"
 
 #include <iostream>
 
@@ -33,23 +35,23 @@ using namespace lightspark;
 void lightspark::cleanGLErrors()
 {
 #ifdef EXPENSIVE_DEBUG
-	int glErrorCount = 0;
+	int errorCount = 0;
 	GLenum err;
 	while(1)
 	{
 		err=glGetError();
 		if(err!=GL_NO_ERROR)
 		{
-			glErrorCount++;
+			errorCount++;
 			LOG(LOG_ERROR,_("GL error ")<< err);
 		}
 		else
 			break;
 	}
 
-	if(glErrorCount)
+	if(errorCount)
 	{
-		LOG(LOG_ERROR,_("Ignoring ") << glErrorCount << _(" openGL errors"));
+		LOG(LOG_ERROR,_("Ignoring ") << errorCount << _(" openGL errors"));
 	}
 #else
 	while(glGetError()!=GL_NO_ERROR);
@@ -98,45 +100,7 @@ TextureBuffer::TextureBuffer(bool initNow, uint32_t w, uint32_t h, GLenum f):tex
 	width(w),height(h),horizontalAlignment(1),verticalAlignment(1),inited(false)
 {
 	if(initNow)
-		init();
-}
-
-void TextureBuffer::init()
-{
-	assert(!inited);
-	inited=true;
-	cleanGLErrors();
-	
-	setAllocSize(width,height);
-	assert(texId==0);
-	glGenTextures(1,&texId);
-	assert(texId!=0);
-	assert(glGetError()!=GL_INVALID_OPERATION);
-	
-	assert(filtering==GL_NEAREST || filtering==GL_LINEAR);
-	
-	//If the previous call has not failed these should not fail (in specs, we trust)
-	glBindTexture(GL_TEXTURE_2D,texId);
-	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,filtering);
-	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,filtering);
-	//Wrapping should not be very useful, we use textures carefully
-	//glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
-	
-	//Allocate the texture
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, allocWidth, allocHeight, 0, GL_BGRA, GL_UNSIGNED_BYTE, 0);
-	GLenum err=glGetError();
-	assert(err!=GL_INVALID_OPERATION);
-	if(err==GL_INVALID_VALUE)
-	{
-		LOG(LOG_ERROR,_("GL_INVALID_VALUE after glTexImage2D, width=") << allocWidth << _(" height=") << allocHeight);
-		throw RunTimeException("GL_INVALID_VALUE in TextureBuffer::init");
-	}
-	
-	glBindTexture(GL_TEXTURE_2D,0);
-	
-#ifdef EXPENSIVE_DEBUG
-	cleanGLErrors();
-#endif
+		init(w, h, f);
 }
 
 void TextureBuffer::init(uint32_t w, uint32_t h, GLenum f)
@@ -208,23 +172,6 @@ void TextureBuffer::resize(uint32_t w, uint32_t h)
 	}
 }
 
-void TextureBuffer::setBGRAData(uint8_t* bgraData, uint32_t w, uint32_t h)
-{
-	cleanGLErrors();
-
-	//First of all resize the texture (if needed)
-	resize(w, h);
-
-	glBindTexture(GL_TEXTURE_2D,texId);
-
-	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, width, height, GL_BGRA, GL_UNSIGNED_BYTE, bgraData);
-	assert(glGetError()==GL_NO_ERROR);
-	
-#ifdef EXPENSIVE_DEBUG
-	cleanGLErrors();
-#endif
-}
-
 void TextureBuffer::setRequestedAlignment(uint32_t w, uint32_t h)
 {
 	assert(w && h);
@@ -276,9 +223,7 @@ TextureBuffer::~TextureBuffer()
 MatrixApplier::MatrixApplier()
 {
 	//First of all try to preserve current matrix
-	glPushMatrix();
-	if(glGetError()==GL_STACK_OVERFLOW)
-		throw RunTimeException("GL matrix stack exceeded");
+	lsglPushMatrix();
 
 	//TODO: implement smart stack flush
 	//Save all the current stack, compute using SSE the final matrix and push that one
@@ -289,27 +234,26 @@ MatrixApplier::MatrixApplier()
 MatrixApplier::MatrixApplier(const MATRIX& m)
 {
 	//First of all try to preserve current matrix
-	glPushMatrix();
-	if(glGetError()==GL_STACK_OVERFLOW)
-	{
-		::abort();
-	}
+	lsglPushMatrix();
 
 	float matrix[16];
 	m.get4DMatrix(matrix);
-	glMultMatrixf(matrix);
+	lsglMultMatrixf(matrix);
+	rt->setMatrixUniform(LSGL_MODELVIEW);
 }
 
 void MatrixApplier::concat(const MATRIX& m)
 {
 	float matrix[16];
 	m.get4DMatrix(matrix);
-	glMultMatrixf(matrix);
+	lsglMultMatrixf(matrix);
+	rt->setMatrixUniform(LSGL_MODELVIEW);
 }
 
 void MatrixApplier::unapply()
 {
-	glPopMatrix();
+	lsglPopMatrix();
+	rt->setMatrixUniform(LSGL_MODELVIEW);
 }
 
 TextureChunk::TextureChunk(uint32_t w, uint32_t h)
@@ -321,8 +265,8 @@ TextureChunk::TextureChunk(uint32_t w, uint32_t h)
 		chunks=NULL;
 		return;
 	}
-	const uint32_t blocksW=(w+127)/128;
-	const uint32_t blocksH=(h+127)/128;
+	const uint32_t blocksW=(w+CHUNKSIZE-1)/CHUNKSIZE;
+	const uint32_t blocksH=(h+CHUNKSIZE-1)/CHUNKSIZE;
 	chunks=new uint32_t[blocksW*blocksH];
 }
 
@@ -344,8 +288,8 @@ TextureChunk& TextureChunk::operator=(const TextureChunk& r)
 	}
 	width=r.width;
 	height=r.height;
-	uint32_t blocksW=(width+127)/128;
-	uint32_t blocksH=(height+127)/128;
+	uint32_t blocksW=(width+CHUNKSIZE-1)/CHUNKSIZE;
+	uint32_t blocksH=(height+CHUNKSIZE-1)/CHUNKSIZE;
 	texId=r.texId;
 	if(r.chunks)
 	{
@@ -383,9 +327,9 @@ bool TextureChunk::resizeIfLargeEnough(uint32_t w, uint32_t h)
 		height=h;
 		return true;
 	}
-	const uint32_t blocksW=(width+127)/128;
-	const uint32_t blocksH=(height+127)/128;
-	if(w<=blocksW*128 && h<=blocksH*128)
+	const uint32_t blocksW=(width+CHUNKSIZE-1)/CHUNKSIZE;
+	const uint32_t blocksH=(height+CHUNKSIZE-1)/CHUNKSIZE;
+	if(w<=blocksW*CHUNKSIZE && h<=blocksH*CHUNKSIZE)
 	{
 		width=w;
 		height=h;
@@ -394,10 +338,10 @@ bool TextureChunk::resizeIfLargeEnough(uint32_t w, uint32_t h)
 	return false;
 }
 
-CairoRenderer::CairoRenderer(ASObject* _o, CachedSurface& _t, const std::vector<GeomToken>& _g, const MATRIX& _m,
-		int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s)
-	: owner(_o),surface(_t),matrix(_m),xOffset(_x),yOffset(_y),width(_w),height(_h),
-	surfaceBytes(NULL),tokens(_g),scaleFactor(_s),uploadNeeded(true)
+CairoRenderer::CairoRenderer(ASObject* _o, CachedSurface& _t, const MATRIX& _m,
+		int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s, float _a)
+	: owner(_o),surface(_t),matrix(_m),xOffset(_x),yOffset(_y),alpha(_a),width(_w),height(_h),
+	surfaceBytes(NULL),scaleFactor(_s),uploadNeeded(true)
 {
 	owner->incRef();
 }
@@ -422,11 +366,14 @@ void CairoRenderer::upload(uint8_t* data, uint32_t w, uint32_t h) const
 
 const TextureChunk& CairoRenderer::getTexture()
 {
+	/* This is called in the render thread,
+	 * so we need no locking for surface */
 	//Verify that the texture is large enough
 	if(!surface.tex.resizeIfLargeEnough(width, height))
 		surface.tex=sys->getRenderThread()->allocateTexture(width, height,false);
 	surface.xOffset=xOffset;
 	surface.yOffset=yOffset;
+	surface.alpha=alpha;
 	return surface.tex;
 }
 
@@ -462,7 +409,7 @@ void CairoRenderer::jobFence()
 		delete this;
 }
 
-void CairoRenderer::quadraticBezier(cairo_t* cr, double control_x, double control_y, double end_x, double end_y)
+void CairoTokenRenderer::quadraticBezier(cairo_t* cr, double control_x, double control_y, double end_x, double end_y)
 {
 	double start_x, start_y;
 	cairo_get_current_point(cr, &start_x, &start_y);
@@ -476,7 +423,7 @@ void CairoRenderer::quadraticBezier(cairo_t* cr, double control_x, double contro
 	               end_x, end_y);
 }
 
-cairo_pattern_t* CairoRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, double scaleCorrection)
+cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, double scaleCorrection)
 {
 	cairo_pattern_t* pattern = NULL;
 
@@ -551,19 +498,16 @@ cairo_pattern_t* CairoRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, double
 				throw RunTimeException("Invalid bitmap");
 
 			IntSize size = style.bitmap->getBitmapSize();
-			//TODO: ARGB32 always give a white surface
+
 			cairo_surface_t* surface = cairo_image_surface_create_for_data (style.bitmap->data,
-										CAIRO_FORMAT_RGB24, size.width, size.height,
-										cairo_format_stride_for_width(CAIRO_FORMAT_RGB24, size.width));
+										CAIRO_FORMAT_ARGB32, size.width, size.height,
+										cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, size.width));
 
 			pattern = cairo_pattern_create_for_surface(surface);
 			cairo_surface_destroy(surface);
 
 			cairo_matrix_t mat = MATRIXToCairo(style.Matrix);
-			#ifndef NDEBUG
-			cairo_status_t st =
-			#endif
-			cairo_matrix_invert(&mat);
+			cairo_status_t st = cairo_matrix_invert(&mat);
 			assert(st == CAIRO_STATUS_SUCCESS);
 			mat.x0 /= scaleCorrection;
 			mat.y0 /= scaleCorrection;
@@ -592,7 +536,7 @@ cairo_pattern_t* CairoRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, double
 	return pattern;
 }
 
-bool CairoRenderer::cairoPathFromTokens(cairo_t* cr, const std::vector<GeomToken>& tokens, double scaleCorrection, bool skipPaint)
+bool CairoTokenRenderer::cairoPathFromTokens(cairo_t* cr, const std::vector<GeomToken>& tokens, double scaleCorrection, bool skipPaint)
 {
 	cairo_scale(cr, scaleCorrection, scaleCorrection);
 
@@ -706,22 +650,24 @@ bool CairoRenderer::cairoPathFromTokens(cairo_t* cr, const std::vector<GeomToken
 				cairo_set_operator(stroke_cr, CAIRO_OPERATOR_DEST);
 				break;
 			default:
-				::abort();
+				assert(false);
 		}
 	}
 
 	#undef PATH
 
-	if(skipPaint)
-		return empty;
-
-	cairo_fill(cr);
-	cairo_stroke(stroke_cr);
+	if(!skipPaint)
+	{
+		cairo_fill(cr);
+		cairo_stroke(stroke_cr);
+	}
 
 	cairo_pattern_t *stroke_pattern = cairo_pop_group(stroke_cr);
 	cairo_set_source(cr, stroke_pattern);
 	cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-	cairo_paint(cr);
+
+	if(!skipPaint)
+		cairo_paint(cr);
 
 	cairo_pattern_destroy(stroke_pattern);
 	cairo_destroy(stroke_cr);
@@ -729,7 +675,7 @@ bool CairoRenderer::cairoPathFromTokens(cairo_t* cr, const std::vector<GeomToken
 	return empty;
 }
 
-void CairoRenderer::cairoClean(cairo_t* cr) const
+void CairoRenderer::cairoClean(cairo_t* cr)
 {
 	cairo_set_source_rgba(cr, 0, 0, 0, 0);
 	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
@@ -745,38 +691,96 @@ cairo_surface_t* CairoRenderer::allocateSurface()
 	return cairo_image_surface_create_for_data(surfaceBytes, CAIRO_FORMAT_ARGB32, width, height, cairoWidthStride);
 }
 
+void CairoTokenRenderer::executeDraw(cairo_t* cr)
+{
+	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+	cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
+
+	cairoPathFromTokens(cr, tokens, scaleFactor, false);
+}
+
 void CairoRenderer::execute()
 {
 	if(width==0 || height==0)
 	{
-		//Nothing to do, move on
-		uploadNeeded=false;
+		uploadNeeded = false;
 		return;
 	}
+
 	int32_t windowWidth=sys->getRenderThread()->windowWidth;
 	int32_t windowHeight=sys->getRenderThread()->windowHeight;
 	//Discard stuff that it's outside the visible part
-	if(xOffset >= windowWidth || yOffset >= windowHeight)
+	if(xOffset >= windowWidth || yOffset >= windowHeight
+		|| xOffset + width <= 0 || yOffset + height <= 0)
 	{
-		uploadNeeded=false;
+		uploadNeeded = false;
 		return;
 	}
+
+	//TODO:clip on the right and bottom also
+	if(xOffset<0)
+		width+=xOffset;
+	if(yOffset<0)
+		height+=yOffset;
+
 	//Clip the size to the screen borders
-	if((width+xOffset) > windowWidth)
+	if((xOffset>=0) && (width+xOffset) > windowWidth)
 		width=windowWidth-xOffset;
-	if((height+yOffset) > windowHeight)
+	if((yOffset>0) && (height+yOffset) > windowHeight)
 		height=windowHeight-yOffset;
 	cairo_surface_t* cairoSurface=allocateSurface();
 
+	cairo_t* cr=cairo_create(cairoSurface);
+	cairo_surface_destroy(cairoSurface); /* cr has an reference to it */
+	cairoClean(cr);
+
+	//Make sure the rendering starts at 0,0 in surface coordinates
+	//This also guarantees that all the shape fills in width/height pixels
+	//We don't translate for negative offsets as we don't want to see what's in negative coords
+	if(xOffset >= 0)
+		matrix.TranslateX-=xOffset;
+	if(yOffset >= 0)
+		matrix.TranslateY-=yOffset;
+	const cairo_matrix_t& mat=MATRIXToCairo(matrix);
+	cairo_transform(cr, &mat);
+
+	executeDraw(cr);
+
+	cairo_destroy(cr);
+}
+
+bool CairoTokenRenderer::hitTest(const std::vector<GeomToken>& tokens, float scaleFactor, number_t x, number_t y)
+{
+	cairo_surface_t* cairoSurface=cairo_image_surface_create_for_data(NULL, CAIRO_FORMAT_ARGB32, 0, 0, 0);
+
+	cairo_t *cr=cairo_create(cairoSurface);
+	bool empty=cairoPathFromTokens(cr, tokens, scaleFactor, true);
+	bool ret=false;
+	if(!empty)
+	{
+		/* reset the matrix so x and y are not scaled
+		 * by the current cairo transformation
+		 */
+		cairo_identity_matrix(cr);
+		ret=cairo_in_fill(cr, x, y);
+	}
+	cairo_destroy(cr);
+	cairo_surface_destroy(cairoSurface);
+	return ret;
+}
+
+bool CairoTokenRenderer::isOpaque(const std::vector<GeomToken>& tokens, float scaleFactor, number_t x, number_t y)
+{
+	//We render the alpha value of a single pixel, hopefully this is not too slow
+	int32_t cairoWidthStride=cairo_format_stride_for_width(CAIRO_FORMAT_A8, 1);
+	uint8_t* pixelBytes=new uint8_t[cairoWidthStride];
+	cairo_surface_t* cairoSurface=cairo_image_surface_create_for_data(pixelBytes, CAIRO_FORMAT_A8, 1, 1, cairoWidthStride);
+
 	cairo_t* cr=cairo_create(cairoSurface);
 	cairoClean(cr);
 
 	//Make sure the rendering starts at 0,0 in surface coordinates
-	//This also guarantees that all the shape fills in width/height pixels
-	matrix.TranslateX-=xOffset;
-	matrix.TranslateY-=yOffset;
-	const cairo_matrix_t& mat=MATRIXToCairo(matrix);
-	cairo_transform(cr, &mat);
+	cairo_translate(cr, -x, -y);
 
 	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
 	cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
@@ -785,32 +789,37 @@ void CairoRenderer::execute()
 
 	cairo_destroy(cr);
 	cairo_surface_destroy(cairoSurface);
+	//CHECK: the condition is alpha > 0x00 or alpha==0xff
+	return pixelBytes[0]!=0x00;
 }
 
-bool CairoRenderer::hitTest(const std::vector<GeomToken>& tokens, float scaleFactor, number_t x, number_t y)
+uint8_t* CairoRenderer::convertBitmapWithAlphaToCairo(uint8_t* inData, uint32_t width, uint32_t height)
 {
-	cairo_surface_t* cairoSurface=cairo_image_surface_create_for_data(NULL, CAIRO_FORMAT_ARGB32, 0, 0, 0);
+	uint32_t stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
+	uint8_t* outData = new uint8_t[stride * height];
+	uint32_t* inData32 = (uint32_t*)inData;
 
-	cairo_t *cr=cairo_create(cairoSurface);
-	bool empty=cairoPathFromTokens(cr, tokens, scaleFactor, true);
-	bool ret=false;
-	if(!empty)
-		ret=cairo_in_fill(cr, x, y);
-	cairo_destroy(cr);
-	cairo_surface_destroy(cairoSurface);
-	return ret;
+	for(uint32_t i = 0; i < height; i++)
+	{
+		for(uint32_t j = 0; j < width; j++)
+		{
+			uint32_t* outDataPos = (uint32_t*)(outData+i*stride) + j;
+			*outDataPos = BigEndianToHost32( *(inData32+(i*width+j)) );
+		}
+	}
+	return outData;
 }
 
 uint8_t* CairoRenderer::convertBitmapToCairo(uint8_t* inData, uint32_t width, uint32_t height)
 {
-	uint32_t stride = cairo_format_stride_for_width(CAIRO_FORMAT_RGB24, width);
+	uint32_t stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
 	uint8_t* outData = new uint8_t[stride * height];
 	for(uint32_t i = 0; i < height; i++)
 	{
 		for(uint32_t j = 0; j < width; j++)
 		{
 			uint32_t* outDataPos = (uint32_t*)(outData+i*stride) + j;
-			uint32_t pdata = 0;
+			uint32_t pdata = 0xFF;
 			/* the alpha channel is set to zero above */
 			uint8_t* rgbData = ((uint8_t*)&pdata)+1;
 			/* copy the RGB bytes to rgbData */
@@ -821,3 +830,112 @@ uint8_t* CairoRenderer::convertBitmapToCairo(uint8_t* inData, uint32_t width, ui
 	}
 	return outData;
 }
+
+Mutex CairoPangoRenderer::pangoMutex("pangoMutex");
+
+void CairoPangoRenderer::pangoLayoutFromData(PangoLayout* layout, const TextData& tData)
+{
+	PangoFontDescription* desc;
+
+	pango_layout_set_text(layout, tData.text.raw_buf(), -1);
+
+	/* setup alignment */
+	PangoAlignment alignment;
+
+	switch(tData.autoSize)
+	{
+		case TextData::AUTO_SIZE::AS_NONE://TODO:check
+		case TextData::AUTO_SIZE::AS_LEFT:
+		{
+			alignment = PANGO_ALIGN_LEFT;
+			break;
+		}
+		case TextData::AUTO_SIZE::AS_RIGHT:
+		{
+			alignment = PANGO_ALIGN_RIGHT;
+			break;
+		}
+		case TextData::AUTO_SIZE::AS_CENTER:
+		{
+			alignment = PANGO_ALIGN_CENTER;
+			break;
+		}
+	}
+	pango_layout_set_alignment(layout,alignment);
+
+	//In case wordWrap is true, we already have the right width
+	if(tData.wordWrap == true)
+	{
+		pango_layout_set_width(layout,PANGO_SCALE*tData.width);
+		pango_layout_set_wrap(layout,PANGO_WRAP_WORD);//I think this is what Adobe does
+	}
+	//In case autoSize is NONE, we also have the height
+	if(tData.autoSize == TextData::AUTO_SIZE::AS_NONE)
+	{
+		pango_layout_set_width(layout,PANGO_SCALE*tData.width);
+		pango_layout_set_height(layout,PANGO_SCALE*tData.height);//TODO:Not sure what Pango does if the text is too long to fit
+	}
+
+	/* setup font description */
+	desc = pango_font_description_new();
+	pango_font_description_set_family(desc, tData.font.raw_buf());
+	pango_font_description_set_size(desc, PANGO_SCALE*tData.fontSize);
+	pango_layout_set_font_description(layout, desc);
+	pango_font_description_free(desc);
+}
+
+void CairoPangoRenderer::executeDraw(cairo_t* cr)
+{
+	/* TODO: pango is not fully thread-safe,
+	 * but we may be able to use finer grained locking.
+	 */
+	Locker l(pangoMutex);
+	PangoLayout* layout;
+
+	layout = pango_cairo_create_layout(cr);
+	pangoLayoutFromData(layout, textData);
+
+	if(textData.background)
+	{
+		cairo_set_source_rgb (cr, textData.backgroundColor.Red, textData.backgroundColor.Green, textData.backgroundColor.Blue);
+		cairo_paint(cr);
+	}
+	cairo_set_source_rgb (cr, textData.textColor.Red, textData.textColor.Green, textData.textColor.Blue);
+
+	/* draw the text */
+	pango_cairo_show_layout(cr, layout);
+
+	g_object_unref(layout);
+}
+
+bool CairoPangoRenderer::getBounds(const TextData& _textData, uint32_t& w, uint32_t& h, uint32_t& tw, uint32_t& th)
+{
+	//TODO:check locking
+	Locker l(pangoMutex);
+	cairo_surface_t* cairoSurface=cairo_image_surface_create_for_data(NULL, CAIRO_FORMAT_ARGB32, 0, 0, 0);
+	cairo_t *cr=cairo_create(cairoSurface);
+
+	PangoLayout* layout;
+
+	layout = pango_cairo_create_layout(cr);
+	pangoLayoutFromData(layout, _textData);
+
+	PangoRectangle ink_rect, logical_rect;
+	pango_layout_get_pixel_extents(layout,&ink_rect,&logical_rect);//TODO: check the rounding during pango conversion
+
+	g_object_unref(layout);
+	cairo_destroy(cr);
+	cairo_surface_destroy(cairoSurface);
+
+	//This should be safe check precision
+	tw = ink_rect.width;
+	th = ink_rect.height;
+	if(_textData.autoSize != TextData::AUTO_SIZE::AS_NONE)
+	{
+		h = logical_rect.height;
+		if(!_textData.wordWrap)
+			w = logical_rect.width;
+	}
+
+	return (h!=0) && (w!=0);
+}
