@@ -20,12 +20,15 @@
 #ifndef _GRAPHICS_H
 #define _GRAPHICS_H
 
+#define CHUNKSIZE 128
+
 #include "compat.h"
-#include <GL/glew.h>
+#include "lsopengl.h"
 #include <vector>
 #include "swftypes.h"
 #include "threading.h"
 #include <cairo.h>
+#include <pango/pango.h>
 #include "backends/geometry.h"
 
 namespace lightspark
@@ -72,12 +75,6 @@ public:
 		@ret The OpenGL texture id
 	*/
 	GLuint getId() {return texId;}
-	/**
-	  	Initialize the texture from the values stored
-
-		@pre Running inside the RenderThread
-	*/
-	void init();
 	/**
 	  	Initialize the texture using new values
 
@@ -111,12 +108,7 @@ public:
 		@pre Running inside the RenderThread
 	*/
 	void setTexScale(GLuint uniformLocation);
-	/**
-		Load data inside the texture
 
-		@pre Running inside the RenderThread
-	*/
-	void setBGRAData(uint8_t* bgraData, uint32_t w, uint32_t h);
 	void resize(uint32_t width, uint32_t height);
 	/**
 		Request a minimum alignment for width and height
@@ -154,7 +146,7 @@ public:
 	TextureChunk& operator=(const TextureChunk& r);
 	~TextureChunk();
 	bool resizeIfLargeEnough(uint32_t w, uint32_t h);
-	uint32_t getNumberOfChunks() const { return ((width+127)/128)*((height+127)/128); }
+	uint32_t getNumberOfChunks() const { return ((width+CHUNKSIZE-1)/CHUNKSIZE)*((height+CHUNKSIZE-1)/CHUNKSIZE); }
 	bool isValid() const { return chunks; }
 	void makeEmpty();
 	uint32_t width;
@@ -164,10 +156,11 @@ public:
 class CachedSurface
 {
 public:
-	CachedSurface():xOffset(0),yOffset(0){}
+	CachedSurface():xOffset(0),yOffset(0),alpha(1.0){}
 	TextureChunk tex;
-	uint32_t xOffset;
-	uint32_t yOffset;
+	int32_t xOffset;
+	int32_t yOffset;
+	float alpha;
 };
 
 class ITextureUploadable
@@ -218,50 +211,26 @@ protected:
 	   The minimal y coordinate for all the points being drawn, in local coordinates
 	*/
 	int32_t yOffset;
+	float alpha;
 	int32_t width;
 	int32_t height;
 	/*
 	   A pointer to a memory buffer where cairo will draw
 	*/
 	uint8_t* surfaceBytes;
-	/*
-	   The tokens to be drawn
-	*/
-	const std::vector<GeomToken> tokens;
 	/*
 	   The scale to be applied in both the x and y axis.
 	   Useful to adapt points defined in pixels and twips (1/20 of pixel)
 	*/
 	const float scaleFactor;
 	bool uploadNeeded;
-	//Internal helpers
 	static cairo_matrix_t MATRIXToCairo(const MATRIX& matrix);
-	static cairo_pattern_t* FILLSTYLEToCairo(const FILLSTYLE& style, double scaleCorrection);
-	static bool cairoPathFromTokens(cairo_t* cr, const std::vector<GeomToken>& tokens, double scaleCorrection, bool skipFill);
-	static void quadraticBezier(cairo_t* cr, double control_x, double control_y, double end_x, double end_y);
-	void cairoClean(cairo_t* cr) const;
+	static void cairoClean(cairo_t* cr);
 	cairo_surface_t* allocateSurface();
+	virtual void executeDraw(cairo_t* cr)=0;
 public:
-	/*
-	   CairoRenderer constructor
-
-	   @param _o Used in a code path that must be reviewed. Please leave it here.
-	   @param _t GL surface where the final drawing will be uploaded
-	   @param _g The tokens to be drawn. This is copied internally.
-	   @param _m The whole transformation matrix
-	   @param _s The scale factor to be applied in both the x and y axis
-	*/
-	CairoRenderer(ASObject* _o, CachedSurface& _t, const std::vector<GeomToken>& _g, const MATRIX& _m,
-			int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s);
-	/*
-	   Hit testing helper. Uses cairo to find if a point in inside the shape
-
-	   @param tokens The tokens of the shape being tested
-	   @param scaleFactor The scale factor to be applied
-	   @param x The X in local coordinates
-	   @param y The Y in local coordinates
-	*/
-	static bool hitTest(const std::vector<GeomToken>& tokens, float scaleFactor, number_t x, number_t y);
+	CairoRenderer(ASObject* _o, CachedSurface& _t, const MATRIX& _m,
+				int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s, float _a);
 	//ITextureUploadable interface
 	void sizeNeeded(uint32_t& w, uint32_t& h) const;
 	void upload(uint8_t* data, uint32_t w, uint32_t h) const;
@@ -271,12 +240,110 @@ public:
 	void threadAbort();
 	void jobFence();
 	void execute();
-
 	/*
 	 * Converts data (which is in RGB format) to the format internally used by cairo.
 	 * This function new[]'s the returned value, which has to be freed by the caller.
 	 */
 	static uint8_t* convertBitmapToCairo(uint8_t* data, uint32_t width, uint32_t height);
+	/*
+	 * Converts data (which is in ARGB format) to the format internally used by cairo.
+	 * This function new[]'s the returned value, which has to be freed by the caller.
+	 */
+	static uint8_t* convertBitmapWithAlphaToCairo(uint8_t* inData, uint32_t width, uint32_t height);
+};
+
+class CairoTokenRenderer : public CairoRenderer
+{
+private:
+	static cairo_pattern_t* FILLSTYLEToCairo(const FILLSTYLE& style, double scaleCorrection);
+	static bool cairoPathFromTokens(cairo_t* cr, const std::vector<GeomToken>& tokens, double scaleCorrection, bool skipFill);
+	static void quadraticBezier(cairo_t* cr, double control_x, double control_y, double end_x, double end_y);
+	/*
+	   The tokens to be drawn
+	*/
+	const std::vector<GeomToken> tokens;
+	/*
+	 * This is run by CairoRenderer::execute()
+	 */
+	void executeDraw(cairo_t* cr);
+public:
+	/*
+	   CairoTokenRenderer constructor
+
+	   @param _o Owner of the surface _t. See comments on 'owner' member.
+	   @param _t GL surface where the final drawing will be uploaded
+	   @param _g The tokens to be drawn. This is copied internally.
+	   @param _m The whole transformation matrix
+	   @param _s The scale factor to be applied in both the x and y axis
+	   @param _a The alpha factor to be applied
+	*/
+	CairoTokenRenderer(ASObject* _o, CachedSurface& _t, const std::vector<GeomToken>& _g, const MATRIX& _m,
+					   int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s, float _a)
+		: CairoRenderer(_o,_t,_m,_x,_y,_w,_h,_s,_a), tokens(_g) {}
+	/*
+	   Hit testing helper. Uses cairo to find if a point in inside the shape
+
+	   @param tokens The tokens of the shape being tested
+	   @param scaleFactor The scale factor to be applied
+	   @param x The X in local coordinates
+	   @param y The Y in local coordinates
+	*/
+	static bool hitTest(const std::vector<GeomToken>& tokens, float scaleFactor, number_t x, number_t y);
+	/*
+	   Opacity test helper. Uses cairo to render a single pixel and see if it's opaque or not
+
+	   @param tokens The tokens of the shape being tested
+	   @param scaleFactor The scale factor to be applied
+	   @param x The X in local coordinates
+	   @param y The Y in local coordinates
+	*/
+	static bool isOpaque(const std::vector<GeomToken>& tokens, float scaleFactor, number_t x, number_t y);
+};
+
+class TextData
+{
+public:
+	/* the default values are from the spec for flash.text.TextField and flash.text.TextFormat */
+	TextData() : width(100), height(100), textWidth(0), textHeight(0), background(false), backgroundColor(0xFFFFFF),
+		border(false), borderColor(0x000000), multiline(false), textColor(0x000000),
+		wordWrap(false), autoSize(AS_NONE), fontSize(12), font("Times New Roman") {}
+	uint32_t width;
+	uint32_t height;
+	uint32_t textWidth;
+	uint32_t textHeight;
+	tiny_string text;
+	bool background;
+	RGB backgroundColor;
+	bool border;
+	RGB borderColor;
+	bool multiline;
+	RGB textColor;
+	bool wordWrap;
+	enum AUTO_SIZE {AS_NONE = 0, AS_LEFT, AS_RIGHT, AS_CENTER };
+	AUTO_SIZE autoSize;
+	uint32_t fontSize;
+	tiny_string font;
+};
+
+class CairoPangoRenderer : public CairoRenderer
+{
+	static Mutex pangoMutex;
+	/*
+	 * This is run by CairoRenderer::execute()
+	 */
+	void executeDraw(cairo_t* cr);
+	TextData textData;
+	static void pangoLayoutFromData(PangoLayout* layout, const TextData& tData);
+public:
+	CairoPangoRenderer(ASObject* _o, CachedSurface& _t, const TextData& _textData, const MATRIX& _m,
+			int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s, float _a)
+		: CairoRenderer(_o,_t,_m,_x,_y,_w,_h,_s,_a), textData(_textData) {}
+	/**
+		Helper. Uses Pango to find the size of the textdata
+		@param _texttData The textData being tested
+		@param w,h,tw,th are the (text)width and (text)height of the textData.
+	*/
+	static bool getBounds(const TextData& _textData, uint32_t& w, uint32_t& h, uint32_t& tw, uint32_t& th);
 };
 
 };
