@@ -19,6 +19,7 @@
 **************************************************************************/
 
 #include "swf.h"
+#include "config.h"
 #include "netutils.h"
 #include "rtmputils.h"
 #include "compat.h"
@@ -44,6 +45,31 @@ DownloadManager::DownloadManager()
 	//== Lock initialized
 }
 
+/**
+ * \brief Download manager destructor
+ */
+DownloadManager::~DownloadManager()
+{
+	//== Destroy lock
+	sem_destroy(&mutex);
+}
+
+/**
+ * \brief Stops all the currently running downloaders
+ */
+void DownloadManager::stopAll()
+{
+	sem_wait(&mutex);
+	//-- Lock acquired
+
+	std::list<Downloader*>::iterator it=downloaders.begin();
+	for(;it!=downloaders.end();it++)
+		(*it)->stop();
+
+	//++ Release lock
+	sem_post(&mutex);
+}
+
 /**
  * \brief Destroyes all the pending downloads, must be called in the destructor of each derived class
  *
@@ -58,9 +84,11 @@ void DownloadManager::cleanUp()
 	sem_wait(&mutex);
 	//-- Lock acquired
 
-	for(std::list<Downloader*>::iterator it=downloaders.begin(); it!=downloaders.end(); ++it)
+	while(!downloaders.empty())
 	{
-		(*it)->stop();
+		std::list<Downloader*>::iterator it=downloaders.begin();
+		//cleanUp should only happen after stopAll has been called
+		assert((*it)->hasFinished());
 
 		//++ Release lock
 		sem_post(&mutex);
@@ -147,20 +175,6 @@ StandaloneDownloadManager::~StandaloneDownloadManager()
 	cleanUp();
 }
 
-/**
- * \brief Create a Downloader for an URL.
- *
- * Returns a pointer to a newly created \c Downloader for the given URL.
- * \param[in] url The URL (as a \c tiny_string) the \c Downloader is requested for
- * \param[in] cached Whether or not to disk-cache the download (default=false)
- * \return A pointer to a newly created \c Downloader for the given URL.
- * \see DownloadManager::destroy()
- */
-Downloader* StandaloneDownloadManager::download(const tiny_string& url, bool cached, LoaderInfo* owner)
-{
-	return download(sys->getOrigin().goToURL(url), cached, owner);
-}
-
 /**
  * \brief Create a Downloader for an URL.
  *
@@ -170,27 +184,26 @@ Downloader* StandaloneDownloadManager::download(const tiny_string& url, bool cac
  * \return A pointer to a newly created \c Downloader for the given URL.
  * \see DownloadManager::destroy()
  */
-Downloader* StandaloneDownloadManager::download(const URLInfo& url, bool cached, LoaderInfo* owner)
+Downloader* StandaloneDownloadManager::download(const URLInfo& url, bool cached, ILoadable* owner)
 {
-	LOG(LOG_NO_INFO, _("NET: STANDALONE: DownloadManager::download '") << url.getParsedURL()
+	LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager::download '") << url.getParsedURL()
 			<< "'" << (cached ? _(" - cached") : ""));
 	ThreadedDownloader* downloader;
 	if(url.getProtocol() == "file")
 	{
-		LOG(LOG_NO_INFO, _("NET: STANDALONE: DownloadManager: local file"));
-		downloader=new LocalDownloader(url.getPath(), cached);
+		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: local file"));
+		downloader=new LocalDownloader(url.getPath(), cached, owner);
 	}
 	else if(url.getProtocol() == "rtmpe")
 	{
-		LOG(LOG_NO_INFO, _("NET: STANDALONE: DownloadManager: RTMPE stream"));
-		downloader=new RTMPDownloader(url.getParsedURL(), url.getStream());
+		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: RTMPE stream"));
+		downloader=new RTMPDownloader(url.getParsedURL(), url.getStream(), owner);
 	}
 	else
 	{
-		LOG(LOG_NO_INFO, _("NET: STANDALONE: DownloadManager: remote file"));
-		downloader=new CurlDownloader(url.getParsedURL(), cached);
+		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: remote file"));
+		downloader=new CurlDownloader(url.getParsedURL(), cached, owner);
 	}
-	downloader->setOwner(owner);
 	downloader->enableFencingWaiting();
 	addDownloader(downloader);
 	sys->addJob(downloader);
@@ -206,23 +219,22 @@ Downloader* StandaloneDownloadManager::download(const URLInfo& url, bool cached,
  * \return A pointer to a newly created \c Downloader for the given URL.
  * \see DownloadManager::destroy()
  */
-Downloader* StandaloneDownloadManager::downloadWithData(const URLInfo& url, const std::vector<uint8_t>& data, LoaderInfo* owner)
+Downloader* StandaloneDownloadManager::downloadWithData(const URLInfo& url, const std::vector<uint8_t>& data, ILoadable* owner)
 {
-	LOG(LOG_NO_INFO, _("NET: STANDALONE: DownloadManager::downloadWithData '") << url.getParsedURL());
+	LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager::downloadWithData '") << url.getParsedURL());
 	ThreadedDownloader* downloader;
 	if(url.getProtocol() == "file")
 	{
-		LOG(LOG_NO_INFO, _("NET: STANDALONE: DownloadManager: local file - Ignoring data field"));
-		downloader=new LocalDownloader(url.getPath(), false);
+		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: local file - Ignoring data field"));
+		downloader=new LocalDownloader(url.getPath(), false, owner);
 	}
 	else if(url.getProtocol() == "rtmpe")
 		throw RunTimeException("RTMPE does not support additional data");
 	else
 	{
-		LOG(LOG_NO_INFO, _("NET: STANDALONE: DownloadManager: remote file"));
-		downloader=new CurlDownloader(url.getParsedURL(), data);
+		LOG(LOG_INFO, _("NET: STANDALONE: DownloadManager: remote file"));
+		downloader=new CurlDownloader(url.getParsedURL(), data, owner);
 	}
-	downloader->setOwner(owner);
 	downloader->enableFencingWaiting();
 	addDownloader(downloader);
 	sys->addJob(downloader);
@@ -236,7 +248,7 @@ Downloader* StandaloneDownloadManager::downloadWithData(const URLInfo& url, cons
  * \param[in] _url The URL for the Downloader.
  * \param[in] _cached Whether or not to cache this download.
  */
-Downloader::Downloader(const tiny_string& _url, bool _cached):
+Downloader::Downloader(const tiny_string& _url, bool _cached, ILoadable* o):
 	cacheHasOpened(false),hasTerminated(false),                   //LOCKING
 	waitingForCache(false),waitingForData(false),waitingForTermination(false), //STATUS
 	forceStop(true),failed(false),finished(false),                //FLAGS
@@ -245,7 +257,7 @@ Downloader::Downloader(const tiny_string& _url, bool _cached):
 	cached(_cached),cachePos(0),cacheSize(0),keepCache(false),    //CACHING
 	length(0),receivedLength(0),                                  //DOWNLOADED DATA
 	redirected(false),requestStatus(0),                           //HTTP REDIR, STATUS & HEADERS
-	owner(NULL)                                                   //PROGRESS
+	owner(o)                                                   //PROGRESS
 {
 	sem_init(&mutex,0,1);
 	//== Lock initialized 
@@ -268,7 +280,7 @@ Downloader::Downloader(const tiny_string& _url, bool _cached):
  * \param[in] _url The URL for the Downloader.
  * \param[in] data Additional data to send to the host
  */
-Downloader::Downloader(const tiny_string& _url, const std::vector<uint8_t>& _data):
+Downloader::Downloader(const tiny_string& _url, const std::vector<uint8_t>& _data, ILoadable* o):
 	cacheHasOpened(false),hasTerminated(false),                   //LOCKING
 	waitingForCache(false),waitingForData(false),waitingForTermination(false), //STATUS
 	forceStop(true),failed(false),finished(false),                //FLAGS
@@ -277,7 +289,7 @@ Downloader::Downloader(const tiny_string& _url, const std::vector<uint8_t>& _dat
 	cached(false),cachePos(0),cacheSize(0),keepCache(false),      //CACHING
 	length(0),receivedLength(0),                                  //DOWNLOADED DATA
 	redirected(false),requestStatus(0),data(_data),               //HTTP REDIR, STATUS & HEADERS
-	owner(NULL)                                                   //PROGRESS
+	owner(o)                                                   //PROGRESS
 {
 	sem_init(&mutex,0,1);
 	//== Lock initialized 
@@ -489,6 +501,21 @@ Downloader::pos_type Downloader::seekpos(pos_type pos, std::ios_base::openmode m
 	sem_wait(&mutex);
 	//-- Lock acquired
 	syncBuffers();
+
+	// read from stream until we have enough data
+	uint32_t tmplen = receivedLength;
+	while (!hasTerminated && pos > receivedLength) 
+	{
+		//++ Release lock
+		sem_post(&mutex);
+		waitForData();
+		sem_wait(&mutex);
+		//-- Lock acquired
+		syncBuffers();
+		if (tmplen == receivedLength)
+			break; // no new data read
+		tmplen = receivedLength;
+	}
 	
 	if(cached)
 	{
@@ -553,19 +580,43 @@ Downloader::pos_type Downloader::seekpos(pos_type pos, std::ios_base::openmode m
  * \brief Called by the streambuf API
  *
  * Called by the streambuf API to seek to a relative position
- * The only supported case is offset==0.
  * Waits for the mutex at start and releases the mutex when finished.
  */
 Downloader::pos_type Downloader::seekoff(off_type off, std::ios_base::seekdir dir, std::ios_base::openmode mode)
 {
 	assert_and_throw(mode==std::ios_base::in);
-	assert_and_throw(off==0);
 	assert_and_throw(buffer != NULL);
 
+	if (off != 0)
+	{
+		switch (dir)
+		{
+			case std::ios_base::beg:
+				seekpos(off,mode);
+				break;
+			case std::ios_base::cur:
+			{
+				sem_wait(&mutex);
+				//-- Lock acquired
+				pos_type tmp = getOffset();
+				//++ Release lock
+				sem_post(&mutex);
+				seekpos(tmp+off,mode);
+				break;
+			}
+			case std::ios_base::end:
+				waitForTermination();
+				if (finished)
+					seekpos(length+off,mode);
+				break;
+			default:
+				break;
+		}
+	}
+
 	sem_wait(&mutex);
 	//-- Lock acquired
 
-	//Nothing special to do here, we only support offset==0 seeks
 	pos_type ret=getOffset();
 
 	//++ Release lock
@@ -669,7 +720,6 @@ void Downloader::allocateBuffer(size_t size)
 	//Create buffer
 	if(buffer == NULL)
 	{
-
 		buffer = (uint8_t*) calloc(size, sizeof(uint8_t));
 		stableBuffer = buffer;
 		setg((char*)buffer,(char*)buffer,(char*)buffer);
@@ -678,7 +728,7 @@ void Downloader::allocateBuffer(size_t size)
 	else
 	{
 		assert(!cached);
-		intptr_t curLen = (intptr_t) (egptr()-eback());
+		intptr_t curLen = receivedLength;
 		//We have to extend the buffer, so create a new one
 		if(stableBuffer!=buffer)
 		{
@@ -774,7 +824,7 @@ void Downloader::openExistingCache(tiny_string filename)
 		sem_wait(&mutex);
 		//-- Lock acquired
 
-		LOG(LOG_NO_INFO, _("NET: Downloading to cache file: ") << cacheFilename);
+		LOG(LOG_INFO, _("NET: Downloading to cache file: ") << cacheFilename);
 
 		//++ Signal cacheOpened
 		sem_post(&cacheOpened);
@@ -819,7 +869,7 @@ void Downloader::setLength(uint32_t _length)
 	else
 	{
 		if(buffer == NULL)
-			LOG(LOG_NO_INFO, _("NET: Downloading to memory"));
+			LOG(LOG_INFO, _("NET: Downloading to memory"));
 		//++ Release lock
 		sem_post(&mutex);
 
@@ -927,7 +977,7 @@ void Downloader::parseHeader(std::string header, bool _setLength)
 	sem_wait(&mutex);
 	//-- Lock acquired
 
-	if(header.substr(0, 9) == "HTTP/1.1 " || header.substr(0, 9) == "HTTP/1.0") 
+	if(header.substr(0, 9) == "HTTP/1.1 " || header.substr(0, 9) == "HTTP/1.0 ") 
 	{
 		std::string status = header.substr(9, 3);
 		requestStatus = atoi(status.c_str());
@@ -939,8 +989,8 @@ void Downloader::parseHeader(std::string header, bool _setLength)
 		{
 			setFailed();
 		}
-		else if(getRequestStatus()/100 == 3); //HTTP redirect
-		else if(getRequestStatus()/100 == 2); //HTTP OK
+		else if(getRequestStatus()/100 == 3) {;} //HTTP redirect
+		else if(getRequestStatus()/100 == 2) {;} //HTTP OK
 	}
 	else
 	{
@@ -963,7 +1013,7 @@ void Downloader::parseHeader(std::string header, bool _setLength)
 			//Set the new real URL when we are being redirected
 			if(getRequestStatus()/100 == 3 && headerName == "location")
 			{
-				LOG(LOG_NO_INFO, _("NET: redirect detected"));
+				LOG(LOG_INFO, _("NET: redirect detected"));
 				setRedirected(URLInfo(url).goToURL(tiny_string(headerValue)).getParsedURL());
 			}
 			if(headerName == "content-length")
@@ -1136,8 +1186,8 @@ void ThreadedDownloader::waitFencing()
  * \param[in] _url The URL for the Downloader.
  * \param[in] _cached Whether or not to cache this download.
  */
-ThreadedDownloader::ThreadedDownloader(const tiny_string& url, bool cached):
-	Downloader(url, cached),fenceState(false)
+ThreadedDownloader::ThreadedDownloader(const tiny_string& url, bool cached, ILoadable* o):
+	Downloader(url, cached, o),fenceState(false)
 {
 }
 
@@ -1148,8 +1198,8 @@ ThreadedDownloader::ThreadedDownloader(const tiny_string& url, bool cached):
  * \param[in] _url The URL for the Downloader.
  * \param[in] data Additional data to send to the host
  */
-ThreadedDownloader::ThreadedDownloader(const tiny_string& url, const std::vector<uint8_t>& data):
-	Downloader(url, data),fenceState(false)
+ThreadedDownloader::ThreadedDownloader(const tiny_string& url, const std::vector<uint8_t>& data, ILoadable* o):
+	Downloader(url, data, o),fenceState(false)
 {
 }
 
@@ -1171,7 +1221,7 @@ ThreadedDownloader::~ThreadedDownloader()
  * \param[in] _url The URL for the Downloader.
  * \param[in] _cached Whether or not to cache this download.
  */
-CurlDownloader::CurlDownloader(const tiny_string& _url, bool _cached):ThreadedDownloader(_url, _cached)
+CurlDownloader::CurlDownloader(const tiny_string& _url, bool _cached, ILoadable* o):ThreadedDownloader(_url, _cached, o)
 {
 }
 
@@ -1181,7 +1231,7 @@ CurlDownloader::CurlDownloader(const tiny_string& _url, bool _cached):ThreadedDo
  * \param[in] _url The URL for the Downloader.
  * \param[in] data Additional data to send to the host
  */
-CurlDownloader::CurlDownloader(const tiny_string& _url, const std::vector<uint8_t>& _data):ThreadedDownloader(_url, _data)
+CurlDownloader::CurlDownloader(const tiny_string& _url, const std::vector<uint8_t>& _data, ILoadable* o):ThreadedDownloader(_url, _data, o)
 {
 }
 
@@ -1205,7 +1255,7 @@ void CurlDownloader::execute()
 		setFailed();
 		return;
 	}
-	LOG(LOG_NO_INFO, _("NET: CurlDownloader::execute: reading remote file: ") << url.raw_buf());
+	LOG(LOG_INFO, _("NET: CurlDownloader::execute: reading remote file: ") << url.raw_buf());
 #ifdef ENABLE_CURL
 	CURL *curl;
 	CURLcode res;
@@ -1316,7 +1366,7 @@ size_t CurlDownloader::write_header(void *buffer, size_t size, size_t nmemb, voi
  * \param[in] _url The URL for the Downloader.
  * \param[in] _cached Whether or not to cache this download.
  */
-LocalDownloader::LocalDownloader(const tiny_string& _url, bool _cached):ThreadedDownloader(_url, _cached)
+LocalDownloader::LocalDownloader(const tiny_string& _url, bool _cached, ILoadable* o):ThreadedDownloader(_url, _cached, o)
 {
 }
 
@@ -1345,7 +1395,7 @@ void LocalDownloader::execute()
 	}
 	else
 	{
-		LOG(LOG_NO_INFO, _("NET: LocalDownloader::execute: reading local file: ") << url.raw_buf());
+		LOG(LOG_INFO, _("NET: LocalDownloader::execute: reading local file: ") << url.raw_buf());
 		//If the caching is selected, we override the normal behaviour and use the local file as the cache file
 		//This prevents unneeded copying of the file's data
 		if(isCached())
