@@ -20,17 +20,9 @@
 #include "scripting/abc.h"
 #include "parsing/textfile.h"
 #include "rendering.h"
+#include "glmatrices.h"
 #include "compat.h"
 #include <sstream>
-//#include "swf.h"
-
-#include <SDL.h>
-
-#include <GL/glew.h>
-#ifndef WIN32
-#include <GL/glx.h>
-#include <fontconfig/fontconfig.h>
-#endif
 
 //The interpretation of texture data change with the endianness
 #if __BYTE_ORDER == __BIG_ENDIAN
@@ -59,9 +51,9 @@ RenderThread::RenderThread(SystemState* s):
 	pixelBufferWidth(0),pixelBufferHeight(0),prevUploadJob(NULL),mutexLargeTexture("Large texture"),largeTextureSize(0),
 	renderNeeded(false),uploadNeeded(false),resizeNeeded(false),newTextureNeeded(false),newWidth(0),newHeight(0),scaleX(1),scaleY(1),
 	offsetX(0),offsetY(0),tempBufferAcquired(false),frameCount(0),secsCount(0),mutexUploadJobs("Upload jobs"),initialized(0),
-	tempTex(false),hasNPOTTextures(false)
+	tempTex(false),hasNPOTTextures(false),cairoTextureContext(NULL)
 {
-	LOG(LOG_NO_INFO,_("RenderThread this=") << this);
+	LOG(LOG_INFO,_("RenderThread this=") << this);
 	
 	m_sys=s;
 	sem_init(&event,0,0);
@@ -69,43 +61,16 @@ RenderThread::RenderThread(SystemState* s):
 #ifdef WIN32
 	fontPath = "TimesNewRoman.ttf";
 #else
-	FcPattern *pat, *match;
-	FcResult result = FcResultMatch;
-	char *font = NULL;
-
-	pat = FcPatternCreate();
-	FcPatternAddString(pat, FC_FAMILY, (const FcChar8 *)"Serif");
-	FcConfigSubstitute(NULL, pat, FcMatchPattern);
-	FcDefaultSubstitute(pat);
-	match = FcFontMatch(NULL, pat, &result);
-	FcPatternDestroy(pat);
-
-	if (result != FcResultMatch)
-	{
-		LOG(LOG_ERROR,_("Unable to find suitable Serif font"));
-		throw RunTimeException("Unable to find Serif font");
-	}
-
-	FcPatternGetString(match, FC_FILE, 0, (FcChar8 **) &font);
-	fontPath = font;
-	FcPatternDestroy(match);
-	LOG(LOG_NO_INFO, _("Font File is ") << fontPath);
+	fontPath = "Serif";
 #endif
 	time_s = compat_get_current_time_ms();
 }
 
-void RenderThread::start(ENGINE e,void* params)
+void RenderThread::start(const EngineData* data)
 {
 	status=STARTED;
-	if(e==SDL)
-		pthread_create(&t,NULL,(thread_worker)sdl_worker,this);
-#ifdef COMPILE_PLUGIN
-	else if(e==GTKPLUG)
-	{
-		npapi_params=(NPAPI_params*)params;
-		pthread_create(&t,NULL,(thread_worker)gtkplug_worker,this);
-	}
-#endif
+	engineData=data;
+	pthread_create(&t,NULL,(thread_worker)worker,this);
 }
 
 void RenderThread::stop()
@@ -117,30 +82,38 @@ RenderThread::~RenderThread()
 {
 	wait();
 	sem_destroy(&event);
-	LOG(LOG_NO_INFO,_("~RenderThread this=") << this);
+	LOG(LOG_INFO,_("~RenderThread this=") << this);
 }
 
-void RenderThread::glAcquireTempBuffer(number_t xmin, number_t xmax, number_t ymin, number_t ymax)
+/*void RenderThread::acquireTempBuffer(number_t xmin, number_t xmax, number_t ymin, number_t ymax)
 {
 	::abort();
+	GLfloat vertex_coords[8];
+	static GLfloat color_coords[16];
 	assert(tempBufferAcquired==false);
 	tempBufferAcquired=true;
 
 	glBindFramebuffer(GL_FRAMEBUFFER, fboId);
 	glDrawBuffer(GL_COLOR_ATTACHMENT0);
 	
-	glColor4f(0,0,0,0); //No output is fairly ok to clear
-	glBegin(GL_QUADS);
-		glVertex2f(xmin,ymin);
-		glVertex2f(xmax,ymin);
-		glVertex2f(xmax,ymax);
-		glVertex2f(xmin,ymax);
-	glEnd();
+	vertex_coords[0] = xmin;vertex_coords[1] = ymin;
+	vertex_coords[2] = xmax;vertex_coords[3] = ymin;
+	vertex_coords[4] = xmin;vertex_coords[5] = ymax;
+	vertex_coords[6] = xmax;vertex_coords[7] = ymax;
+
+	glVertexAttribPointer(VERTEX_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, vertex_coords);
+	glVertexAttribPointer(COLOR_ATTRIB, 4, GL_FLOAT, GL_FALSE, 0, color_coords);
+	glEnableVertexAttribArray(VERTEX_ATTRIB);
+	glEnableVertexAttribArray(COLOR_ATTRIB);
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+	glDisableVertexAttribArray(VERTEX_ATTRIB);
+	glDisableVertexAttribArray(COLOR_ATTRIB);
 }
 
-void RenderThread::glBlitTempBuffer(number_t xmin, number_t xmax, number_t ymin, number_t ymax)
+void RenderThread::blitTempBuffer(number_t xmin, number_t xmax, number_t ymin, number_t ymax)
 {
 	assert(tempBufferAcquired==true);
+	GLfloat vertex_coords[8];
 	tempBufferAcquired=false;
 
 	//Use the blittler program to blit only the used buffer
@@ -149,14 +122,19 @@ void RenderThread::glBlitTempBuffer(number_t xmin, number_t xmax, number_t ymin,
 	glDrawBuffer(GL_BACK);
 
 	rt->tempTex.bind();
-	glBegin(GL_QUADS);
-		glVertex2f(xmin,ymin);
-		glVertex2f(xmax,ymin);
-		glVertex2f(xmax,ymax);
-		glVertex2f(xmin,ymax);
-	glEnd();
+
+	vertex_coords[0] = xmin;vertex_coords[1] = ymin;
+	vertex_coords[2] = xmax;vertex_coords[3] = ymin;
+	vertex_coords[4] = xmin;vertex_coords[5] = ymax;
+	vertex_coords[6] = xmax;vertex_coords[7] = ymax;
+
+	glVertexAttribPointer(VERTEX_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, vertex_coords);
+	glEnableVertexAttribArray(VERTEX_ATTRIB);
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+	glDisableVertexAttribArray(VERTEX_ATTRIB);
+
 	glUseProgram(gpu_program);
-}
+}*/
 
 void RenderThread::handleNewTexture()
 {
@@ -170,6 +148,10 @@ void RenderThread::handleNewTexture()
 	newTextureNeeded=false;
 }
 
+#ifdef ENABLE_GLES2
+uint8_t* pixelBuf = 0;
+#endif
+
 void RenderThread::finalizeUpload()
 {
 	ITextureUploadable* u=prevUploadJob;
@@ -177,8 +159,12 @@ void RenderThread::finalizeUpload()
 	u->sizeNeeded(w,h);
 	const TextureChunk& tex=u->getTexture();
 	glBindBuffer(GL_PIXEL_UNPACK_BUFFER, pixelBuffers[currentPixelBuffer]);
+#ifndef ENABLE_GLES2
 	//Copy content of the pbo to the texture, currentPixelBufferOffset is the offset in the pbo
 	loadChunkBGRA(tex, w, h, (uint8_t*)currentPixelBufferOffset);
+#else
+	loadChunkBGRA(tex, w, h, pixelBuf);
+#endif
 	glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
 	u->uploadFence();
 	prevUploadJob=NULL;
@@ -193,8 +179,8 @@ void RenderThread::handleUpload()
 	if(w>pixelBufferWidth || h>pixelBufferHeight)
 		resizePixelBuffers(w,h);
 	//Increment and wrap current buffer index
+#ifndef ENABLE_GLES2
 	unsigned int nextBuffer = (currentPixelBuffer + 1)%2;
-
 	glBindBuffer(GL_PIXEL_UNPACK_BUFFER, pixelBuffers[nextBuffer]);
 	uint8_t* buf=(uint8_t*)glMapBuffer(GL_PIXEL_UNPACK_BUFFER,GL_WRITE_ONLY);
 	uint8_t* alignedBuf=(uint8_t*)(uintptr_t((buf+15))&(~0xfL));
@@ -206,25 +192,43 @@ void RenderThread::handleUpload()
 
 	currentPixelBufferOffset=alignedBuf-buf;
 	currentPixelBuffer=nextBuffer;
-
+#else
+	//TODO See if a more elegant way of handling the non-PBO case can be found.
+	//for now, each frame is uploaded one at a time synchronously to the server
+	if(!pixelBuf)
+		if(posix_memalign((void **)&pixelBuf, 16, w*h*4)) {
+			LOG(LOG_ERROR, "posix_memalign could not allocate memory");
+			return;
+		}
+	u->upload(pixelBuf, w, h);
+#endif
 	//Get the texture to be sure it's allocated when the upload comes
 	u->getTexture();
 	prevUploadJob=u;
 }
 
-#ifdef COMPILE_PLUGIN
-void* RenderThread::gtkplug_worker(RenderThread* th)
+void RenderThread::SizeAllocateCallback(GtkWidget* widget, GdkRectangle* allocation, gpointer data)
+{
+	RenderThread* th=static_cast<RenderThread*>(data);
+	th->requestResize(allocation->width, allocation->height);
+}
+
+void* RenderThread::worker(RenderThread* th)
 {
 	sys=th->m_sys;
 	rt=th;
-	NPAPI_params* p=th->npapi_params;
+	const EngineData* e=th->engineData;
 	SemaphoreLighter lighter(th->initialized);
 
-	th->windowWidth=p->width;
-	th->windowHeight=p->height;
+	//Get information about changes in the available space
+	g_signal_connect(e->container,"size-allocate",G_CALLBACK(SizeAllocateCallback),th);
+
+	th->windowWidth=e->width;
+	th->windowHeight=e->height;
 	
 	Display* d=XOpenDisplay(NULL);
 
+#ifndef ENABLE_GLES2
 	int a,b;
 	Bool glx_present=glXQueryVersion(d,&a,&b);
 	if(!glx_present)
@@ -232,13 +236,13 @@ void* RenderThread::gtkplug_worker(RenderThread* th)
 		LOG(LOG_ERROR,_("glX not present"));
 		return NULL;
 	}
-	int attrib[10]={GLX_BUFFER_SIZE,24,GLX_DOUBLEBUFFER,True,None};
+	int attrib[10]={GLX_RED_SIZE, 8, GLX_GREEN_SIZE, 8, GLX_BLUE_SIZE, 8, GLX_DOUBLEBUFFER, True, None};
 	GLXFBConfig* fb=glXChooseFBConfig(d, 0, attrib, &a);
 	if(!fb)
 	{
-		attrib[2]=None;
-		fb=glXChooseFBConfig(d, 0, attrib, &a);
+		attrib[6]=None;
 		LOG(LOG_ERROR,_("Falling back to no double buffering"));
+		fb=glXChooseFBConfig(d, 0, attrib, &a);
 	}
 	if(!fb)
 	{
@@ -250,7 +254,7 @@ void* RenderThread::gtkplug_worker(RenderThread* th)
 	{
 		int id;
 		glXGetFBConfigAttrib(d,fb[i],GLX_VISUAL_ID,&id);
-		if(id==(int)p->visual)
+		if(id==(int)e->visual)
 			break;
 	}
 	if(i==a)
@@ -264,10 +268,84 @@ void* RenderThread::gtkplug_worker(RenderThread* th)
 	XFree(fb);
 
 	th->mContext = glXCreateNewContext(d,th->mFBConfig,GLX_RGBA_TYPE ,NULL,1);
-	GLXWindow glxWin=p->window;
+	GLXWindow glxWin=e->window;
 	glXMakeCurrent(d, glxWin,th->mContext);
 	if(!glXIsDirect(d,th->mContext))
 		cout << "Indirect!!" << endl;
+#else
+	int a;
+	eglBindAPI(EGL_OPENGL_ES_API);
+	EGLDisplay ed = EGL_NO_DISPLAY;
+	ed = eglGetDisplay(d);
+
+	if (ed == EGL_NO_DISPLAY) {
+		LOG(LOG_ERROR, _("EGL not present"));
+		return NULL;
+	}
+
+	EGLint major, minor;
+	if (eglInitialize(ed, &major, &minor) == EGL_FALSE) {
+		LOG(LOG_ERROR, _("EGL initialization failed"));
+		return NULL;
+	}
+
+	LOG(LOG_INFO, _("EGL version: ") << eglQueryString(ed, EGL_VERSION));
+
+	EGLint config_attribs[] = {
+				EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
+				EGL_RED_SIZE, 8,
+				EGL_GREEN_SIZE, 8,
+				EGL_BLUE_SIZE, 8,
+				EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
+				EGL_NONE
+	};
+
+	EGLint context_attribs[] = {
+		EGL_CONTEXT_CLIENT_VERSION, 2,
+		EGL_NONE
+	};
+
+	if (!eglChooseConfig(ed, config_attribs, 0, 0, &a)) {
+		LOG(LOG_ERROR,_("Could not get number of EGL configurations"));
+	} else {
+	    LOG(LOG_INFO, "Number of EGL configurations: " << a);
+	}
+	EGLConfig *conf = new EGLConfig[a];
+	if (!eglChooseConfig(ed, config_attribs, conf, a, &a))
+	{
+		LOG(LOG_ERROR,_("Could not find any EGL configuration"));
+		::abort();
+	}
+
+	int i;
+	for(i=0;i<a;i++)
+	{
+		EGLint id;
+		eglGetConfigAttrib(ed, conf[i], EGL_NATIVE_VISUAL_ID, &id);
+		if(id==(int)e->visual)
+			break;
+	}
+	if(i==a)
+	{
+		//No suitable id found
+		LOG(LOG_ERROR,_("No suitable graphics configuration available"));
+		return NULL;
+	}
+	th->mEGLConfig=conf[i];
+	cout << "Chosen config " << hex << conf[i] << dec << endl;
+
+	th->mEGLContext = eglCreateContext(ed, th->mEGLConfig, EGL_NO_CONTEXT, context_attribs);
+	if (th->mEGLContext == EGL_NO_CONTEXT) {
+		LOG(LOG_ERROR,_("Could not create EGL context"));
+		return NULL;
+	}
+	EGLSurface win = eglCreateWindowSurface(ed, th->mEGLConfig, e->window, NULL);
+	if (win == EGL_NO_SURFACE) {
+		LOG(LOG_ERROR,_("Could not create EGL surface"));
+		return NULL;
+	}
+	eglMakeCurrent(ed, win, win, th->mEGLContext);
+#endif
 
 	th->commonGLInit(th->windowWidth, th->windowHeight);
 	th->commonGLResize();
@@ -275,14 +353,6 @@ void* RenderThread::gtkplug_worker(RenderThread* th)
 	
 	ThreadProfile* profile=sys->allocateProfiler(RGB(200,0,0));
 	profile->setTag("Render");
-	FTTextureFont font(th->fontPath.c_str());
-	if(font.Error())
-	{
-		LOG(LOG_ERROR,_("Unable to load serif font"));
-		throw RunTimeException("Unable to load font");
-	}
-	
-	font.FaceSize(12);
 
 	glEnable(GL_TEXTURE_2D);
 	try
@@ -297,17 +367,14 @@ void* RenderThread::gtkplug_worker(RenderThread* th)
 			
 			if(th->resizeNeeded)
 			{
-				if(th->windowWidth!=th->newWidth ||
-					th->windowHeight!=th->newHeight)
-				{
-					th->windowWidth=th->newWidth;
-					th->windowHeight=th->newHeight;
-					LOG(LOG_ERROR,_("Window resize not supported in plugin"));
-				}
+				th->windowWidth=th->newWidth;
+				th->windowHeight=th->newHeight;
 				th->newWidth=0;
 				th->newHeight=0;
 				th->resizeNeeded=false;
+				LOG(LOG_INFO,_("Window resized to ") << th->windowWidth << 'x' << th->windowHeight);
 				th->commonGLResize();
+				th->m_sys->resizeCompleted();
 				profile->accountTime(chronometer.checkpoint());
 				continue;
 			}
@@ -327,44 +394,21 @@ void* RenderThread::gtkplug_worker(RenderThread* th)
 
 			if(th->m_sys->isOnError())
 			{
-				glLoadIdentity();
-				glScalef(1.0f/th->scaleX,-1.0f/th->scaleY,1);
-				glTranslatef(-th->offsetX,(th->windowHeight-th->offsetY)*(-1.0f),0);
-				glUseProgram(0);
-				glActiveTexture(GL_TEXTURE1);
-				glDisable(GL_TEXTURE_2D);
-				glActiveTexture(GL_TEXTURE0);
-
-				glBindFramebuffer(GL_FRAMEBUFFER, 0);
-				glDrawBuffer(GL_BACK);
-
-				glClearColor(0,0,0,1);
-				glClear(GL_COLOR_BUFFER_BIT);
-				glColor3f(0.8,0.8,0.8);
-					    
-				font.Render("We're sorry, Lightspark encountered a yet unsupported Flash file",
-						-1,FTPoint(0,th->windowHeight/2+20));
-
-				stringstream errorMsg;
-				errorMsg << "SWF file: " << th->m_sys->getOrigin().getParsedURL();
-				font.Render(errorMsg.str().c_str(),
-						-1,FTPoint(0,th->windowHeight/2));
-					    
-				errorMsg.str("");
-				errorMsg << "Cause: " << th->m_sys->errorCause;
-				font.Render(errorMsg.str().c_str(),
-						-1,FTPoint(0,th->windowHeight/2-20));
-
-				font.Render("Press C to copy this error to clipboard",
-						-1,FTPoint(0,th->windowHeight/2-40));
-				
-				glFlush();
+				th->renderErrorPage(th, th->m_sys->standalone);
+#ifndef ENABLE_GLES2
 				glXSwapBuffers(d,glxWin);
+#else
+				eglSwapBuffers(ed, win);
+#endif
 			}
 			else
 			{
+#ifndef ENABLE_GLES2
 				glXSwapBuffers(d,glxWin);
-				th->coreRendering(font);
+#else
+				eglSwapBuffers(ed, win);
+#endif
+				th->coreRendering();
 				//Call glFlush to offload work on the GPU
 				glFlush();
 			}
@@ -379,12 +423,16 @@ void* RenderThread::gtkplug_worker(RenderThread* th)
 	}
 	glDisable(GL_TEXTURE_2D);
 	th->commonGLDeinit();
+#ifndef ENABLE_GLES2
 	glXMakeCurrent(d,None,NULL);
 	glXDestroyContext(d,th->mContext);
+#else
+	eglMakeCurrent(ed, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
+	eglDestroyContext(ed, th->mEGLContext);
+#endif
 	XCloseDisplay(d);
 	return NULL;
 }
-#endif
 
 bool RenderThread::loadShaderPrograms()
 {
@@ -407,11 +455,17 @@ bool RenderThread::loadShaderPrograms()
 	bool ret=true;
 	char str[1024];
 	int a;
+	GLint stat;
 	assert(glCompileShader);
 	glCompileShader(f);
 	assert(glGetShaderInfoLog);
 	glGetShaderInfoLog(f,1024,&a,str);
-	LOG(LOG_NO_INFO,_("Fragment shader compilation ") << str);
+	LOG(LOG_INFO,_("Fragment shader compilation ") << str);
+	glGetShaderiv(f, GL_COMPILE_STATUS, &stat);
+	if (!stat)
+	{
+		throw RunTimeException("Could not compile fragment shader");
+	}
 
 	fs = dataFileRead("lightspark.vert");
 	if(fs==NULL)
@@ -422,18 +476,28 @@ bool RenderThread::loadShaderPrograms()
 	glShaderSource(g, 1, &fs,NULL);
 	free((void*)fs);
 
-	glCompileShader(g);
 	glGetShaderInfoLog(g,1024,&a,str);
-	LOG(LOG_NO_INFO,_("Vertex shader compilation ") << str);
+	LOG(LOG_INFO,_("Vertex shader compilation ") << str);
+
+	glCompileShader(g);
+	glGetShaderiv(g, GL_COMPILE_STATUS, &stat);
+	if (!stat)
+	{
+		throw RunTimeException("Could not compile vertex shader");
+	}
 
 	assert(glCreateProgram);
 	gpu_program = glCreateProgram();
+	glBindAttribLocation(gpu_program, VERTEX_ATTRIB, "ls_Vertex");
+	glBindAttribLocation(gpu_program, COLOR_ATTRIB, "ls_Color");
+	glBindAttribLocation(gpu_program, TEXCOORD_ATTRIB, "ls_TexCoord");
 	assert(glAttachShader);
 	glAttachShader(gpu_program,f);
 	glAttachShader(gpu_program,g);
 
 	assert(glLinkProgram);
 	glLinkProgram(gpu_program);
+
 	assert(glGetProgramiv);
 	glGetProgramiv(gpu_program,GL_LINK_STATUS,&a);
 	if(a==GL_FALSE)
@@ -454,9 +518,15 @@ bool RenderThread::loadShaderPrograms()
 	glShaderSource(v, 1, &fs,NULL);
 	free((void*)fs);
 
-	glCompileShader(v);
 	glGetShaderInfoLog(v,1024,&a,str);
-	LOG(LOG_NO_INFO,_("Vertex shader compilation ") << str);
+	LOG(LOG_INFO,_("Vertex shader compilation ") << str);
+
+	glCompileShader(v);
+	glGetShaderiv(v, GL_COMPILE_STATUS, &stat);
+	if (!stat)
+	{
+		throw RunTimeException("Could not compile vertex blitter shader");
+	}
 
 	blitter_program = glCreateProgram();
 	glAttachShader(blitter_program,v);
@@ -483,7 +553,7 @@ void RenderThread::commonGLDeinit()
 	for(;it!=uploadJobs.end();it++)
 		(*it)->uploadFence();
 	glBindFramebuffer(GL_FRAMEBUFFER,0);
-	glDeleteFramebuffers(1,&rt->fboId);
+	glDeleteFramebuffers(1,&fboId);
 	tempTex.shutdown();
 	for(uint32_t i=0;i<largeTextures.size();i++)
 	{
@@ -491,13 +561,17 @@ void RenderThread::commonGLDeinit()
 		delete[] largeTextures[i].bitmap;
 	}
 	glDeleteBuffers(2,pixelBuffers);
+	glDeleteTextures(1, &cairoTextureID);
 }
 
 void RenderThread::commonGLInit(int width, int height)
 {
+	GLenum err;
+//For now GLEW does not work with GLES2
+#ifndef ENABLE_GLES2
 	//Now we can initialize GLEW
 	glewExperimental = GL_TRUE;
-	GLenum err = glewInit();
+	err = glewInit();
 	if (GLEW_OK != err)
 	{
 		LOG(LOG_ERROR,_("Cannot initialize GLEW: cause ") << glewGetErrorString(err));;
@@ -510,7 +584,10 @@ void RenderThread::commonGLInit(int width, int height)
 	}
 	if(GLEW_ARB_texture_non_power_of_two)
 		hasNPOTTextures=true;
-
+#else
+		//Open GLES 2.0 has NPOT textures
+		hasNPOTTextures=true;
+#endif
 	//Load shaders
 	loadShaderPrograms();
 
@@ -546,6 +623,19 @@ void RenderThread::commonGLInit(int width, int height)
 	tex=glGetUniformLocation(gpu_program,"g_tex2");
 	if(tex!=-1)
 		glUniform1i(tex,1);
+
+	//The uniform that enables YUV->RGB transform on the texels (needed for video)
+	yuvUniform =glGetUniformLocation(gpu_program,"yuv");
+	//The uniform that tells the shader if a mask is being rendered
+	maskUniform =glGetUniformLocation(gpu_program,"mask");
+	//The uniform that tells the alpha value multiplied to the alpha of every pixel
+	alphaUniform =glGetUniformLocation(gpu_program,"alpha");
+	//The uniform that tells to draw directly using the selected color
+	directUniform =glGetUniformLocation(gpu_program,"direct");
+	//The uniform that contains the coordinate matrix
+	projectionMatrixUniform =glGetUniformLocation(gpu_program,"ls_ProjectionMatrix");
+	modelviewMatrixUniform =glGetUniformLocation(gpu_program,"ls_ModelViewMatrix");
+
 	fragmentTexScaleUniform=glGetUniformLocation(gpu_program,"texScale");
 	if(fragmentTexScaleUniform!=-1)
 		glUniform2f(fragmentTexScaleUniform,1.0f/width,1.0f/height);
@@ -553,13 +643,10 @@ void RenderThread::commonGLInit(int width, int height)
 
 	//Texturing must be enabled otherwise no tex coord will be sent to the shaders
 	glEnable(GL_TEXTURE_2D);
-	//Default to replace
-	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
 	//At least two texture unit are guaranteed in OpenGL 1.3
 	//The second unit will be used to access the temporary buffer
 	glActiveTexture(GL_TEXTURE1);
 	glEnable(GL_TEXTURE_2D);
-	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
 	glBindTexture(GL_TEXTURE_2D, tempTex.getId());
 
 	glActiveTexture(GL_TEXTURE0);
@@ -573,6 +660,7 @@ void RenderThread::commonGLInit(int width, int height)
 	if(status != GL_FRAMEBUFFER_COMPLETE)
 	{
 		LOG(LOG_ERROR,_("Incomplete FBO status ") << status << _("... Aborting"));
+		err=glGetError();
 		while(err!=GL_NO_ERROR)
 		{
 			LOG(LOG_ERROR,_("GL errors during initialization: ") << err);
@@ -580,6 +668,7 @@ void RenderThread::commonGLInit(int width, int height)
 		}
 		::abort();
 	}
+	glGenTextures(1, &cairoTextureID);
 }
 
 void RenderThread::commonGLResize()
@@ -656,16 +745,13 @@ void RenderThread::commonGLResize()
 			break;
 	}
 	glViewport(0,0,windowWidth,windowHeight);
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-	glOrtho(0,windowWidth,0,windowHeight,-100,0);
+	lsglLoadIdentity();
+	lsglOrtho(0,windowWidth,0,windowHeight,-100,0);
 	//scaleY is negated to adapt the flash and gl coordinates system
 	//An additional translation is added for the same reason
-	glTranslatef(offsetX,windowHeight-offsetY,0);
-	glScalef(scaleX,-scaleY,1);
-
-	glMatrixMode(GL_MODELVIEW);
-
+	lsglTranslatef(offsetX,windowHeight-offsetY,0);
+	lsglScalef(scaleX,-scaleY,1);
+	setMatrixUniform(LSGL_PROJECTION);
 	tempTex.resize(windowWidth, windowHeight);
 }
 
@@ -687,6 +773,12 @@ void RenderThread::resizePixelBuffers(uint32_t w, uint32_t h)
 	glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
 	pixelBufferWidth=w;
 	pixelBufferHeight=h;
+#ifdef ENABLE_GLES2
+	if (pixelBuf) {
+		free(pixelBuf);
+		pixelBuf = 0;
+	}
+#endif
 }
 
 void RenderThread::renderMaskToTmpBuffer() const
@@ -701,14 +793,110 @@ void RenderThread::renderMaskToTmpBuffer() const
 	{
 		float matrix[16];
 		maskStack[i].m.get4DMatrix(matrix);
-		glLoadMatrixf(matrix);
+		lsglLoadMatrixf(matrix);
+		setMatrixUniform(LSGL_MODELVIEW);
 		maskStack[i].d->Render(true);
 	}
 	glBindFramebuffer(GL_FRAMEBUFFER, 0);
 	glDrawBuffer(GL_BACK);
 }
 
-void RenderThread::coreRendering(FTFont& font)
+cairo_t* RenderThread::getCairoContext(int w, int h)
+{
+	if (!cairoTextureContext) {
+		cairoTextureData = new uint8_t[w*h*4];
+		cairoTextureSurface = cairo_image_surface_create_for_data(cairoTextureData, CAIRO_FORMAT_ARGB32, w, h, w*4);
+		cairoTextureContext = cairo_create(cairoTextureSurface);
+
+		cairo_select_font_face (cairoTextureContext, fontPath.c_str(), CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
+		cairo_set_font_size(cairoTextureContext, 11);
+	}
+	return cairoTextureContext;
+}
+
+//Render strings using Cairo's 'toy' text API
+void RenderThread::renderText(cairo_t *cr, const char *text, int x, int y)
+{
+	cairo_move_to(cr, x, y);
+	cairo_save(cr);
+	cairo_scale(cr, 1.0, -1.0);
+	cairo_show_text(cr, text);
+	cairo_restore(cr);
+}
+
+//Send the texture drawn by Cairo to the GPU
+void RenderThread::mapCairoTexture(int w, int h)
+{
+	glEnable(GL_TEXTURE_2D);
+	glBindTexture(GL_TEXTURE_2D, cairoTextureID);
+
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, w, h, 0, GL_BGRA, GL_UNSIGNED_BYTE, cairoTextureData);
+
+	GLfloat vertex_coords[] = {0,0, GLfloat(w),0, 0,GLfloat(h), GLfloat(w),GLfloat(h)};
+	GLfloat texture_coords[] = {0,0, 1,0, 0,1, 1,1};
+	glVertexAttribPointer(VERTEX_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, vertex_coords);
+	glVertexAttribPointer(TEXCOORD_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, texture_coords);
+	glEnableVertexAttribArray(VERTEX_ATTRIB);
+	glEnableVertexAttribArray(TEXCOORD_ATTRIB);
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+	glDisableVertexAttribArray(VERTEX_ATTRIB);
+	glDisableVertexAttribArray(TEXCOORD_ATTRIB);
+}
+
+void RenderThread::plotProfilingData()
+{
+	lsglLoadIdentity();
+	lsglScalef(1.0f/scaleX,-1.0f/scaleY,1);
+	lsglTranslatef(-offsetX,(windowHeight-offsetY)*(-1.0f),0);
+	setMatrixUniform(LSGL_MODELVIEW);
+
+	cairo_t *cr = getCairoContext(windowWidth, windowHeight);
+
+	glUniform1f(rt->directUniform, 1);
+
+	char frameBuf[20];
+	snprintf(frameBuf,20,"Frame %u",m_sys->state.FP);
+
+	GLfloat vertex_coords[40];
+	GLfloat color_coords[80];
+
+	//Draw bars
+	for (int i=0;i<9;i++)
+	{
+		vertex_coords[i*4] = 0;
+		vertex_coords[i*4+1] = (i+1)*windowHeight/10;
+		vertex_coords[i*4+2] = windowWidth;
+		vertex_coords[i*4+3] = (i+1)*windowHeight/10;
+	}
+	for (int i=0;i<80;i++)
+		color_coords[i] = 0.7;
+
+	glVertexAttribPointer(VERTEX_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, vertex_coords);
+	glVertexAttribPointer(COLOR_ATTRIB, 4, GL_FLOAT, GL_FALSE, 0, color_coords);
+	glEnableVertexAttribArray(VERTEX_ATTRIB);
+	glEnableVertexAttribArray(COLOR_ATTRIB);
+	glDrawArrays(GL_LINES, 0, 20);
+	glDisableVertexAttribArray(VERTEX_ATTRIB);
+	glDisableVertexAttribArray(COLOR_ATTRIB);
+ 
+	list<ThreadProfile>::iterator it=m_sys->profilingData.begin();
+	for(;it!=m_sys->profilingData.end();it++)
+		it->plot(1000000/m_sys->getFrameRate(),cr);
+	glUniform1f(rt->directUniform, 0);
+
+	mapCairoTexture(windowWidth, windowHeight);
+
+	//clear the surface
+	cairo_save(cr);
+	cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+	cairo_paint(cr);
+	cairo_restore(cr);
+
+}
+
+void RenderThread::coreRendering()
 {
 	glBindFramebuffer(GL_FRAMEBUFFER, 0);
 	glDrawBuffer(GL_BACK);
@@ -716,174 +904,58 @@ void RenderThread::coreRendering(FTFont& font)
 	RGB bg=sys->getBackground();
 	glClearColor(bg.Red/255.0F,bg.Green/255.0F,bg.Blue/255.0F,1);
 	glClear(GL_COLOR_BUFFER_BIT);
-	glLoadIdentity();
+	lsglLoadIdentity();
+	setMatrixUniform(LSGL_MODELVIEW);
 
 	m_sys->Render(false);
 	assert(maskStack.empty());
 
 	if(m_sys->showProfilingData)
-	{
-		glLoadIdentity();
-		glScalef(1.0f/scaleX,-1.0f/scaleY,1);
-		glTranslatef(-offsetX,(windowHeight-offsetY)*(-1.0f),0);
-		glUseProgram(0);
-		glActiveTexture(GL_TEXTURE1);
-		glDisable(GL_TEXTURE_2D);
-		glActiveTexture(GL_TEXTURE0);
-		glDisable(GL_TEXTURE_2D);
-		glColor3f(0,0,0);
-		char frameBuf[20];
-		snprintf(frameBuf,20,"Frame %u",m_sys->state.FP);
-		font.Render(frameBuf,-1,FTPoint(0,0));
-
-		//Draw bars
-		glColor4f(0.7,0.7,0.7,0.7);
-		glBegin(GL_LINES);
-		for(int i=1;i<10;i++)
-		{
-			glVertex2i(0,(i*windowHeight/10));
-			glVertex2i(windowWidth,(i*windowHeight/10));
-		}
-		glEnd();
-		
-		list<ThreadProfile>::iterator it=m_sys->profilingData.begin();
-		for(;it!=m_sys->profilingData.end();it++)
-			it->plot(1000000/m_sys->getFrameRate(),&font);
-		glActiveTexture(GL_TEXTURE1);
-		glEnable(GL_TEXTURE_2D);
-		glActiveTexture(GL_TEXTURE0);
-		glEnable(GL_TEXTURE_2D);
-		glUseProgram(gpu_program);
-	}
+		plotProfilingData();
 }
 
-void* RenderThread::sdl_worker(RenderThread* th)
+//Renders the error message which caused the VM to stop.
+void RenderThread::renderErrorPage(RenderThread *th, bool standalone)
 {
-	sys=th->m_sys;
-	rt=th;
-	SemaphoreLighter lighter(th->initialized);
-	RECT size=sys->getFrameSize();
-	int initialWidth=size.Xmax/20;
-	int initialHeight=size.Ymax/20;
-	th->windowWidth=initialWidth;
-	th->windowHeight=initialHeight;
-	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
-	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
-	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
-	SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 0 );
-	SDL_GL_SetAttribute( SDL_GL_ACCELERATED_VISUAL, 1); 
-	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+	lsglLoadIdentity();
+	lsglScalef(1.0f/th->scaleX,-1.0f/th->scaleY,1);
+	lsglTranslatef(-th->offsetX,(th->windowHeight-th->offsetY)*(-1.0f),0);
 
-	SDL_SetVideoMode(th->windowWidth, th->windowHeight, 24, SDL_OPENGL|SDL_RESIZABLE);
-	SDL_WM_SetCaption("Lightspark","Lightspark");
-	th->commonGLInit(th->windowWidth, th->windowHeight);
-	th->commonGLResize();
-	lighter.light();
+	setMatrixUniform(LSGL_MODELVIEW);
 
-	ThreadProfile* profile=sys->allocateProfiler(RGB(200,0,0));
-	profile->setTag("Render");
-	FTTextureFont font(th->fontPath.c_str());
-	if(font.Error())
-		throw RunTimeException("Unable to load font");
-	
-	font.FaceSize(12);
-	try
+	cairo_t *cr = getCairoContext(windowWidth, windowHeight);
+
+	cairo_set_source_rgb(cr, 0, 0, 0);
+	cairo_paint(cr);
+	cairo_set_source_rgb(cr, 0.8, 0.8, 0.8);
+
+	renderText(cr, "We're sorry, Lightspark encountered a yet unsupported Flash file",
+			0,th->windowHeight/2+20);
+
+	stringstream errorMsg;
+	errorMsg << "SWF file: " << th->m_sys->getOrigin().getParsedURL();
+	renderText(cr, errorMsg.str().c_str(),0,th->windowHeight/2);
+
+	errorMsg.str("");
+	errorMsg << "Cause: " << th->m_sys->errorCause;
+	renderText(cr, errorMsg.str().c_str(),0,th->windowHeight/2-20);
+
+	if (standalone)
 	{
-		Chronometer chronometer;
-		while(1)
-		{
-			sem_wait(&th->event);
-			if(th->m_sys->isShuttingDown())
-				break;
-			chronometer.checkpoint();
+		renderText(cr, "Please look at the console output to copy this error",
+			0,th->windowHeight/2-40);
 
-			if(th->resizeNeeded)
-			{
-				if(th->windowWidth!=th->newWidth ||
-					th->windowHeight!=th->newHeight)
-				{
-					th->windowWidth=th->newWidth;
-					th->windowHeight=th->newHeight;
-					SDL_SetVideoMode(th->windowWidth, th->windowHeight, 24, SDL_OPENGL|SDL_RESIZABLE);
-				}
-				th->newWidth=0;
-				th->newHeight=0;
-				th->resizeNeeded=false;
-				th->commonGLResize();
-				profile->accountTime(chronometer.checkpoint());
-				continue;
-			}
-
-			if(th->newTextureNeeded)
-				th->handleNewTexture();
-
-			if(th->prevUploadJob)
-				th->finalizeUpload();
-
-			if(th->uploadNeeded)
-			{
-				th->handleUpload();
-				profile->accountTime(chronometer.checkpoint());
-				continue;
-			}
-
-			SDL_PumpEvents();
-			if(th->m_sys->isOnError())
-			{
-				glLoadIdentity();
-				glScalef(1.0f/th->scaleX,-1.0f/th->scaleY,1);
-				glTranslatef(-th->offsetX,(th->windowHeight-th->offsetY)*(-1.0f),0);
-				glUseProgram(0);
-				glActiveTexture(GL_TEXTURE1);
-				glDisable(GL_TEXTURE_2D);
-				glActiveTexture(GL_TEXTURE0);
-
-				glBindFramebuffer(GL_FRAMEBUFFER, 0);
-				glDrawBuffer(GL_BACK);
-
-				glClearColor(0,0,0,1);
-				glClear(GL_COLOR_BUFFER_BIT);
-				glColor3f(0.8,0.8,0.8);
-					    
-				font.Render("We're sorry, Lightspark encountered a yet unsupported Flash file",
-						-1,FTPoint(0,th->windowHeight/2+20));
-
-				stringstream errorMsg;
-				errorMsg << "SWF file: " << th->m_sys->getOrigin().getParsedURL();
-				font.Render(errorMsg.str().c_str(),
-						-1,FTPoint(0,th->windowHeight/2));
-					    
-				errorMsg.str("");
-				errorMsg << "Cause: " << th->m_sys->errorCause;
-				font.Render(errorMsg.str().c_str(),
-						-1,FTPoint(0,th->windowHeight/2-20));
-				
-				font.Render("Please look at the console output to copy this error",
-						-1,FTPoint(0,th->windowHeight/2-40));
-
-				font.Render("Press 'Q' to exit",-1,FTPoint(0,th->windowHeight/2-60));
-				
-				glFlush();
-				SDL_GL_SwapBuffers( );
-			}
-			else
-			{
-				SDL_GL_SwapBuffers( );
-				th->coreRendering(font);
-				//Call glFlush to offload work on the GPU
-				glFlush();
-			}
-			profile->accountTime(chronometer.checkpoint());
-			th->renderNeeded=false;
-		}
+		renderText(cr, "Press 'Q' to exit",0,th->windowHeight/2-60);
 	}
-	catch(LightsparkException& e)
+	else
 	{
-		LOG(LOG_ERROR,_("Exception in RenderThread ") << e.cause);
-		sys->setError(e.cause);
+		renderText(cr, "Press C to copy this error to clipboard",
+				0,th->windowHeight/2-40);
 	}
-	th->commonGLDeinit();
-	return NULL;
+
+	glUniform1f(rt->alphaUniform, 1);
+	mapCairoTexture(windowWidth, windowHeight);
+	glFlush();
 }
 
 void RenderThread::addUploadJob(ITextureUploadable* u)
@@ -921,7 +993,7 @@ void RenderThread::draw(bool force)
 	if(diff>1000)
 	{
 		time_s=time_d;
-		LOG(LOG_NO_INFO,_("FPS: ") << dec << frameCount);
+		LOG(LOG_INFO,_("FPS: ") << dec << frameCount);
 		frameCount=0;
 		secsCount++;
 	}
@@ -936,8 +1008,8 @@ void RenderThread::tick()
 
 void RenderThread::releaseTexture(const TextureChunk& chunk)
 {
-	uint32_t blocksW=(chunk.width+127)/128;
-	uint32_t blocksH=(chunk.height+127)/128;
+	uint32_t blocksW=(chunk.width+CHUNKSIZE-1)/CHUNKSIZE;
+	uint32_t blocksH=(chunk.height+CHUNKSIZE-1)/CHUNKSIZE;
 	uint32_t numberOfBlocks=blocksW*blocksH;
 	Locker l(mutexLargeTexture);
 	LargeTexture& tex=largeTextures[chunk.texId];
@@ -976,8 +1048,8 @@ RenderThread::LargeTexture& RenderThread::allocateNewTexture()
 {
 	//Signal that a new texture is needed
 	newTextureNeeded=true;
-	//Let's allocate the bitmap for the texture blocks, minumum block size is 128
-	uint32_t bitmapSize=(largeTextureSize/128)*(largeTextureSize/128)/8;
+	//Let's allocate the bitmap for the texture blocks, minumum block size is CHUNKSIZE
+	uint32_t bitmapSize=(largeTextureSize/CHUNKSIZE)*(largeTextureSize/CHUNKSIZE)/8;
 	uint8_t* bitmap=new uint8_t[bitmapSize];
 	memset(bitmap,0,bitmapSize);
 	largeTextures.emplace_back(bitmap);
@@ -988,7 +1060,7 @@ bool RenderThread::allocateChunkOnTextureCompact(LargeTexture& tex, TextureChunk
 {
 	//Find a contiguos set of blocks
 	uint32_t start;
-	uint32_t blockPerSide=largeTextureSize/128;
+	uint32_t blockPerSide=largeTextureSize/CHUNKSIZE;
 	uint32_t bitmapSize=blockPerSide*blockPerSide;
 	for(start=0;start<bitmapSize;start++)
 	{
@@ -1035,7 +1107,7 @@ bool RenderThread::allocateChunkOnTextureSparse(LargeTexture& tex, TextureChunk&
 {
 	//Allocate a sparse set of texture chunks
 	uint32_t found=0;
-	uint32_t blockPerSide=largeTextureSize/128;
+	uint32_t blockPerSide=largeTextureSize/CHUNKSIZE;
 	uint32_t bitmapSize=blockPerSide*blockPerSide;
 	//TODO: use the already allocated array
 	uint32_t* tmp=new uint32_t[blocksW*blocksH];
@@ -1075,8 +1147,8 @@ TextureChunk RenderThread::allocateTexture(uint32_t w, uint32_t h, bool compact)
 	assert(w && h);
 	Locker l(mutexLargeTexture);
 	//Find the number of blocks needed for the given w and h
-	uint32_t blocksW=(w+127)/128;
-	uint32_t blocksH=(h+127)/128;
+	uint32_t blocksW=(w+CHUNKSIZE-1)/CHUNKSIZE;
+	uint32_t blocksH=(h+CHUNKSIZE-1)/CHUNKSIZE;
 	TextureChunk ret(w, h);
 	//Try to find a good place in the available textures
 	uint32_t index=0;
@@ -1117,27 +1189,37 @@ TextureChunk RenderThread::allocateTexture(uint32_t w, uint32_t h, bool compact)
 	return ret;
 }
 
-void RenderThread::renderTextured(const TextureChunk& chunk, uint32_t x, uint32_t y, uint32_t w, uint32_t h)
+void RenderThread::renderTextured(const TextureChunk& chunk, int32_t x, int32_t y, uint32_t w, uint32_t h)
 {
 	glBindTexture(GL_TEXTURE_2D, largeTextures[chunk.texId].id);
-	const uint32_t blocksPerSide=largeTextureSize/128;
+	const uint32_t blocksPerSide=largeTextureSize/CHUNKSIZE;
 	uint32_t startX, startY, endX, endY;
-	assert(chunk.getNumberOfChunks()==((chunk.width+127)/128)*((chunk.height+127)/128));
+	assert(chunk.getNumberOfChunks()==((chunk.width+CHUNKSIZE-1)/CHUNKSIZE)*((chunk.height+CHUNKSIZE-1)/CHUNKSIZE));
 	
 	uint32_t curChunk=0;
-	for(uint32_t i=0;i<chunk.height;i+=128)
+	//The 4 corners of each texture are specified as the vertices of 2 triangles,
+	//so there are 6 vertices per quad, two of them duplicated (the diagonal)
+	GLfloat *vertex_coords = new GLfloat[chunk.getNumberOfChunks()*12];
+	GLfloat *texture_coords = new GLfloat[chunk.getNumberOfChunks()*12];
+	for(uint32_t i=0, k=0;i<chunk.height;i+=CHUNKSIZE)
 	{
 		startY=h*i/chunk.height;
-		endY=min(h*(i+128)/chunk.height,h);
-		for(uint32_t j=0;j<chunk.width;j+=128)
+		endY=min(h*(i+CHUNKSIZE)/chunk.height,h);
+		//Take yOffset into account
+		startY = (y<0)?startY:y+startY;
+		endY = (y<0)?endY:y+endY;
+		for(uint32_t j=0;j<chunk.width;j+=CHUNKSIZE)
 		{
 			startX=w*j/chunk.width;
-			endX=min(w*(j+128)/chunk.width,w);
+			endX=min(w*(j+CHUNKSIZE)/chunk.width,w);
+			//Take xOffset into account
+			startX = (x<0)?startX:x+startX;
+			endX = (x<0)?endX:x+endX;
 			const uint32_t curChunkId=chunk.chunks[curChunk];
-			const uint32_t blockX=((curChunkId%blocksPerSide)*128);
-			const uint32_t blockY=((curChunkId/blocksPerSide)*128);
-			const uint32_t availX=min(int(chunk.width-j),128);
-			const uint32_t availY=min(int(chunk.height-i),128);
+			const uint32_t blockX=((curChunkId%blocksPerSide)*CHUNKSIZE);
+			const uint32_t blockY=((curChunkId/blocksPerSide)*CHUNKSIZE);
+			const uint32_t availX=min(int(chunk.width-j),CHUNKSIZE);
+			const uint32_t availY=min(int(chunk.height-i),CHUNKSIZE);
 			float startU=blockX;
 			startU/=largeTextureSize;
 			float startV=blockY;
@@ -1146,22 +1228,54 @@ void RenderThread::renderTextured(const TextureChunk& chunk, uint32_t x, uint32_
 			endU/=largeTextureSize;
 			float endV=blockY+availY;
 			endV/=largeTextureSize;
-			glBegin(GL_QUADS);
-				glTexCoord2f(startU,startV);
-				glVertex2i(x+startX,y+startY);
 
-				glTexCoord2f(endU,startV);
-				glVertex2i(x+endX,y+startY);
+			//Upper-right triangle of the quad
+			texture_coords[k] = startU;
+			texture_coords[k+1] = startV;
+			vertex_coords[k] = startX;
+			vertex_coords[k+1] = startY;
+			k+=2;
+			texture_coords[k] = endU;
+			texture_coords[k+1] = startV;
+			vertex_coords[k] = endX;
+			vertex_coords[k+1] = startY;
+			k+=2;
+			texture_coords[k] = endU;
+			texture_coords[k+1] = endV;
+			vertex_coords[k] = endX;
+			vertex_coords[k+1] = endY;
+			k+=2;
 
-				glTexCoord2f(endU,endV);
-				glVertex2i(x+endX,y+endY);
+			//Lower-left triangle of the quad
+			texture_coords[k] = startU;
+			texture_coords[k+1] = startV;
+			vertex_coords[k] = startX;
+			vertex_coords[k+1] = startY;
+			k+=2;
+			texture_coords[k] = endU;
+			texture_coords[k+1] = endV;
+			vertex_coords[k] = endX;
+			vertex_coords[k+1] = endY;
+			k+=2;
+			texture_coords[k] = startU;
+			texture_coords[k+1] = endV;
+			vertex_coords[k] = startX;
+			vertex_coords[k+1] = endY;
+			k+=2;
 
-				glTexCoord2f(startU,endV);
-				glVertex2i(x+startX,y+endY);
-			glEnd();
 			curChunk++;
 		}
 	}
+
+	glVertexAttribPointer(VERTEX_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, vertex_coords);
+	glVertexAttribPointer(TEXCOORD_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, texture_coords);
+	glEnableVertexAttribArray(VERTEX_ATTRIB);
+	glEnableVertexAttribArray(TEXCOORD_ATTRIB);
+	glDrawArrays(GL_TRIANGLES, 0, curChunk*6);
+	glDisableVertexAttribArray(VERTEX_ATTRIB);
+	glDisableVertexAttribArray(TEXCOORD_ATTRIB);
+	delete[] vertex_coords;
+	delete[] texture_coords;
 }
 
 void RenderThread::loadChunkBGRA(const TextureChunk& chunk, uint32_t w, uint32_t h, uint8_t* data)
@@ -1173,27 +1287,45 @@ void RenderThread::loadChunkBGRA(const TextureChunk& chunk, uint32_t w, uint32_t
 	//TODO: Detect continuos
 	//The size is ok if doesn't grow over the allocated size
 	//this allows some alignment freedom
-	assert(w<=((chunk.width+127)&0xffffff80));
-	assert(h<=((chunk.height+127)&0xffffff80));
+	assert(w<=((chunk.width+CHUNKSIZE-1)&0xffffff80));
+	assert(h<=((chunk.height+CHUNKSIZE-1)&0xffffff80));
 	const uint32_t numberOfChunks=chunk.getNumberOfChunks();
-	const uint32_t blocksPerSide=largeTextureSize/128;
-	const uint32_t blocksW=(w+127)/128;
+	const uint32_t blocksPerSide=largeTextureSize/CHUNKSIZE;
+	const uint32_t blocksW=(w+CHUNKSIZE-1)/CHUNKSIZE;
 	glPixelStorei(GL_UNPACK_ROW_LENGTH,w);
 	for(uint32_t i=0;i<numberOfChunks;i++)
 	{
-		uint32_t curX=(i%blocksW)*128;
-		uint32_t curY=(i/blocksW)*128;
-		uint32_t sizeX=min(int(w-curX),128);
-		uint32_t sizeY=min(int(h-curY),128);
+		uint32_t curX=(i%blocksW)*CHUNKSIZE;
+		uint32_t curY=(i/blocksW)*CHUNKSIZE;
+		uint32_t sizeX=min(int(w-curX),CHUNKSIZE);
+		uint32_t sizeY=min(int(h-curY),CHUNKSIZE);
 		glPixelStorei(GL_UNPACK_SKIP_PIXELS,curX);
 		glPixelStorei(GL_UNPACK_SKIP_ROWS,curY);
-		const uint32_t blockX=((chunk.chunks[i]%blocksPerSide)*128);
-		const uint32_t blockY=((chunk.chunks[i]/blocksPerSide)*128);
+		const uint32_t blockX=((chunk.chunks[i]%blocksPerSide)*CHUNKSIZE);
+		const uint32_t blockY=((chunk.chunks[i]/blocksPerSide)*CHUNKSIZE);
 		while(glGetError()!=GL_NO_ERROR);
+#ifndef ENABLE_GLES2
 		glTexSubImage2D(GL_TEXTURE_2D, 0, blockX, blockY, sizeX, sizeY, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_HOST, data);
+#else
+		//We need to copy the texture area to a contiguous memory region first,
+		//as GLES2 does not support UNPACK state (skip pixels, skip rows, row_lenght).
+		uint8_t *gdata = new uint8_t[4*sizeX*sizeY];
+		for(unsigned int j=0;j<sizeY;j++) {
+			memcpy(gdata+4*j*sizeX, data+4*w*(j+curY)+4*curX, sizeX*4);
+		}
+		glTexSubImage2D(GL_TEXTURE_2D, 0, blockX, blockY, sizeX, sizeY, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_HOST, gdata);
+		delete[] gdata;
+#endif
 		assert(glGetError()!=GL_INVALID_OPERATION);
 	}
 	glPixelStorei(GL_UNPACK_SKIP_PIXELS,0);
 	glPixelStorei(GL_UNPACK_SKIP_ROWS,0);
 	glPixelStorei(GL_UNPACK_ROW_LENGTH,0);
 }
+
+void RenderThread::setMatrixUniform(LSGL_MATRIX m) const
+{
+	GLint uni = (m == LSGL_MODELVIEW) ? modelviewMatrixUniform:projectionMatrixUniform;
+
+	glUniformMatrix4fv(uni, 1, GL_FALSE, lsMVPMatrix);
+}
