@@ -103,11 +103,19 @@ public:
 	{
 		return tiny_string(encode(std::string(u.raw_buf()), type));
 	};
+	static Glib::ustring encode(const Glib::ustring& u, ENCODING type=ENCODE_URICOMPONENT)
+	{
+		return encode(u.raw(), type);
+	}
 	static std::string encode(const std::string& u, ENCODING type=ENCODE_URICOMPONENT);
 	static tiny_string decode(const tiny_string& u, ENCODING type=ENCODE_URICOMPONENT)
 	{
 		return tiny_string(decode(std::string(u.raw_buf()), type));
 	};
+	static Glib::ustring decode(const Glib::ustring& u, ENCODING type=ENCODE_URICOMPONENT)
+	{
+		return decode(u.raw(), type);
+	}
 	static std::string decode(const std::string& u, ENCODING type=ENCODE_URICOMPONENT);
 };
 
