@@ -35,12 +35,12 @@ FLV_HEADER::FLV_HEADER(std::istream& in):dataOffset(0),_hasAudio(false),_hasVide
 	
 	if(Signature[0]=='F' && Signature[1]=='L' && Signature[2]=='V')
 	{
-		LOG(LOG_NO_INFO, _("PARSING: FLV file: Version ") << (int)Version);
+		LOG(LOG_INFO, _("PARSING: FLV file: Version ") << (int)Version);
 		valid=true;
 	}
 	else
 	{
-		LOG(LOG_NO_INFO,_("PARSING: No FLV file signature found"));
+		LOG(LOG_INFO,_("PARSING: No FLV file signature found"));
 		valid=false;
 		return;
 	}
