@@ -26,13 +26,14 @@
 #include <sstream>
 #include "scripting/abc.h"
 #include "tags.h"
-#include "scripting/actions.h"
 #include "backends/geometry.h"
 #include "backends/rendering.h"
+#include "backends/security.h"
 #include "swftypes.h"
 #include "swf.h"
 #include "logger.h"
 #include "compat.h"
+#include "streams.h"
 
 #undef RGB
 
@@ -41,11 +42,23 @@ using namespace lightspark;
 
 extern TLSDATA ParseThread* pt;
 
-Tag* TagFactory::readTag()
+_NR<Tag> TagFactory::readTag()
 {
 	RECORDHEADER h;
-	f >> h;
-	
+
+	//Catch eofs
+	try
+	{
+		f >> h;
+	}
+	catch (ifstream::failure e) {
+		if(!f.eof()) //Only handle eof
+			throw e;
+		f.clear();
+		LOG(LOG_INFO,"Simulating EndTag at EOF @ " << f.tellg());
+		return _MR(new EndTag(h,f));
+	}
+
 	unsigned int expectedLen=h.getLength();
 	unsigned int start=f.tellg();
 	Tag* ret=NULL;
@@ -53,7 +66,7 @@ Tag* TagFactory::readTag()
 	switch(h.getTagType())
 	{
 		case 0:
-			LOG(LOG_TRACE, _("EndTag at position ") << f.tellg());
+			LOG(LOG_INFO,_("End of parsing @ ") << f.tellg());
 			ret=new EndTag(h,f);
 			break;
 		case 1:
@@ -76,9 +89,6 @@ Tag* TagFactory::readTag()
 		case 11:
 			ret=new DefineTextTag(h,f);
 			break;
-		case 12:
-			ret=new DoActionTag(h,f);
-			break;
 		case 13:
 			ret=new DefineFontInfoTag(h,f);
 			break;
@@ -95,7 +105,7 @@ Tag* TagFactory::readTag()
 			ret=new SoundStreamBlockTag(h,f);
 			break;
 		case 20:
-			ret=new DefineBitsLosslessTag(h,f);
+			ret=new DefineBitsLosslessTag(h,f,1);
 			break;
 		case 21:
 			ret=new DefineBitsJPEG2Tag(h,f);
@@ -125,7 +135,7 @@ Tag* TagFactory::readTag()
 			ret=new DefineBitsJPEG3Tag(h,f);
 			break;
 		case 36:
-			ret=new DefineBitsLossless2Tag(h,f);
+			ret=new DefineBitsLosslessTag(h,f,2);
 			break;
 		case 37:
 			ret=new DefineEditTextTag(h,f);
@@ -148,15 +158,9 @@ Tag* TagFactory::readTag()
 		case 48:
 			ret=new DefineFont2Tag(h,f);
 			break;
-		case 56:
-			ret=new ExportAssetsTag(h,f);
-			break;
 		case 58:
 			ret=new EnableDebuggerTag(h,f);
 			break;
-		case 59:
-			ret=new DoInitActionTag(h,f);
-			break;
 		case 60:
 			ret=new DefineVideoStreamTag(h,f);
 			break;
@@ -226,8 +230,16 @@ Tag* TagFactory::readTag()
 	}
 
 	//Check if this clip is the main clip and if AVM2 has been enabled by a FileAttributes tag
-	if(topLevel && firstTag && pt->root==sys)
+	if(topLevel && firstTag && pt->getRootMovie()==sys)
+	{
 		sys->needsAVM2(pt->useAVM2);
+		if(pt->useNetwork
+		&& sys->securityManager->getSandboxType() == SecurityManager::LOCAL_WITH_FILE)
+		{
+			sys->securityManager->setSandboxType(SecurityManager::LOCAL_WITH_NETWORK);
+			LOG(LOG_INFO, _("Switched to local-with-networking sandbox by FileAttributesTag"));
+		}
+	}
 	firstTag=false;
 
 	unsigned int end=f.tellg();
@@ -245,7 +257,7 @@ Tag* TagFactory::readTag()
 		throw ParseException("Malformed SWF file");
 	}
 	
-	return ret;
+	return _MNR(ret);
 }
 
 RemoveObject2Tag::RemoveObject2Tag(RECORDHEADER h, std::istream& in):DisplayListTag(h)
@@ -292,6 +304,7 @@ DefineEditTextTag::DefineEditTextTag(RECORDHEADER h, std::istream& in):Dictionar
 		if(HasFontClass)
 			in >> FontClass;
 		in >> FontHeight;
+		textData.fontSize = FontHeight;
 	}
 	if(HasTextColor)
 		in >> TextColor;
@@ -304,20 +317,19 @@ DefineEditTextTag::DefineEditTextTag(RECORDHEADER h, std::istream& in):Dictionar
 	in >> VariableName;
 	LOG(LOG_NOT_IMPLEMENTED,_("Sync to variable name ") << VariableName);
 	if(HasText)
+	{
 		in >> InitialText;
-}
-
-void DefineEditTextTag::Render(bool maskEnabled)
-{
-	LOG(LOG_NOT_IMPLEMENTED,_("DefineEditTextTag: Render"));
+		textData.text = (const char*)InitialText;
+	}
+	LOG(LOG_NOT_IMPLEMENTED, "DefineEditTextTag does not parse many attributes");
 }
 
 ASObject* DefineEditTextTag::instance() const
 {
-	DefineEditTextTag* ret=new DefineEditTextTag(*this);
+	TextField* ret=new TextField(textData);
 	//TODO: check
 	assert_and_throw(bindedTo==NULL);
-	ret->setPrototype(Class<TextField>::getClass());
+	ret->setClass(Class<TextField>::getClass());
 	return ret;
 }
 
@@ -328,7 +340,7 @@ DefineSpriteTag::DefineSpriteTag(RECORDHEADER h, std::istream& in):DictionaryTag
 	LOG(LOG_TRACE,_("DefineSprite ID: ") << SpriteID);
 	//Create a non top level TagFactory
 	TagFactory factory(in, false);
-	Tag* tag;
+	_NR<Tag> tag;
 	bool done=false;
 	bool empty=true;
 	do
@@ -342,7 +354,7 @@ DefineSpriteTag::DefineSpriteTag(RECORDHEADER h, std::istream& in):DictionaryTag
 			case DICT_TAG:
 				throw ParseException("Dictionary tag inside a sprite. Should not happen.");
 			case DISPLAY_LIST_TAG:
-				addToFrame(static_cast<DisplayListTag*>(tag));
+				addToFrame(tag.cast<DisplayListTag>());
 				empty=false;
 				break;
 			case SHOW_TAG:
@@ -355,7 +367,7 @@ DefineSpriteTag::DefineSpriteTag(RECORDHEADER h, std::istream& in):DictionaryTag
 			case CONTROL_TAG:
 				throw ParseException("Control tag inside a sprite. Should not happen.");
 			case FRAMELABEL_TAG:
-				addFrameLabel(frames.size()-1,static_cast<FrameLabelTag*>(tag)->Name);
+				addFrameLabel(frames.size()-1,static_cast<FrameLabelTag*>(tag.getPtr())->Name);
 				empty=false;
 				break;
 			case TAG:
@@ -388,12 +400,11 @@ ASObject* DefineSpriteTag::instance() const
 	if(bindedTo)
 	{
 		//A class is binded to this tag
-		ret->setPrototype(bindedTo);
+		ret->setClass(bindedTo);
 	}
 	else
-		ret->setPrototype(Class<MovieClip>::getClass());
-	ret->bootstrap();
-	//TODO: should we call the frameScripts?
+		ret->setClass(Class<MovieClip>::getClass());
+
 	return ret;
 }
 
@@ -549,8 +560,12 @@ DefineFont3Tag::DefineFont3Tag(RECORDHEADER h, std::istream& in):FontTag(h, 1)
 			in >> t;
 			OffsetTable.push_back(t);
 		}
-		in >> t;
-		CodeTableOffset=t;
+		//CodeTableOffset is missing with zero NumGlyphs
+		if(NumGlyphs)
+		{
+			in >> t;
+			CodeTableOffset=t;
+		}
 	}
 	else
 	{
@@ -560,8 +575,12 @@ DefineFont3Tag::DefineFont3Tag(RECORDHEADER h, std::istream& in):FontTag(h, 1)
 			in >> t;
 			OffsetTable.push_back(t);
 		}
-		in >> t;
-		CodeTableOffset=t;
+		//CodeTableOffset is missing with zero NumGlyphs
+		if(NumGlyphs)
+		{
+			in >> t;
+			CodeTableOffset=t;
+		}
 	}
 	GlyphShapeTable.resize(NumGlyphs);
 	for(int i=0;i<NumGlyphs;i++)
@@ -615,7 +634,7 @@ DefineFont4Tag::DefineFont4Tag(RECORDHEADER h, std::istream& in):DictionaryTag(h
 	ignore(in,dest-in.tellg());
 }
 
-DefineBitsLosslessTag::DefineBitsLosslessTag(RECORDHEADER h, istream& in):DictionaryTag(h)
+DefineBitsLosslessTag::DefineBitsLosslessTag(RECORDHEADER h, istream& in, int version):DictionaryTag(h)
 {
 	int dest=in.tellg();
 	dest+=h.getLength();
@@ -624,40 +643,46 @@ DefineBitsLosslessTag::DefineBitsLosslessTag(RECORDHEADER h, istream& in):Dictio
 	if(BitmapFormat==3)
 		in >> BitmapColorTableSize;
 
-	//TODO: read bitmap data
-	ignore(in,dest-in.tellg());
-}
-
-DefineBitsLossless2Tag::DefineBitsLossless2Tag(RECORDHEADER h, istream& in):DictionaryTag(h)
-{
-	int dest=in.tellg();
-	dest+=h.getLength();
-	in >> CharacterId >> BitmapFormat >> BitmapWidth >> BitmapHeight;
-
-	if(BitmapFormat==3)
-		in >> BitmapColorTableSize;
-
-	//TODO: read bitmap data
-	ignore(in,dest-in.tellg());
-}
-
-ASObject* DefineBitsLossless2Tag::instance() const
-{
-	DefineBitsLossless2Tag* ret=new DefineBitsLossless2Tag(*this);
-	//TODO: check
-	if(bindedTo)
+	if(BitmapFormat != 5)
 	{
-		//A class is binded to this tag
-		ret->setPrototype(bindedTo);
+		LOG(LOG_NOT_IMPLEMENTED,"DefineBitsLossless(2)Tag with unsupported BitmapFormat");
+		ignore(in,dest-in.tellg());
+		return;
 	}
-	else
-		ret->setPrototype(Class<Bitmap>::getClass());
-	return ret;
+
+	string cData;
+	size_t cSize = dest-in.tellg(); //rest of this tag
+	cData.resize(cSize);
+	in.read(&cData[0], cSize);
+	istringstream cDataStream(cData);
+	zlib_filter zf(cDataStream.rdbuf());
+	istream zfstream(&zf);
+
+	size_t size = BitmapWidth * BitmapHeight * 4;
+	uint8_t* inData=new(nothrow) uint8_t[size];
+	zfstream.read((char*)inData,size);
+	assert(!zfstream.fail() && !zfstream.eof());
+
+	if(version == 1)
+	{	/* for version 1, the alpha field is always zero
+		 * but should not be interpreted. Setting it to
+		 * 0xff (opaque) allows us to handle it as ARGB
+		 */
+		for(size_t i=0;i<size;i+=4)
+			inData[i] = 0xFF;
+	}
+
+	Bitmap::fromRGB(inData, BitmapWidth, BitmapHeight, true);
 }
 
-void DefineBitsLossless2Tag::Render(bool maskEnabled)
+ASObject* DefineBitsLosslessTag::instance() const
 {
-	LOG(LOG_NOT_IMPLEMENTED,"DefineBitsLossless2Tag::Render");
+	DefineBitsLosslessTag* ret=new DefineBitsLosslessTag(*this);
+	if(bindedTo)
+		ret->setClass(bindedTo);
+	else
+		ret->setClass(Class<Bitmap>::getClass());
+	return ret;
 }
 
 DefineTextTag::DefineTextTag(RECORDHEADER h, istream& in, int v):DictionaryTag(h),version(v)
@@ -688,7 +713,7 @@ ASObject* DefineTextTag::instance() const
 		computeCached();
 
 	StaticText* ret=new StaticText(tokens);
-	ret->setPrototype(Class<StaticText>::getClass());
+	ret->setClass(Class<StaticText>::getClass());
 	return ret;
 }
 
@@ -709,8 +734,8 @@ void DefineTextTag::computeCached() const
 	{
 		if(TextRecords[i].StyleFlagsHasFont)
 		{
-			DictionaryTag* it3=loadedFrom->dictionaryLookup(TextRecords[i].FontID);
-			curFont=dynamic_cast<FontTag*>(it3);
+			_R<DictionaryTag> it3=loadedFrom->dictionaryLookup(TextRecords[i].FontID);
+			curFont=dynamic_cast<FontTag*>(it3.getPtr());
 			assert_and_throw(curFont);
 		}
 		assert_and_throw(curFont);
@@ -793,11 +818,11 @@ ASObject* DefineMorphShapeTag::instance() const
 {
 	DefineMorphShapeTag* ret=new DefineMorphShapeTag(*this);
 	assert_and_throw(bindedTo==NULL);
-	ret->setPrototype(Class<MorphShape>::getClass());
+	ret->setClass(Class<MorphShape>::getClass());
 	return ret;
 }
 
-void DefineMorphShapeTag::Render(bool maskEnabled)
+void DefineMorphShapeTag::renderImpl(bool maskEnabled, number_t t1,number_t t2,number_t t3,number_t t4) const
 {
 	if(alpha==0)
 		return;
@@ -814,24 +839,25 @@ void DefineMorphShapeTag::Render(bool maskEnabled)
 
 	float matrix[16];
 	Matrix.get4DMatrix(matrix);
-	glPushMatrix();
-	glMultMatrixf(matrix);
+	lsglPushMatrix();
+	lsglMultMatrixf(matrix);
+	setMatrixUniform(LSGL_MODELVIEW);
 
-	rt->glAcquireFramebuffer();
+	rt->acquireFramebuffer();
 
 	std::vector < GeomShape >::iterator it=shapes.begin();
 	for(;it!=shapes.end();++it)
 		it->Render();
 
-	rt->glBlitFramebuffer();
-	if(rt->glAcquireIdBuffer())
+	rt->blitFramebuffer();
+	if(rt->acquireIdBuffer())
 	{
 		std::vector < GeomShape >::iterator it=shapes.begin();
 		for(;it!=shapes.end();++it)
 			it->Render();
-		rt->glReleaseIdBuffer();
+		rt->releaseIdBuffer();
 	}
-	glPopMatrix();*/
+	lsglPopMatrix();*/
 }
 
 //void DefineFont3Tag::genGlyphShape(vector<GeomShape>& s, int glyph)
@@ -947,7 +973,7 @@ void PlaceObject2Tag::setProperties(DisplayObject* obj, DisplayObjectContainer*
 	if(PlaceFlagHasName)
 	{
 		//Set a variable on the parent to link this object
-		LOG(LOG_NO_INFO,_("Registering ID ") << CharacterId << _(" with name ") << Name);
+		LOG(LOG_INFO,_("Registering ID ") << CharacterId << _(" with name ") << Name);
 		if(!PlaceFlagMove)
 		{
 			obj->name = (const char*)Name;
@@ -984,15 +1010,14 @@ void PlaceObject2Tag::execute(DisplayObjectContainer* parent)
 			localRoot=parentDict->loadedFrom;
 		else
 			localRoot=parent->getRoot().getPtr();
-		DictionaryTag* dict=localRoot->dictionaryLookup(CharacterId);
+		_R<DictionaryTag> dict=localRoot->dictionaryLookup(CharacterId);
 
 		//We can create the object right away
 		DisplayObject* toAdd=dynamic_cast<DisplayObject*>(dict->instance());
+
 		assert_and_throw(toAdd);
 		//The matrix must be set before invoking the constructor
 		toAdd->setMatrix(Matrix);
-		if(toAdd->getPrototype())
-			toAdd->getPrototype()->handleConstruction(toAdd,NULL,0,true);
 
 		setProperties(toAdd, parent);
 
@@ -1130,7 +1155,7 @@ ProductInfoTag::ProductInfoTag(RECORDHEADER h, std::istream& in):Tag(h)
 	longlongTime<<=32;
 	longlongTime|=CompileTimeLo;
 
-	LOG(LOG_NO_INFO,_("SWF Info:") << 
+	LOG(LOG_INFO,_("SWF Info:") << 
 	endl << "\tProductId:\t\t" << ProductId <<
 	endl << "\tEdition:\t\t" << Edition <<
 	endl << "\tVersion:\t\t" << int(MajorVersion) << "." << int(MinorVersion) << "." << MajorBuild << "." << MinorBuild <<
@@ -1148,9 +1173,8 @@ FrameLabelTag::FrameLabelTag(RECORDHEADER h, std::istream& in):Tag(h)
 	}
 }
 
-DefineButton2Tag::DefineButton2Tag(RECORDHEADER h, std::istream& in):DictionaryTag(h),IdleToOverUp(false)
+DefineButton2Tag::DefineButton2Tag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
 {
-	state=BUTTON_UP;
 	in >> ButtonId;
 	BitStream bs(in);
 	UB(7,bs);
@@ -1162,83 +1186,78 @@ DefineButton2Tag::DefineButton2Tag(RECORDHEADER h, std::istream& in):DictionaryT
 	do
 	{
 		in >> br;
+		if(br.isNull())
+			break;
 		Characters.push_back(br);
 	}
-	while(!br.isNull());
+	while(true);
 
 	if(ActionOffset)
-	{
-		BUTTONCONDACTION bca;
-		do
-		{
-			in >> bca;
-			Actions.push_back(bca);
-		}
-		while(!bca.isLast());
-	}
+		LOG(LOG_NOT_IMPLEMENTED,"DefineButton2Tag: Actions are not supported");
 }
 
 ASObject* DefineButton2Tag::instance() const
 {
-	return new DefineButton2Tag(*this);
-}
+	DisplayObject* states[4] = {NULL, NULL, NULL, NULL};
+	bool isSprite[4] = {false, false, false, false};
+	uint32_t curDepth[4];
 
-void DefineButton2Tag::handleEvent(Event* e)
-{
-	state=BUTTON_OVER;
-	IdleToOverUp=true;
-}
+	/* There maybe multiple DisplayObjects for one state. The official
+	 * implementation seems to create a Sprite in that case with
+	 * all DisplayObjects as children.
+	 */
 
-void DefineButton2Tag::Render(bool maskEnabled)
-{
-	LOG(LOG_NOT_IMPLEMENTED,_("DefineButton2Tag::Render"));
-	if(alpha==0)
-		return;
-	if(!visible)
-		return;
-	/*	for(unsigned int i=0;i<Characters.size();i++)
+	auto i = Characters.begin();
+	for(;i != Characters.end(); ++i)
 	{
-		if(Characters[i].ButtonStateUp && state==BUTTON_UP)
+		for(int j=0;j<4;++j)
 		{
+			if(j==0 && !i->ButtonStateDown)
+				continue;
+			if(j==1 && !i->ButtonStateHitTest)
+				continue;
+			if(j==2 && !i->ButtonStateOver)
+				continue;
+			if(j==3 && !i->ButtonStateUp)
+				continue;
+			_R<DictionaryTag> dict=loadedFrom->dictionaryLookup(i->CharacterID);
+
+			//We can create the object right away
+			DisplayObject* state=dynamic_cast<DisplayObject*>(dict->instance());
+			assert_and_throw(state);
+			//The matrix must be set before invoking the constructor
+			state->setMatrix(i->PlaceMatrix);
+			/*
+			 * TODO: BlendMode, filerList, PlaceDepth, ColorTransfrom
+			 */
+			if(states[j] == NULL)
+			{
+				states[j] = state;
+				curDepth[j] = i->PlaceDepth;
+			}
+			else
+			{
+				if(!isSprite[j])
+				{
+					Sprite* spr = Class<Sprite>::getInstanceS();
+					spr->insertLegacyChildAt(curDepth[j],states[j]);
+					states[j] = spr;
+					spr->name = "Button_spr";
+				}
+				Sprite* spr = Class<Sprite>::cast(states[j]);
+				spr->insertLegacyChildAt(i->PlaceDepth,state);
+			}
 		}
-		else if(Characters[i].ButtonStateOver && state==BUTTON_OVER)
-		{
-		}
-		else
-			continue;
-		DictionaryTag* r=sys->dictionaryLookup(Characters[i].CharacterID);
-		IRenderObject* c=dynamic_cast<IRenderObject*>(r);
-		if(c==NULL)
-			LOG(ERROR,_("Rendering something strange"));
-		float matrix[16];
-		Characters[i].PlaceMatrix.get4DMatrix(matrix);
-		glPushMatrix();
-		glMultMatrixf(matrix);
-		c->Render();
-		glPopMatrix();
 	}
-	for(unsigned int i=0;i<Actions.size();i++)
-	{
-		if(IdleToOverUp && Actions[i].CondIdleToOverUp)
-		{
-			IdleToOverUp=false;
-		}
-		else
-			continue;
 
-		for(unsigned int j=0;j<Actions[i].Actions.size();j++)
-			Actions[i].Actions[j]->Execute();
-	}*/
-}
-
-bool DefineButton2Tag::getBounds(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const
-{
-	return false;
+	SimpleButton* ret=new SimpleButton(states[0], states[1], states[2], states[3]);
+	ret->setClass(Class<SimpleButton>::getClass());
+	return ret;
 }
 
 DefineVideoStreamTag::DefineVideoStreamTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
 {
-	LOG(LOG_NO_INFO,_("DefineVideoStreamTag"));
+	LOG(LOG_INFO,_("DefineVideoStreamTag"));
 	in >> CharacterID >> NumFrames >> Width >> Height;
 	BitStream bs(in);
 	UB(4,bs);
@@ -1249,14 +1268,14 @@ DefineVideoStreamTag::DefineVideoStreamTag(RECORDHEADER h, std::istream& in):Dic
 
 ASObject* DefineVideoStreamTag::instance() const
 {
-	DefineVideoStreamTag* ret=new DefineVideoStreamTag(*this);
+	Video* ret=new Video(Width, Height);
 	if(bindedTo)
 	{
 		//A class is binded to this tag
-		ret->setPrototype(bindedTo);
+		ret->setClass(bindedTo);
 	}
 	else
-		ret->setPrototype(Class<Video>::getClass());
+		ret->setClass(Class<Video>::getClass());
 	return ret;
 }
 
@@ -1286,6 +1305,8 @@ FileAttributesTag::FileAttributesTag(RECORDHEADER h, std::istream& in):Tag(h)
 
 	if(ActionScript3)
 		pt->useAVM2=true;
+
+	pt->useNetwork = UseNetwork;
 }
 
 DefineSoundTag::DefineSoundTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
@@ -1304,23 +1325,15 @@ DefineSoundTag::DefineSoundTag(RECORDHEADER h, std::istream& in):DictionaryTag(h
 
 ASObject* DefineSoundTag::instance() const
 {
-	DefineSoundTag* ret=new DefineSoundTag(*this);
-	//TODO: check
-	if(bindedTo)
-	{
-		//A class is binded to this tag
-		ret->setPrototype(bindedTo);
-	}
-	else
-		ret->setPrototype(Class<Sound>::getClass());
-	return ret;
+	//TODO: use the tag sound data
+	return Class<Sound>::getInstanceS();
 }
 
-ScriptLimitsTag::ScriptLimitsTag(RECORDHEADER h, std::istream& in):Tag(h)
+ScriptLimitsTag::ScriptLimitsTag(RECORDHEADER h, std::istream& in):ControlTag(h)
 {
 	LOG(LOG_TRACE,_("ScriptLimitsTag Tag"));
 	in >> MaxRecursionDepth >> ScriptTimeoutSeconds;
-	LOG(LOG_NO_INFO,_("MaxRecursionDepth: ") << MaxRecursionDepth << _(", ScriptTimeoutSeconds: ") << ScriptTimeoutSeconds);
+	LOG(LOG_INFO,_("MaxRecursionDepth: ") << MaxRecursionDepth << _(", ScriptTimeoutSeconds: ") << ScriptTimeoutSeconds);
 }
 
 DebugIDTag::DebugIDTag(RECORDHEADER h, std::istream& in):Tag(h)
@@ -1330,7 +1343,7 @@ DebugIDTag::DebugIDTag(RECORDHEADER h, std::istream& in):Tag(h)
 		in >> DebugId[i];
 
 	//Note the switch to hex formatting on the ostream, and switch back to dec
-	LOG(LOG_NO_INFO,_("DebugId ") << hex <<
+	LOG(LOG_INFO,_("DebugId ") << hex <<
 		int(DebugId[0]) << int(DebugId[1]) << int(DebugId[2]) << int(DebugId[3]) << "-" <<
 		int(DebugId[4]) << int(DebugId[5]) << "-" <<
 		int(DebugId[6]) << int(DebugId[7]) << "-" <<
@@ -1345,7 +1358,7 @@ EnableDebuggerTag::EnableDebuggerTag(RECORDHEADER h, std::istream& in):Tag(h)
 	DebugPassword = "";
 	if(h.getLength() > 0)
 		in >> DebugPassword;
-	LOG(LOG_NO_INFO,_("Debugger enabled, password: ") << DebugPassword);
+	LOG(LOG_INFO,_("Debugger enabled, password: ") << DebugPassword);
 }
 
 EnableDebugger2Tag::EnableDebugger2Tag(RECORDHEADER h, std::istream& in):Tag(h)
@@ -1356,7 +1369,7 @@ EnableDebugger2Tag::EnableDebugger2Tag(RECORDHEADER h, std::istream& in):Tag(h)
 	DebugPassword = "";
 	if(h.getLength() > sizeof(ReservedWord))
 		in >> DebugPassword;
-	LOG(LOG_NO_INFO,_("Debugger enabled, reserved: ") << ReservedWord << _(", password: ") << DebugPassword);
+	LOG(LOG_INFO,_("Debugger enabled, reserved: ") << ReservedWord << _(", password: ") << DebugPassword);
 }
 
 MetadataTag::MetadataTag(RECORDHEADER h, std::istream& in):Tag(h)
@@ -1373,7 +1386,7 @@ MetadataTag::MetadataTag(RECORDHEADER h, std::istream& in):Tag(h)
 		if(xml.get_depth() == 2 && xml.get_node_type() == xmlpp::TextReader::Element)
 			output << endl << "\t" << xml.get_local_name() << ":\t\t" << xml.read_string();
 	}
-	LOG(LOG_NO_INFO, "SWF Metadata:" << output.str());
+	LOG(LOG_INFO, "SWF Metadata:" << output.str());
 }
 
 DefineBitsTag::DefineBitsTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
@@ -1404,6 +1417,16 @@ DefineBitsJPEG2Tag::DefineBitsJPEG2Tag(RECORDHEADER h, std::istream& in):Diction
 	delete[] inData;
 }
 
+ASObject* DefineBitsJPEG2Tag::instance() const
+{
+	DefineBitsJPEG2Tag* ret=new DefineBitsJPEG2Tag(*this);
+	if(bindedTo)
+		ret->setClass(bindedTo);
+	else
+		ret->setClass(Class<Bitmap>::getClass());
+	return ret;
+}
+
 DefineBitsJPEG3Tag::DefineBitsJPEG3Tag(RECORDHEADER h, std::istream& in):DictionaryTag(h),alphaData(NULL)
 {
 	LOG(LOG_TRACE,_("DefineBitsJPEG3Tag Tag"));
@@ -1427,6 +1450,16 @@ DefineBitsJPEG3Tag::DefineBitsJPEG3Tag(RECORDHEADER h, std::istream& in):Diction
 	}
 }
 
+ASObject* DefineBitsJPEG3Tag::instance() const
+{
+	DefineBitsJPEG3Tag* ret=new DefineBitsJPEG3Tag(*this);
+	if(bindedTo)
+		ret->setClass(bindedTo);
+	else
+		ret->setClass(Class<Bitmap>::getClass());
+	return ret;
+}
+
 DefineBitsJPEG3Tag::~DefineBitsJPEG3Tag()
 {
 	delete[] alphaData;
