@@ -34,11 +34,14 @@ uint64_t ABCVm::profilingCheckpoint(uint64_t& startTime)
 	return ret;
 }
 
-ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* context)
+ASObject* ABCVm::executeFunction(const SyntheticFunction* function, call_context* context)
 {
 	method_info* mi=function->mi;
 
-	istringstream& code=*(context->code);
+	istringstream code(mi->body->code);
+	//This may be non-zero and point to the position of an exception handler
+	code.seekg(context->exec_pos);
+
 	int code_len=code.str().length();
 
 	u8 opcode;
@@ -48,10 +51,10 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 		mi->profTime.resize(code_len,0);
 	uint64_t startTime=compat_get_thread_cputime_us();
 #define PROF_ACCOUNT_TIME(a, b)  do{a+=b;}while(0)
-#define PROF_IGNORE_TIME(a) a
+#define PROF_IGNORE_TIME(a) do{ a; } while(0)
 #else
-#define PROF_ACCOUNT_TIME(a, b)
-#define PROF_IGNORE_TIME(a)
+#define PROF_ACCOUNT_TIME(a, b) do{ ; }while(0)
+#define PROF_IGNORE_TIME(a) do{ ; } while(0)
 #endif
 
 	//Each case block builds the correct parameters for the interpreter function and call it
@@ -62,8 +65,10 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 #endif
 		code >> opcode;
 		if(code.eof())
-			throw ParseException("End of code in intepreter");
+			throw ParseException("End of code in interpreter");
 
+		//Save ip for exception handling in SyntheticFunction::callImpl
+		context->exec_pos = code.tellg();
 		switch(opcode)
 		{
 			case 0x02:
@@ -123,7 +128,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -143,7 +148,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -163,7 +168,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -183,7 +188,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -199,7 +204,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 				//Now 'jump' to the destination, validate on the length
 				if(dest >= code_len)
-					throw ParseException("Jump out of bounds in intepreter");
+					throw ParseException("Jump out of bounds in interpreter");
 				code.seekg(dest);
 				break;
 			}
@@ -218,7 +223,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -238,7 +243,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -259,7 +264,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -280,7 +285,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -301,7 +306,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -322,7 +327,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -343,7 +348,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -364,7 +369,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -385,7 +390,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -406,7 +411,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 
 					//Now 'jump' to the destination, validate on the length
 					if(dest >= code_len)
-						throw ParseException("Jump out of bounds in intepreter");
+						throw ParseException("Jump out of bounds in interpreter");
 					code.seekg(dest);
 				}
 				break;
@@ -434,11 +439,11 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				index_obj->decRef();
 
 				int dest=defaultdest;
-				if(index>=0 && index<=count)
+				if(index<=count)
 					dest=here+offsets[index];
 
 				if(dest >= code_len)
-					throw ParseException("Jump out of bounds in intepreter");
+					throw ParseException("Jump out of bounds in interpreter");
 				code.seekg(dest);
 				break;
 			}
@@ -622,7 +627,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				code >> t;
 				method_info* called_mi=NULL;
 				PROF_ACCOUNT_TIME(mi->profTime[instructionPointer],profilingCheckpoint(startTime));
-				call(context,t,called_mi);
+				call(context,t,&called_mi);
 				if(called_mi)
 					PROF_ACCOUNT_TIME(mi->profCalls[called_mi],profilingCheckpoint(startTime));
 				else
@@ -645,7 +650,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				code >> t2;
 				method_info* called_mi=NULL;
 				PROF_ACCOUNT_TIME(mi->profTime[instructionPointer],profilingCheckpoint(startTime));
-				callSuper(context,t,t2,called_mi);
+				callSuper(context,t,t2,&called_mi,true);
 				if(called_mi)
 					PROF_ACCOUNT_TIME(mi->profCalls[called_mi],profilingCheckpoint(startTime));
 				else
@@ -661,7 +666,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				code >> t2;
 				method_info* called_mi=NULL;
 				PROF_ACCOUNT_TIME(mi->profTime[instructionPointer],profilingCheckpoint(startTime));
-				callProperty(context,t,t2,called_mi);
+				callProperty(context,t,t2,&called_mi,true);
 				if(called_mi)
 					PROF_ACCOUNT_TIME(mi->profCalls[called_mi],profilingCheckpoint(startTime));
 				else
@@ -708,7 +713,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				code >> t2;
 				method_info* called_mi=NULL;
 				PROF_ACCOUNT_TIME(mi->profTime[instructionPointer],profilingCheckpoint(startTime));
-				callSuperVoid(context,t,t2,called_mi);
+				callSuper(context,t,t2,&called_mi,false);
 				if(called_mi)
 					PROF_ACCOUNT_TIME(mi->profCalls[called_mi],profilingCheckpoint(startTime));
 				else
@@ -723,7 +728,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				code >> t2;
 				method_info* called_mi=NULL;
 				PROF_ACCOUNT_TIME(mi->profTime[instructionPointer],profilingCheckpoint(startTime));
-				callPropVoid(context,t,t2,called_mi);
+				callProperty(context,t,t2,&called_mi,false);
 				if(called_mi)
 					PROF_ACCOUNT_TIME(mi->profCalls[called_mi],profilingCheckpoint(startTime));
 				else
@@ -789,7 +794,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				//findpropstrict
 				u30 t;
 				code >> t;
-				context->runtime_stack_push(findPropStrict(context,t));
+				context->runtime_stack_push(findPropStrict(context,context->context->getMultiname(t,context)));
 				break;
 			}
 			case 0x5e:
@@ -797,7 +802,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				//findproperty
 				u30 t;
 				code >> t;
-				context->runtime_stack_push(findProperty(context,t));
+				context->runtime_stack_push(findProperty(context,context->context->getMultiname(t,context)));
 				break;
 			}
 			case 0x60:
@@ -829,7 +834,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				code >> i;
 				assert_and_throw(context->locals[i]);
 				context->locals[i]->incRef();
-				LOG(LOG_CALLS, _("getLocal ") << i << _(": ") << context->locals[i]->toString(true) );
+				LOG(LOG_CALLS, _("getLocal ") << i << _(": ") << context->locals[i]->toDebugString() );
 				context->runtime_stack_push(context->locals[i]);
 				break;
 			}
@@ -879,7 +884,10 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				//initproperty
 				u30 t;
 				code >> t;
-				initProperty(context,t);
+				ASObject* value=context->runtime_stack_pop();
+			        multiname* name=context->context->getMultiname(t,context);
+			        ASObject* obj=context->runtime_stack_pop();
+				initProperty(obj,value,name);
 				break;
 			}
 			case 0x6a:
@@ -887,7 +895,10 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				//deleteproperty
 				u30 t;
 				code >> t;
-				deleteProperty(context, t);
+				multiname* name = context->context->getMultiname(t,context);
+				ASObject* obj=context->runtime_stack_pop();
+				bool ret = deleteProperty(obj,name);
+				context->runtime_stack_push(abstract_b(ret));
 				break;
 			}
 			case 0x6c:
@@ -919,7 +930,22 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				context->runtime_stack_push(convert_s(val));
 				break;
 			}
-			case 0x73:
+			case 0x71:
+			{
+				//FIXME: Properly escape as described in ECMA-357 section 10.2
+				//esc_xelem
+				ASObject* val=context->runtime_stack_pop();
+				context->runtime_stack_push(convert_s(val));
+				break;
+			}
+			case 0x72:
+			{
+				//FIXME: Properly escape as described in ECMA-357 section 10.2
+				//esc_xattr
+				ASObject* val=context->runtime_stack_pop();
+				context->runtime_stack_push(convert_s(val));
+				break;
+			}case 0x73:
 			{
 				//convert_i
 				ASObject* val=context->runtime_stack_pop();
@@ -1343,7 +1369,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				//getlocal_n
 				int i=opcode&3;
 				assert_and_throw(context->locals[i]);
-				LOG(LOG_CALLS, _("getLocal ") << i << _(": ") << context->locals[i]->toString(true) );
+				LOG(LOG_CALLS, _("getLocal ") << i << _(": ") << context->locals[i]->toDebugString() );
 				context->locals[i]->incRef();
 				context->runtime_stack_push(context->locals[i]);
 				break;
@@ -1393,7 +1419,7 @@ ASObject* ABCVm::executeFunction(SyntheticFunction* function, call_context* cont
 				break;
 			}
 			default:
-				LOG(LOG_ERROR,_("Not intepreted instruction @") << code.tellg());
+				LOG(LOG_ERROR,_("Not interpreted instruction @") << code.tellg());
 				LOG(LOG_ERROR,_("dump ") << hex << (unsigned int)opcode << dec);
 				throw ParseException("Not implemented instruction in interpreter");
 		}
