@@ -23,28 +23,24 @@
 
 using namespace lightspark;
 
-ASObject* Class<ASObject>::getVariableByMultiname(const multiname& name, bool skip_impl, ASObject* base)
+ASObject* lightspark::new_asobject()
 {
-	ASObject* ret=ASObject::getVariableByMultiname(name, skip_impl, base);
-	//No super here, ever
-	if(!ret)
-	{
-		//Check if we should do lazy definition
-		if(name.name_s=="toString")
-		{
-			ASObject* ret=Class<IFunction>::getFunction(ASObject::_toString);
-			setVariableByQName("toString","",ret);
-			return ret;
-		}
-		else if(name.name_s=="hasOwnProperty")
-		{
-			ASObject* ret=Class<IFunction>::getFunction(ASObject::hasOwnProperty);
-			setVariableByQName("hasOwnProperty","",ret);
-			return ret;
-		}
+	return Class<ASObject>::getInstanceS();
+}
 
+ASObject* Class<ASObject>::lazyDefine(const multiname& name)
+{
+	if(name.ns.empty())
+		return NULL;
+
+	if(binary_search(name.ns.begin(),name.ns.end(),nsNameAndKind(AS3,NAMESPACE)) && name.name_s=="hasOwnProperty")
+	{
+		ASObject* ret=Class<IFunction>::getFunction(ASObject::hasOwnProperty);
+		setVariableByQName("hasOwnProperty",AS3,ret,BORROWED_TRAIT);
+		return ret;
 	}
-	return ret;
+	else
+		return NULL;
 }
 
 void Class_inherit::finalize()
@@ -68,8 +64,8 @@ ASObject* Class_inherit::getInstance(bool construct, ASObject* const* args, cons
 		//Our super should not construct, we are going to do it ourselves
 		ret=super->getInstance(false,NULL,0);
 	}
-	//We override the prototype
-	ret->setPrototype(this);
+	//We override the classdef
+	ret->setClass(this);
 	if(construct)
 		handleConstruction(ret,args,argslen,true);
 	return ret;
