@@ -21,18 +21,20 @@
 #include <pthread.h>
 #include <algorithm>
 #include "scripting/abc.h"
-#include "scripting/flashdisplay.h"
-#include "scripting/flashevents.h"
-#include "scripting/flashutils.h"
+#include "scripting/flash/events/flashevents.h"
+#include "scripting/flash/utils/flashutils.h"
 #include "swf.h"
 #include "logger.h"
 #include "parsing/streams.h"
 #include "asobject.h"
 #include "scripting/class.h"
+#include "backends/audio.h"
+#include "backends/config.h"
+#include "backends/pluginmanager.h"
 #include "backends/rendering.h"
+#include "backends/security.h"
+#include "backends/image.h"
 
-#include "SDL.h"
-#include <GL/glew.h>
 #ifdef ENABLE_CURL
 #include <curl/curl.h>
 #endif
@@ -45,41 +47,30 @@ extern "C" {
 }
 #endif
 
-#ifdef COMPILE_PLUGIN
 #include <gdk/gdkx.h>
-#endif
 
 using namespace std;
 using namespace lightspark;
 
 extern TLSDATA ParseThread* pt;
 
-RootMovieClip::RootMovieClip(LoaderInfo* li, bool isSys):mutex("mutexRoot"),initialized(false),parsingIsFailed(false),frameRate(0),
-	toBind(false),mutexChildrenClips("mutexChildrenClips")
+RootMovieClip::RootMovieClip(LoaderInfo* li, bool isSys):mutex("mutexRoot"),parsingIsFailed(false),frameRate(0),
+	toBind(false), finishedLoading(false)
 {
-	this->incRef();
-	sem_init(&new_frame,0,0);
 	if(li)
 		li->incRef();
 	loaderInfo=_MNR(li);
 
 	//We set the protoype to a generic MovieClip
 	if(!isSys)
-		setPrototype(Class<MovieClip>::getClass());
+		setClass(Class<MovieClip>::getClass());
 }
 
-RootMovieClip::~RootMovieClip()
-{
-	if(this!=sys)
-		sys->removeJob(this);
-	sem_destroy(&new_frame);
-}
 
 void RootMovieClip::parsingFailed()
 {
 	//The parsing is failed, we have no change to be ever valid
 	parsingIsFailed=true;
-	sem_post(&new_frame);
 }
 
 void RootMovieClip::bindToName(const tiny_string& n)
@@ -106,38 +97,17 @@ void RootMovieClip::setOrigin(const tiny_string& u, const tiny_string& filename)
 	}
 }
 
-void RootMovieClip::registerChildClip(MovieClip* clip)
+void SystemState::registerFrameListener(_R<DisplayObject> obj)
 {
-	Locker l(mutexChildrenClips);
-	clip->incRef();
-	childrenClips.insert(clip);
-}
-
-void RootMovieClip::unregisterChildClip(MovieClip* clip)
-{
-	Locker l(mutexChildrenClips);
-	if(childrenClips.erase(clip))
-		clip->decRef();
-}
-
-void SystemState::registerEnterFrameListener(DisplayObject* obj)
-{
-	Locker l(mutexEnterFrameListeners);
+	Locker l(mutexFrameListeners);
 	obj->incRef();
-	enterFrameListeners.insert(obj);
+	frameListeners.insert(obj);
 }
 
-void SystemState::unregisterEnterFrameListener(DisplayObject* obj)
+void SystemState::unregisterFrameListener(_R<DisplayObject> obj)
 {
-	Locker l(mutexEnterFrameListeners);
-	if(enterFrameListeners.erase(obj))
-		obj->decRef();
-}
-
-void SystemState::registerTag(Tag* t)
-{
-	SpinlockLocker l(tagsStorageLock);
-	tagsStorage.push_back(t);
+	Locker l(mutexFrameListeners);
+	frameListeners.erase(obj);
 }
 
 void RootMovieClip::setOnStage(bool staged)
@@ -148,7 +118,7 @@ void RootMovieClip::setOnStage(bool staged)
 RootMovieClip* RootMovieClip::getInstance(LoaderInfo* li)
 {
 	RootMovieClip* ret=new RootMovieClip(li);
-	ret->setPrototype(Class<MovieClip>::getClass());
+	ret->setClass(Class<MovieClip>::getClass());
 	return ret;
 }
 
@@ -169,6 +139,8 @@ void SystemState::staticInit()
 
 void SystemState::staticDeinit()
 {
+	delete Type::anyType;
+	delete Type::voidType;
 #ifdef ENABLE_CURL
 	curl_global_cleanup();
 #endif
@@ -176,11 +148,11 @@ void SystemState::staticDeinit()
 
 SystemState::SystemState(ParseThread* parseThread, uint32_t fileSize):
 	RootMovieClip(NULL,true),renderRate(0),error(false),shutdown(false),
-	renderThread(NULL),inputThread(NULL),engine(NONE),fileDumpAvailable(0),
+	renderThread(NULL),inputThread(NULL),engineData(NULL),fileDumpAvailable(0),
 	waitingForDump(false),vmVersion(VMNONE),childPid(0),useGnashFallback(false),
-	parameters(NullRef),mutexEnterFrameListeners("mutexEnterFrameListeners"),
+	parameters(NullRef),mutexFrameListeners("mutexFrameListeners"),
 	invalidateQueueHead(NullRef),invalidateQueueTail(NullRef),showProfilingData(false),
-	currentVm(NULL),useInterpreter(true),useJit(false),downloadManager(NULL),
+	currentVm(NULL),useInterpreter(true),useJit(false),exitOnError(false),downloadManager(NULL),
 	extScriptObject(NULL),scaleMode(SHOW_ALL)
 {
 	cookiesFileName[0]=0;
@@ -190,7 +162,7 @@ SystemState::SystemState(ParseThread* parseThread, uint32_t fileSize):
 
 	//Get starting time
 	if(parseThread) //ParseThread may be null in tightspark
-		parseThread->root=this;
+		parseThread->setRootMovie(this);
 	threadPool=new ThreadPool(this);
 	timerThread=new TimerThread(this);
 	config=new Config;
@@ -204,14 +176,14 @@ SystemState::SystemState(ParseThread* parseThread, uint32_t fileSize):
 	loaderInfo->setBytesLoaded(fileSize);
 	loaderInfo->setBytesTotal(fileSize);
 	stage=Class<Stage>::getInstanceS();
-	stage->incRef();
-	setParent(_MR(stage));
+	this->incRef();
+	stage->_addChildAt(_MR(this),0);
 	startTime=compat_msectiming();
 	
-	setPrototype(Class<MovieClip>::getClass());
+	setClass(Class<MovieClip>::getClass());
 
 	//Override getStage as for SystemState that can't be null
-	setGetterByQName("stage","",Class<IFunction>::getFunction(_getStage),false);
+	setDeclaredMethodByQName("stage","",Class<IFunction>::getFunction(_getStage),GETTER_METHOD,false);
 
 	renderThread=new RenderThread(this);
 	inputThread=new InputThread(this);
@@ -230,6 +202,18 @@ void SystemState::setCookies(const char* c)
 	rawCookies=c;
 }
 
+static int hexToInt(char c)
+{
+	if(c>='0' && c<='9')
+		return c-'0';
+	else if(c>='a' && c<='f')
+		return c-'a'+10;
+	else if(c>='A' && c<='F')
+		return c-'A'+10;
+	else
+		return -1;
+}
+
 void SystemState::parseParametersFromFlashvars(const char* v)
 {
 	if(useGnashFallback) //Save a copy of the string
@@ -238,6 +222,10 @@ void SystemState::parseParametersFromFlashvars(const char* v)
 	//Add arguments to SystemState
 	string vars(v);
 	uint32_t cur=0;
+	char* pfile = getenv("LIGHTSPARK_PLUGIN_PARAMFILE");
+        ofstream f;
+	if(pfile)
+		f.open(pfile);
 	while(cur<vars.size())
 	{
 		int n1=vars.find('=',cur);
@@ -266,8 +254,8 @@ void SystemState::parseParametersFromFlashvars(const char* v)
 					break;
 				}
 
-				int t1=Math::hexToInt(vars[j+1]);
-				int t2=Math::hexToInt(vars[j+2]);
+				int t1=hexToInt(vars[j+1]);
+				int t2=hexToInt(vars[j+2]);
 				if(t1==-1 || t2==-1)
 				{
 					ok=false;
@@ -283,8 +271,10 @@ void SystemState::parseParametersFromFlashvars(const char* v)
 		if(ok)
 		{
 			//cout << varName << ' ' << varValue << endl;
-			params->setVariableByQName(varName.c_str(),"",
-					lightspark::Class<lightspark::ASString>::getInstanceS(varValue));
+			if(pfile)
+				f << varName << endl << varValue << endl;
+			params->setVariableByQName(varName,"",
+					lightspark::Class<lightspark::ASString>::getInstanceS(varValue),DYNAMIC_TRAIT);
 		}
 		cur=n2+1;
 	}
@@ -306,7 +296,7 @@ void SystemState::parseParametersFromFile(const char* f)
 		getline(i,name);
 		getline(i,value);
 
-		ret->setVariableByQName(name,"",Class<ASString>::getInstanceS(value));
+		ret->setVariableByQName(name,"",Class<ASString>::getInstanceS(value),DYNAMIC_TRAIT);
 	}
 	setParameters(ret);
 	i.close();
@@ -316,7 +306,7 @@ void SystemState::setParameters(_R<ASObject> p)
 {
 	parameters=p;
 	p->incRef();
-	loaderInfo->setVariableByQName("parameters","",p.getPtr());
+	loaderInfo->setVariableByQName("parameters","",p.getPtr(), DECLARED_TRAIT);
 }
 
 _NR<ASObject> SystemState::getParameters() const
@@ -367,10 +357,15 @@ void SystemState::finalize()
 	invalidateQueueHead.reset();
 	invalidateQueueTail.reset();
 	parameters.reset();
+	frameListeners.clear();
 }
 
 SystemState::~SystemState()
 {
+}
+
+void SystemState::destroy()
+{
 #ifdef PROFILING_SUPPORT
 	saveProfilingInformation();
 #endif
@@ -386,60 +381,69 @@ SystemState::~SystemState()
 	assert(shutdown);
 
 	renderThread->stop();
+	/*
+	   Stop the downloads so that the thread pool does not keep waiting for data.
+	   Standalone downloader does not really need this as the downloading threads will
+	   be stopped with the whole thread pool, but in plugin mode this is necessary.
+	*/
+	if(downloadManager)
+		downloadManager->stopAll();
 	//The thread pool should be stopped before everything
 	if(threadPool)
 		threadPool->forceStop();
 	stopEngines();
 
+	delete intervalManager;
 	//Finalize ourselves
 	finalize();
 
-	//We are already being destroyed, make our prototype abandon us
-	setPrototype(NULL);
+	//We are already being destroyed, make our classdef abandon us
+	setClass(NULL);
 	
+	//Free the stage. This should free all objects on the displaylist
+	stage->decRef();
+	stage = NULL;
+
 	/*
-	   Now we have to kill all objects that are still alive. This is done is two passes
-	   1) call finalize on all objects, this will decRef all referenced objects
-	   2) delete all the objects. Now destroying an object should not cause accesses to
-	   	any other object */
+	 * 1) call finalize on all objects, this will free all referenced objects and thereby
+	 * cut circular references. After that, all ASObjects but classes and templates should
+	 * have been deleted through decRef. Else it is an error.
+	 * 2) decRef all the classes and templates to which we hold a reference through the
+	 * 'classes' and 'templates' maps.
+	 */
 
 	std::map<QName, Class_base*>::iterator it=classes.begin();
 	for(;it!=classes.end();++it)
 		it->second->finalize();
 
-	//Destroy the contents of all the classes
-	it=classes.begin();
-	for(;it!=classes.end();++it)
-	{
-		//Make sure classes survives their cleanUp
-		it->second->incRef();
-		it->second->cleanUp();
-	}
+	//Here we clean the events queue
+	if(currentVm)
+		currentVm->finalize();
+
+	//Free classes by decRef'ing them
+	for(auto i = classes.begin(); i != classes.end(); ++i)
+		i->second->decRef();
+
+	//Free templates by decRef'ing them
+	for(auto i = templates.begin(); i != templates.end(); ++i)
+		i->second->decRef();
 
-	//Destroy all registered classes
-	it=classes.begin();
-	for(;it!=classes.end();++it)
-	{
-		//DEPRECATED: to force garbage collection we delete all the classes
-		delete it->second;
-		//it->second->decRef()
-	}
 	//The Vm must be destroyed this late to clean all managed integers and numbers
+	//This deletes the {int,uint,number}_managers; therefore no Number/.. object may be
+	//decRef'ed after this line as it would cause a manager->put()
 	delete currentVm;
 
 	//Some objects needs to remove the jobs when destroyed so keep the timerThread until now
 	delete timerThread;
 	timerThread=NULL;
 
-	//Also destroy all tags
-	for(unsigned int i=0;i<tagsStorage.size();i++)
-		delete tagsStorage[i];
-
 	delete renderThread;
 	renderThread=NULL;
 	delete inputThread;
 	inputThread=NULL;
+	delete engineData;
 	sem_destroy(&terminated);
+	this->decRef(); //free a reference we obtained by 'new SystemState'
 }
 
 bool SystemState::isOnError() const
@@ -459,6 +463,8 @@ bool SystemState::shouldTerminate() const
 
 void SystemState::setError(const string& c)
 {
+	if(exitOnError)
+		exit(1);
 	//We record only the first error for easier fix and reporting
 	if(!error)
 	{
@@ -482,20 +488,13 @@ void SystemState::setShutdownFlag()
 	shutdown=true;
 
 	sem_post(&terminated);
+	if(standalone)
+		gtk_main_quit();
 }
 
 void SystemState::wait()
 {
 	sem_wait(&terminated);
-	if(engine==SDL)
-	{
-		SDL_Event event;
-		event.type = SDL_USEREVENT;
-		event.user.code = SHUTDOWN;
-		event.user.data1 = 0;
-		event.user.data1 = 0;
-		SDL_PushEvent(&event);
-	}
 	//Acquire the mutex to sure that the engines are not being started right now
 	Locker l(mutex);
 	renderThread->wait();
@@ -541,26 +540,39 @@ void SystemState::enableGnashFallback()
 	f.close();
 }
 
-#ifdef COMPILE_PLUGIN
-
 void SystemState::delayedCreation(SystemState* th)
 {
-	NPAPI_params& p=th->npapiParams;
+	EngineData* d=th->engineData;
 	//Create a plug in the XEmbed window
-	p.container=gtk_plug_new((GdkNativeWindow)p.window);
+	GtkWidget* plug=gtk_plug_new(d->window);
+	if(d->isSizable())
+	{
+		int32_t reqWidth=th->getFrameSize().Xmax/20;
+		int32_t reqHeight=th->getFrameSize().Ymax/20;
+		if(th->standalone)
+			gtk_widget_set_size_request(plug, reqWidth, reqHeight);
+		d->width=reqWidth;
+		d->height=reqHeight;
+	}
+	d->container = plug;
 	//Realize the widget now, as we need the X window
-	gtk_widget_realize(p.container);
+	gtk_widget_realize(plug);
 	//Show it now
-	gtk_widget_show(p.container);
-	gtk_widget_map(p.container);
-	p.window=GDK_WINDOW_XWINDOW(p.container->window);
-	XSync(p.display, False);
+	gtk_widget_show(plug);
+	gtk_widget_map(plug);
+	if (th->standalone)
+	{
+		gtk_widget_set_can_focus(plug, true);
+		gtk_widget_grab_focus(plug);
+	}
+	d->window=GDK_WINDOW_XID(gtk_widget_get_window(plug));
+	XSync(d->display, False);
 	//The lock is needed to avoid thread creation/destruction races
 	Locker l(th->mutex);
 	if(th->shutdown)
 		return;
-	th->renderThread->start(th->engine, &th->npapiParams);
-	th->inputThread->start(th->engine, &th->npapiParams);
+	th->renderThread->start(th->engineData);
+	th->inputThread->start(th->engineData);
 	//If the render rate is known start the render ticks
 	if(th->renderRate)
 		th->startRenderTicks();
@@ -570,13 +582,11 @@ void SystemState::delayedStopping(SystemState* th)
 {
 	sys=th;
 	//This is called from the plugin, also kill the stream
-	th->npapiParams.stopDownloaderHelper(th->npapiParams.helperArg);
+	th->engineData->stopMainDownload();
 	th->stopEngines();
 	sys=NULL;
 }
 
-#endif
-
 void SystemState::createEngines()
 {
 	Locker l(mutex);
@@ -585,9 +595,8 @@ void SystemState::createEngines()
 		//A shutdown request has arrived before the creation of engines
 		return;
 	}
-#ifdef COMPILE_PLUGIN
 	//Check if we should fall back on gnash
-	if(useGnashFallback && engine==GTKPLUG && vmVersion!=AVM2)
+	if(useGnashFallback && vmVersion!=AVM2)
 	{
 		if(dumpedSWFPath.len()==0) //The path is not known yet
 		{
@@ -598,14 +607,14 @@ void SystemState::createEngines()
 				return;
 			l.lock();
 		}
-		LOG(LOG_NO_INFO,_("Trying to invoke gnash!"));
+		LOG(LOG_INFO,_("Trying to invoke gnash!"));
 		//Dump the cookies to a temporary file
 		strcpy(cookiesFileName,"/tmp/lightsparkcookiesXXXXXX");
 		int file=mkstemp(cookiesFileName);
 		if(file!=-1)
 		{
 			std::string data("Set-Cookie: " + rawCookies);
-			size_t res;
+			ssize_t res;
 			size_t written = 0;
 			// Keep writing until everything we wanted to write actually got written
 			do
@@ -631,7 +640,8 @@ void SystemState::createEngines()
 
 		// This will be used to pipe the SWF's data to Gnash's stdin
 		int gnashStdin[2];
-		pipe(gnashStdin);
+		if (pipe(gnashStdin) != 0)
+			LOG(LOG_ERROR,_("Pipe creation failed"));
 
 		childPid=fork();
 		if(childPid==-1)
@@ -654,9 +664,9 @@ void SystemState::createEngines()
 			char bufXid[32];
 			char bufWidth[32];
 			char bufHeight[32];
-			snprintf(bufXid,32,"%lu",npapiParams.window);
-			snprintf(bufWidth,32,"%u",npapiParams.width);
-			snprintf(bufHeight,32,"%u",npapiParams.height);
+			snprintf(bufXid,32,"%lu",engineData->window);
+			snprintf(bufWidth,32,"%u",engineData->width);
+			snprintf(bufHeight,32,"%u",engineData->height);
 			string params("FlashVars=");
 			params+=rawParameters;
 			char *const args[] =
@@ -739,36 +749,24 @@ void SystemState::createEngines()
 			//Engines should not be started, stop everything
 			l.unlock();
 			//We cannot stop the engines now, as this is inside a ThreadPool job
-			npapiParams.helper(npapiParams.helperArg, (helper_t)delayedStopping, this);
+			engineData->setupMainThreadCallback((ls_callback_t)delayedStopping, this);
 			return;
 		}
 	}
-#else 
-	//COMPILE_PLUGIN not defined
-	if(useGnashFallback && engine==GTKPLUG && vmVersion!=AVM2)
-	{
-		throw new UnsupportedException("GNASH fallback not available when not built with COMPILE_PLUGIN");
-	}
-#endif
 
-	if(engine==GTKPLUG) //The engines must be created in the context of the main thread
-	{
-#ifdef COMPILE_PLUGIN
-		npapiParams.helper(npapiParams.helperArg, (helper_t)delayedCreation, this);
-#else
-		throw new UnsupportedException("Plugin engine not available when not built with COMPILE_PLUGIN");
-#endif
-	}
-	else //SDL engine
-	{
-		renderThread->start(engine, NULL);
-		inputThread->start(engine, NULL);
-		//If the render rate is known start the render ticks
-		if(renderRate)
-			startRenderTicks();
-	}
 	l.unlock();
+	//The engines must be created in the context of the main thread
+	engineData->setupMainThreadCallback((ls_callback_t)delayedCreation, this);
+
 	renderThread->waitForInitialization();
+
+	// If the SWF file is AVM1 and Gnash fallback isn't enabled, just shut down.
+	if(vmVersion != AVM2)
+	{
+		LOG(LOG_INFO, "Unsupported flash file (AVM1), shutting down...");
+		setShutdownFlag();
+	}
+
 	l.lock();
 	//As we lost the lock the shutdown procesure might have started
 	if(shutdown)
@@ -785,21 +783,21 @@ void SystemState::needsAVM2(bool n)
 	if(n)
 	{
 		vmVersion=AVM2;
-		LOG(LOG_NO_INFO,_("Creating VM"));
+		LOG(LOG_INFO,_("Creating VM"));
 		currentVm=new ABCVm(this);
 	}
 	else
 		vmVersion=AVM1;
-	if(engine)
+
+	if(engineData)
 		addJob(new EngineCreator);
 }
 
-void SystemState::setParamsAndEngine(ENGINE e, NPAPI_params* p)
+void SystemState::setParamsAndEngine(EngineData* e, bool s)
 {
 	Locker l(mutex);
-	if(p)
-		npapiParams=*p;
-	engine=e;
+	engineData=e;
+	standalone=s;
 	if(vmVersion)
 		addJob(new EngineCreator);
 }
@@ -815,38 +813,6 @@ void SystemState::setRenderRate(float rate)
 	startRenderTicks();
 }
 
-void SystemState::tick()
-{
-	RootMovieClip::tick();
-	{
-		SpinlockLocker l(profileDataSpinlock);
-		list<ThreadProfile>::iterator it=profilingData.begin();
-		for(;it!=profilingData.end();++it)
-			it->tick();
-	}
-	//Send enterFrame events, if needed
-	{
-		Locker l(mutexEnterFrameListeners);
-		if(!enterFrameListeners.empty())
-		{
-			_R<Event> e(Class<Event>::getInstanceS("enterFrame"));
-			auto it=enterFrameListeners.begin();
-			for(;it!=enterFrameListeners.end();it++)
-			{
-				(*it)->incRef();
-				getVm()->addEvent(_MR(*it),e);
-			}
-		}
-	}
-	//Enter frame should be sent to the stage too
-	if(stage->hasEventListener("enterFrame"))
-	{
-		_R<Event> e(Class<Event>::getInstanceS("enterFrame"));
-		stage->incRef();
-		getVm()->addEvent(_MR(stage),e);
-	}
-}
-
 void SystemState::addJob(IThreadJob* j)
 {
 	threadPool->addJob(j);
@@ -964,7 +930,7 @@ void ThreadProfile::tick()
 	}
 }
 
-void ThreadProfile::plot(uint32_t maxTime, FTFont* font)
+void ThreadProfile::plot(uint32_t maxTime, cairo_t *cr)
 {
 	if(data.size()<=1)
 		return;
@@ -974,30 +940,40 @@ void ThreadProfile::plot(uint32_t maxTime, FTFont* font)
 	int width=size.Xmax/20;
 	int height=size.Ymax/20;
 	
+	GLfloat *vertex_coords = new GLfloat[data.size()*2];
+	GLfloat *color_coords = new GLfloat[data.size()*4];
+
 	int32_t start=tickCount-len;
 	if(int32_t(data[0].index-start)>0)
 		start=data[0].index;
 	
-	glPushAttrib(GL_TEXTURE_BIT | GL_LINE_BIT);
-	glColor3ub(color.Red,color.Green,color.Blue);
-	glDisable(GL_TEXTURE_2D);
-	glLineWidth(2);
+	for(unsigned int i=0;i<data.size();i++)
+	{
+		vertex_coords[i*2] = int32_t(data[i].index-start)*width/len;
+		vertex_coords[i*2+1] = data[i].timing*height/maxTime;
+		color_coords[i*4] = color.Red;
+		color_coords[i*4+1] = color.Green;
+		color_coords[i*4+2] = color.Blue;
+		color_coords[i*4+3] = 1;
+	}
+
+	glVertexAttribPointer(VERTEX_ATTRIB, 2, GL_FLOAT, GL_FALSE, 0, vertex_coords);
+	glVertexAttribPointer(COLOR_ATTRIB, 4, GL_FLOAT, GL_FALSE, 0, color_coords);
+	glEnableVertexAttribArray(VERTEX_ATTRIB);
+	glEnableVertexAttribArray(COLOR_ATTRIB);
+	glDrawArrays(GL_LINE_STRIP, 0, data.size());
+	glDisableVertexAttribArray(VERTEX_ATTRIB);
+	glDisableVertexAttribArray(COLOR_ATTRIB);
+
+	cairo_set_source_rgb(cr, float(color.Red)/255, float(color.Green)/255, float(color.Blue)/255);
 
-	glBegin(GL_LINE_STRIP);
-		for(unsigned int i=0;i<data.size();i++)
-		{
-			int32_t relx=int32_t(data[i].index-start)*width/len;
-			glVertex2i(relx,data[i].timing*height/maxTime);
-		}
-	glEnd();
-	glPopAttrib();
-	
 	//Draw tags
 	string* curTag=NULL;
 	int curTagX=0;
 	int curTagY=maxTime;
 	int curTagLen=0;
 	int curTagH=0;
+	cairo_text_extents_t te;
 	for(unsigned int i=0;i<data.size();i++)
 	{
 		int32_t relx=int32_t(data[i].index-start)*width/len;
@@ -1005,11 +981,11 @@ void ThreadProfile::plot(uint32_t maxTime, FTFont* font)
 		{
 			//New tag, flush the old one if present
 			if(curTag)
-				font->Render(curTag->c_str() ,-1,FTPoint(curTagX,imax(curTagY-curTagH,0)));
+				rt->renderText(cr, curTag->c_str(),curTagX,imax(curTagY-curTagH,0));
 			//Measure tag
-			FTBBox tagBox=font->BBox(data[i].tag.c_str(),-1);
-			curTagLen=(tagBox.Upper()-tagBox.Lower()).X();
-			curTagH=(tagBox.Upper()-tagBox.Lower()).Y();
+			cairo_text_extents (cr, data[i].tag.c_str(), &te);
+			curTagLen=te.width;
+			curTagH=te.height;
 			curTag=&data[i].tag;
 			curTagX=relx;
 			curTagY=maxTime;
@@ -1021,18 +997,28 @@ void ThreadProfile::plot(uint32_t maxTime, FTFont* font)
 			else
 			{
 				//Tag is before this sample
-				font->Render(curTag->c_str() ,-1,FTPoint(curTagX,imax(curTagY-curTagH,0)));
+				rt->renderText(cr, curTag->c_str(), curTagX, imax(curTagY-curTagH,0));
 				curTag=NULL;
 			}
 		}
 	}
 }
 
-ParseThread::ParseThread(RootMovieClip* r,istream& in):root(NULL),version(0),useAVM2(false),
-	f(in),zlibFilter(NULL),backend(NULL),isEnded(false),fileType(NONE)
+ParseThread::ParseThread(istream& in, Loader *_loader, tiny_string srcurl)
+  : version(0),useAVM2(false),useNetwork(false),
+    f(in),zlibFilter(NULL),backend(NULL),loader(_loader),
+    parsedObject(NullRef),url(srcurl),fileType(FT_UNKNOWN)
 {
-	root=r;
-	sem_init(&ended,0,0);
+	f.exceptions ( istream::eofbit | istream::failbit | istream::badbit );
+}
+
+ParseThread::ParseThread(std::istream& in, RootMovieClip *root)
+  : version(0),useAVM2(false),useNetwork(false),
+    f(in),zlibFilter(NULL),backend(NULL),loader(NULL),
+    parsedObject(NullRef),url(),fileType(FT_UNKNOWN)
+{
+	f.exceptions ( istream::eofbit | istream::failbit | istream::badbit );
+	setRootMovie(root);
 }
 
 ParseThread::~ParseThread()
@@ -1043,48 +1029,45 @@ ParseThread::~ParseThread()
 		f.rdbuf(backend);
 		delete zlibFilter;
 	}
-	sem_destroy(&ended);
+	parsedObject.reset();
 }
 
-void ParseThread::checkType(uint8_t c1, uint8_t c2, uint8_t c3, uint8_t c4)
+FILE_TYPE ParseThread::recognizeFile(uint8_t c1, uint8_t c2, uint8_t c3, uint8_t c4)
 {
 	if(c1=='F' && c2=='W' && c3=='S')
-	{
-		fileType=SWF;
-		version=c4;
-		root->version=version;
-	}
+		return FT_SWF;
 	else if(c1=='C' && c2=='W' && c3=='S')
-	{
-		fileType=COMPRESSED_SWF;
-		version=c4;
-		root->version=version;
-	}
+		return FT_COMPRESSED_SWF;
 	else if((c1&0x80) && c2=='P' && c3=='N' && c4=='G')
-		fileType=PNG;
-	else if(c1==0xff && c2==0xd8 && c3==0xff && c4==0xe0)
-		fileType=JPEG;
+		return FT_PNG;
+	else if(c1==0xff && c2==0xd8 && c3==0xff)
+		return FT_JPEG;
 	else if(c1=='G' && c2=='I' && c3=='F' && c4=='8')
-		fileType=GIF;
+		return FT_GIF;
+	else
+		return FT_UNKNOWN;
 }
 
-void ParseThread::parseSWFHeader()
+void ParseThread::parseSWFHeader(RootMovieClip *root, UI8 ver)
 {
 	UI32_SWF FileLength;
 	RECT FrameSize;
 	UI16_SWF FrameRate;
 	UI16_SWF FrameCount;
 
+	version=ver;
+	root->version=version;
 	f >> FileLength;
 	//Enable decompression if needed
-	if(fileType==SWF)
-		LOG(LOG_NO_INFO, _("Uncompressed SWF file: Version ") << (int)version);
-	else if(fileType==COMPRESSED_SWF)
+	if(fileType==FT_SWF)
+		LOG(LOG_INFO, _("Uncompressed SWF file: Version ") << (int)version);
+	else if(fileType==FT_COMPRESSED_SWF)
 	{
-		LOG(LOG_NO_INFO, _("Compressed SWF file: Version ") << (int)version);
+		LOG(LOG_INFO, _("Compressed SWF file: Version ") << (int)version);
 		//The file is compressed, create a filtering streambuf
 		backend=f.rdbuf();
-		f.rdbuf(new zlib_filter(backend));
+		zlibFilter = new zlib_filter(backend);
+		f.rdbuf(zlibFilter);
 	}
 
 	f >> FrameSize >> FrameRate >> FrameCount;
@@ -1092,7 +1075,7 @@ void ParseThread::parseSWFHeader()
 	root->fileLength=FileLength;
 	float frameRate=FrameRate;
 	frameRate/=256;
-	LOG(LOG_NO_INFO,_("FrameRate ") << frameRate);
+	LOG(LOG_INFO,_("FrameRate ") << frameRate);
 	root->setFrameRate(frameRate);
 	//TODO: setting render rate should be done when the clip is added to the displaylist
 	sys->setRenderRate(frameRate);
@@ -1107,17 +1090,40 @@ void ParseThread::execute()
 	{
 		UI8 Signature[4];
 		f >> Signature[0] >> Signature[1] >> Signature[2] >> Signature[3];
-		checkType(Signature[0],Signature[1],Signature[2],Signature[3]);
-		if(fileType==NONE)
+		fileType=recognizeFile(Signature[0],Signature[1],Signature[2],Signature[3]);
+		if(fileType==FT_UNKNOWN)
 			throw ParseException("Not a supported file");
-		if(fileType==PNG || fileType==JPEG || fileType==GIF)
+		else if(fileType==FT_PNG || fileType==FT_JPEG || fileType==FT_GIF)
 		{
-			//Not really supported
-			pt=NULL;
-			sem_post(&ended);
-			return;
+			f.putback(Signature[3]).putback(Signature[2]).
+			  putback(Signature[1]).putback(Signature[0]);
+			parseBitmap();
 		}
-		parseSWFHeader();
+		else
+		{
+			parseSWF(Signature[3]);
+		}
+	}
+	catch(LightsparkException& e)
+	{
+		LOG(LOG_ERROR,_("Exception in ParseThread ") << e.cause);
+		sys->setError(e.cause);
+	}
+	catch(std::exception& e)
+	{
+		LOG(LOG_ERROR,_("Stream exception in ParseThread ") << e.what());
+	}
+	pt=NULL;
+}
+
+void ParseThread::parseSWF(UI8 ver)
+{
+	RootMovieClip* root=getRootMovie();
+	assert_and_throw(root);
+
+	try
+	{
+		parseSWFHeader(root, ver);
 
 		//Create a top level TagFactory
 		TagFactory factory(f, true);
@@ -1125,31 +1131,29 @@ void ParseThread::execute()
 		bool empty=true;
 		while(!done)
 		{
-			Tag* tag=factory.readTag();
-			if(tag->getType() != CONTROL_TAG && tag->getType() != ABC_TAG)
-				sys->registerTag(tag);
+			_NR<Tag> tag=factory.readTag();
 			switch(tag->getType())
 			{
 				case END_TAG:
 				{
-					LOG(LOG_NO_INFO,_("End of parsing @ ") << f.tellg());
 					if(!empty)
 						root->commitFrame(false);
 					else
 						root->revertFrame();
+					RELEASE_WRITE(root->finishedLoading,true);
 					done=true;
 					root->check();
 					break;
 				}
 				case DICT_TAG:
 				{
-					DictionaryTag* d=static_cast<DictionaryTag*>(tag);
+					_R<DictionaryTag> d=tag.cast<DictionaryTag>();
 					d->setLoadedFrom(root);
 					root->addToDictionary(d);
 					break;
 				}
 				case DISPLAY_LIST_TAG:
-					root->addToFrame(static_cast<DisplayListTag*>(tag));
+					root->addToFrame(tag.cast<DisplayListTag>());
 					empty=false;
 					break;
 				case SHOW_TAG:
@@ -1169,16 +1173,15 @@ void ParseThread::execute()
 					//fall through
 				case ABC_TAG:
 				{
-					ControlTag* ctag = static_cast<ControlTag*>(tag);
+					_R<ControlTag> ctag = tag.cast<ControlTag>();
 					ctag->execute(root);
-					delete ctag;
 					break;
 				}
 				case FRAMELABEL_TAG:
 					/* No locking required, as the last frames is not
 					 * commited to the vm yet.
 					 */
-					root->addFrameLabel(root->frames.size()-1,static_cast<FrameLabelTag*>(tag)->Name);
+					root->addFrameLabel(root->frames.size()-1,static_cast<FrameLabelTag*>(tag.getPtr())->Name);
 					empty=false;
 					break;
 				case TAG:
@@ -1189,59 +1192,68 @@ void ParseThread::execute()
 				break;
 		}
 	}
-	catch(LightsparkException& e)
+	catch(std::exception& e)
 	{
-		LOG(LOG_ERROR,_("Exception in ParseThread ") << e.cause);
 		root->parsingFailed();
-		sys->setError(e.cause);
+		throw;
 	}
-	pt=NULL;
+}
 
-	sem_post(&ended);
+void ParseThread::parseBitmap()
+{
+	_NR<Bitmap> tmp=_MNR(Class<Bitmap>::getInstanceS(&f, fileType));
+
+	{
+		SpinlockLocker l(objectSpinlock);
+		parsedObject=tmp;
+	}
+}
+
+_NR<DisplayObject> ParseThread::getParsedObject()
+{
+	SpinlockLocker l(objectSpinlock);
+	return parsedObject;
+}
+
+void ParseThread::setRootMovie(RootMovieClip *root)
+{
+	SpinlockLocker l(objectSpinlock);
+	assert(root);
+	root->incRef();
+	parsedObject=_MNR(root);
+}
+
+RootMovieClip *ParseThread::getRootMovie()
+{
+	objectSpinlock.lock();
+	RootMovieClip *root=Class<RootMovieClip>::dyncast(parsedObject.getPtr());
+	objectSpinlock.unlock();
+	if(root)
+		return root;
+	else if(fileType==FT_SWF || fileType==FT_COMPRESSED_SWF)
+	{
+		LoaderInfo *li=loader?loader->getContentLoaderInfo().getPtr():NULL;
+		root=RootMovieClip::getInstance(li);
+		objectSpinlock.lock();
+		parsedObject=_MNR(root);
+		objectSpinlock.unlock();
+		if(url.len()>0)
+			root->setOrigin(url, "");
+		return root;
+	}
+	else
+		return NULL;
 }
 
 void ParseThread::threadAbort()
 {
 	//Tell the our RootMovieClip that the parsing is ending
-	root->parsingFailed();
+	RootMovieClip *root=getRootMovie();
+	if(root)
+		root->parsingFailed();
 }
 
-void RootMovieClip::initialize()
-{
-	//NOTE: No references to frames must be done here!
-	if(!initialized && sys->currentVm)
-	{
-		initialized=true;
-		if(sys->currentVm)
-		{
-			//Let's see if we have to bind the root movie clip itself
-			if(bindName.len())
-			{
-				//The object is constructed after binding
-				this->incRef();
-				sys->currentVm->addEvent(NullRef,_MR(new BindClassEvent(_MR(this),bindName,BindClassEvent::ISROOT)));
-				sys->currentVm->addEvent(NullRef,_MR(new SysOnStageEvent()));
-			}
-			else
-			{
-				//Construction can be consider complete
-				constructionComplete();
-				setOnStage(true);
-			}
-
-			//Now signal the completion for this root
-			loaderInfo->sendInit();
-		}
-		else
-		{
-			//Construction can be consider complete
-			constructionComplete();
-			setOnStage(true);
-		}
-	}
-}
-
-bool RootMovieClip::getBounds(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const
+bool RootMovieClip::boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const
 {
 	RECT f=getFrameSize();
 	xmin=0;
@@ -1251,22 +1263,6 @@ bool RootMovieClip::getBounds(number_t& xmin, number_t& xmax, number_t& ymin, nu
 	return true;
 }
 
-void RootMovieClip::Render(bool maskEnabled)
-{
-	while(1)
-	{
-		//Check if the next frame we are going to play is available
-		if(state.next_FP<getFramesLoaded())
-			break;
-
-		sem_wait(&new_frame);
-		if(parsingIsFailed)
-			return;
-	}
-
-	MovieClip::Render(maskEnabled);
-}
-
 void RootMovieClip::setFrameSize(const lightspark::RECT& f)
 {
 	frameSize=f;
@@ -1295,18 +1291,19 @@ void RootMovieClip::commitFrame(bool another)
 	if(another)
 		frames.emplace_back();
 
-	if(getFramesLoaded()==1)
+	if(getFramesLoaded()==1 && frameRate!=0)
 	{
-		//Let's initialize the first frame of this movieclip
-		bootstrap();
-
-		//Root movie clips are initialized now, after the first frame is really ready 
-		initialize();
-
-		//When the first frame is committed the frame rate is known
-		sys->addTick(1000/frameRate,this);
+		if(this==sys)
+		{
+			/* now the frameRate is available and all SymbolClass tags have created their classes */
+			sys->addTick(1000/frameRate,sys);
+		}
+		else
+		{
+			this->incRef();
+			sys->currentVm->addEvent(NullRef, _MR(new InitFrameEvent(_MNR(this))));
+		}
 	}
-	sem_post(&new_frame);
 }
 
 void RootMovieClip::revertFrame()
@@ -1327,17 +1324,17 @@ void RootMovieClip::setBackground(const RGB& bg)
 }
 
 /* called in parser's thread context */
-void RootMovieClip::addToDictionary(DictionaryTag* r)
+void RootMovieClip::addToDictionary(_R<DictionaryTag> r)
 {
 	SpinlockLocker l(dictSpinlock);
 	dictionary.push_back(r);
 }
 
 /* called in vm's thread context */
-DictionaryTag* RootMovieClip::dictionaryLookup(int id)
+_R<DictionaryTag> RootMovieClip::dictionaryLookup(int id)
 {
 	SpinlockLocker l(dictSpinlock);
-	list< DictionaryTag*>::iterator it = dictionary.begin();
+	auto it = dictionary.begin();
 	for(;it!=dictionary.end();++it)
 	{
 		if((*it)->getId()==id)
@@ -1348,8 +1345,7 @@ DictionaryTag* RootMovieClip::dictionaryLookup(int id)
 		LOG(LOG_ERROR,_("No such Id on dictionary ") << id);
 		throw RunTimeException("Could not find an object on the dictionary");
 	}
-	DictionaryTag* ret=*it;
-	return ret;
+	return *it;
 }
 
 _NR<RootMovieClip> RootMovieClip::getRoot()
@@ -1358,33 +1354,6 @@ _NR<RootMovieClip> RootMovieClip::getRoot()
 	return _MR(this);
 }
 
-void RootMovieClip::tick()
-{
-	//Frame advancement may cause exceptions
-	try
-	{
-		advanceFrame();
-		//advanceFrame may be blocking (if Frame init happens)
-		Locker l(mutexChildrenClips);
-		vector<MovieClip*> tmpChildren(childrenClips.begin(),childrenClips.end());
-		for(uint32_t i=0;i<tmpChildren.size();i++)
-			tmpChildren[i]->incRef();
-		l.unlock();
-		//Advance all the children to the next frame using the temporary copy
-		for(uint32_t i=0;i<tmpChildren.size();i++)
-		{
-			tmpChildren[i]->advanceFrame();
-			//Release when done
-			tmpChildren[i]->decRef();
-		}
-	}
-	catch(LightsparkException& e)
-	{
-		LOG(LOG_ERROR,_("Exception in RootMovieClip::tick ") << e.cause);
-		sys->setError(e.cause);
-	}
-}
-
 /*ASObject* RootMovieClip::getVariableByQName(const tiny_string& name, const tiny_string& ns, ASObject*& owner)
 {
 	sem_wait(&mutex);
@@ -1443,3 +1412,104 @@ void RootMovieClip::setVariableByString(const string& s, ASObject* o)
 }*/
 
 
+void SystemState::tick()
+{
+	{
+		SpinlockLocker l(profileDataSpinlock);
+		list<ThreadProfile>::iterator it=profilingData.begin();
+		for(;it!=profilingData.end();++it)
+			it->tick();
+	}
+	if(sys->currentVm==NULL)
+		return;
+	/* See http://www.senocular.com/flash/tutorials/orderofoperations/
+	 * for the description of steps.
+	 */
+	/* TODO: Step 1: declare new objects */
+
+	/* Step 2: Send enterFrame events, if needed */
+	{
+		Locker l(mutexFrameListeners);
+		if(!frameListeners.empty())
+		{
+			_R<Event> e(Class<Event>::getInstanceS("enterFrame"));
+			auto it=frameListeners.begin();
+			for(;it!=frameListeners.end();it++)
+				getVm()->addEvent(*it,e);
+		}
+	}
+
+	/* Step 3: create legacy objects, which are new in this frame (top-down),
+	 * run their constructors (bottom-up)
+	 * and their frameScripts (Step 5) (bottom-up) */
+	sys->currentVm->addEvent(NullRef, _MR(new InitFrameEvent()));
+
+	/* Step 4: dispatch frameConstructed events */
+	/* (TODO: should be run between step 3 and 5 */
+	{
+		Locker l(mutexFrameListeners);
+		if(!frameListeners.empty())
+		{
+			_R<Event> e(Class<Event>::getInstanceS("frameConstructed"));
+			auto it=frameListeners.begin();
+			for(;it!=frameListeners.end();it++)
+				getVm()->addEvent(*it,e);
+		}
+	}
+	/* Step 6: dispatch exitFrame event */
+	{
+		Locker l(mutexFrameListeners);
+		if(!frameListeners.empty())
+		{
+			_R<Event> e(Class<Event>::getInstanceS("exitFrame"));
+			auto it=frameListeners.begin();
+			for(;it!=frameListeners.end();it++)
+				getVm()->addEvent(*it,e);
+		}
+	}
+	/* TODO: Step 7: dispatch render event (Assuming stage.invalidate() has been called) */
+
+	/* Step 0: Set current frame number to the next frame */
+	_R<AdvanceFrameEvent> advFrame = _MR(new AdvanceFrameEvent());
+	if(sys->currentVm->addEvent(NullRef, advFrame))
+		advFrame->done.wait();
+}
+
+void SystemState::resizeCompleted() const
+{
+	if(currentVm && scaleMode==NO_SCALE)
+	{
+		stage->incRef();
+		currentVm->addEvent(_MR(stage),_MR(Class<Event>::getInstanceS("resize",false)));
+	}
+}
+
+/* This is run in vm's thread context */
+void RootMovieClip::initFrame()
+{
+	LOG(LOG_CALLS,"Root:initFrame " << getFramesLoaded() << " " << state.FP);
+	/* We have to wait for at least one frame
+	 * so our class get the right classdef. Else we will
+	 * call the wrong constructor. */
+	if(getFramesLoaded() == 0)
+		return;
+
+	MovieClip::initFrame();
+}
+
+/* This is run in vm's thread context */
+void RootMovieClip::advanceFrame()
+{
+	/* We have to wait for at least one frame */
+	if(getFramesLoaded() == 0)
+		return;
+
+	MovieClip::advanceFrame();
+}
+
+void RootMovieClip::constructionComplete()
+{
+	MovieClip::constructionComplete();
+	if(this==sys)
+		loaderInfo->sendInit();
+}
