@@ -21,71 +21,63 @@
 #define SWF_H
 
 #include "compat.h"
-#include <iostream>
 #include <fstream>
 #include <list>
 #include <map>
 #include <semaphore.h>
 #include <string>
 #include "swftypes.h"
-#include "scripting/flashdisplay.h"
-#include "scripting/flashnet.h"
-#include "scripting/flashsystem.h"
+#include "scripting/flash/display/flashdisplay.h"
+#include "scripting/flash/net/flashnet.h"
 #include "timer.h"
-#include "backends/audio.h"
-#include "backends/config.h"
-#include "backends/graphics.h"
-#include "backends/pluginmanager.h"
-#include "backends/security.h"
-#include "backends/urlutils.h"
-#include "backends/extscriptobject.h"
 
-#include "platforms/pluginutils.h"
-
-#ifndef WIN32
-#include <GL/glx.h>
-#else
-//#include <windows.h>
-#endif
+#include "platforms/engineutils.h"
 
 namespace lightspark
 {
 
+class ABCVm;
+class AudioManager;
+class Config;
 class DownloadManager;
 class DisplayListTag;
 class DictionaryTag;
-class ABCVm;
+class ExtScriptObject;
 class InputThread;
-class RenderThread;
 class ParseThread;
+class PluginManager;
+class RenderThread;
+class SecurityManager;
 class Tag;
 
 //RootMovieClip is used as a ThreadJob for timed rendering purpose
-class RootMovieClip: public MovieClip, public ITickJob
+class RootMovieClip: public MovieClip
 {
 friend class ParseThread;
 protected:
 	Mutex mutex;
-	bool initialized;
 	URLInfo origin;
-	void tick();
 private:
-	//Semaphore to wait for new frames to be available
-	sem_t new_frame;
 	bool parsingIsFailed;
 	RGB Background;
 	Spinlock dictSpinlock;
-	std::list < DictionaryTag* > dictionary;
+	std::list < _R<DictionaryTag> > dictionary;
 	//frameSize and frameRate are valid only after the header has been parsed
 	RECT frameSize;
 	float frameRate;
 	bool toBind;
 	tiny_string bindName;
-	Mutex mutexChildrenClips;
-	std::set<MovieClip*> childrenClips;
+	void constructionComplete();
+	/* those are private because you shouldn't call sys->*,
+	 * but sys->getStage()->* instead.
+	 */
+	void initFrame();
+	void advanceFrame();
+	void setOnStage(bool staged);
+	ACQUIRE_RELEASE_FLAG(finishedLoading);
 public:
 	RootMovieClip(LoaderInfo* li, bool isSys=false);
-	~RootMovieClip();
+	bool hasFinishedLoading() { return ACQUIRE_READ(finishedLoading); }
 	uint32_t version;
 	uint32_t fileLength;
 	RGB getBackground();
@@ -94,25 +86,20 @@ public:
 	RECT getFrameSize() const;
 	float getFrameRate() const;
 	void setFrameRate(float f);
-	void setOnStage(bool staged);
-	void addToDictionary(DictionaryTag* r);
-	DictionaryTag* dictionaryLookup(int id);
+	void addToDictionary(_R<DictionaryTag> r);
+	_R<DictionaryTag> dictionaryLookup(int id);
 	void labelCurrentFrame(const STRING& name);
 	void commitFrame(bool another);
 	void revertFrame();
-	void Render(bool maskEnabled);
 	void parsingFailed();
-	bool getBounds(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const;
+	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const;
 	void bindToName(const tiny_string& n);
-	void initialize();
 	void DLL_PUBLIC setOrigin(const tiny_string& u, const tiny_string& filename="");
 	URLInfo& getOrigin() DLL_PUBLIC { return origin; };
 /*	ASObject* getVariableByQName(const tiny_string& name, const tiny_string& ns);
 	void setVariableByQName(const tiny_string& name, const tiny_string& ns, ASObject* o);
 	void setVariableByMultiname(multiname& name, ASObject* o);
 	void setVariableByString(const std::string& s, ASObject* o);*/
-	void registerChildClip(MovieClip* clip);
-	void unregisterChildClip(MovieClip* clip);
 	static RootMovieClip* getInstance(LoaderInfo* li);
 	//DisplayObject interface
 	_NR<RootMovieClip> getRoot();
@@ -139,10 +126,10 @@ public:
 	void accountTime(uint32_t time);
 	void setTag(const std::string& tag);
 	void tick();
-	void plot(uint32_t max, FTFont* font);
+	void plot(uint32_t max, cairo_t *cr);
 };
 
-class SystemState: public RootMovieClip
+class SystemState: public RootMovieClip, public ITickJob
 {
 private:
 	class EngineCreator: public IThreadJob
@@ -165,8 +152,7 @@ private:
 	bool shutdown;
 	RenderThread* renderThread;
 	InputThread* inputThread;
-	NPAPI_params npapiParams;
-	ENGINE engine;
+	EngineData* engineData;
 	void startRenderTicks();
 	/**
 		Create the rendering and input engines
@@ -177,11 +163,10 @@ private:
 	/**
 	  	Destroys all the engines used in lightspark: timer, thread pool, vm...
 	*/
-#ifdef COMPILE_PLUGIN
+	void stopEngines();
+
 	static void delayedCreation(SystemState* th);
 	static void delayedStopping(SystemState* th);
-#endif
-	void stopEngines();
 	//Useful to wait for complete download of the SWF
 	Semaphore fileDumpAvailable;
 	tiny_string dumpedSWFPath;
@@ -207,8 +192,8 @@ private:
 	URLInfo url;
 	Spinlock profileDataSpinlock;
 
-	Mutex mutexEnterFrameListeners;
-	std::set<DisplayObject*> enterFrameListeners;
+	Mutex mutexFrameListeners;
+	std::set<_R<DisplayObject>> frameListeners;
 	/*
 	   The head of the invalidate queue
 	*/
@@ -221,26 +206,20 @@ private:
 	   The lock for the invalidate queue
 	*/
 	Spinlock invalidateQueueLock;
-	/*
-	   Vector to keep track of the created tags
-	*/
-	std::vector<Tag*> tagsStorage;
-	/*
-	   The lock for the tags storage
-	*/
-	Spinlock tagsStorageLock;
 #ifdef PROFILING_SUPPORT
 	/*
 	   Output file for the profiling data
 	*/
 	tiny_string profOut;
 #endif
+protected:
+	~SystemState();
 public:
 	void setURL(const tiny_string& url) DLL_PUBLIC;
-	ENGINE getEngine() DLL_PUBLIC { return engine; };
 
 	//Interative analysis flags
 	bool showProfilingData;
+	bool standalone;
 	
 	std::string errorCause;
 	void setError(const std::string& c);
@@ -254,7 +233,7 @@ public:
 	void wait() DLL_PUBLIC;
 	RenderThread* getRenderThread() const { return renderThread; }
 	InputThread* getInputThread() const { return inputThread; }
-	void setParamsAndEngine(ENGINE e, NPAPI_params* p) DLL_PUBLIC;
+	void setParamsAndEngine(EngineData* e, bool s) DLL_PUBLIC;
 	void setDownloadedPath(const tiny_string& p) DLL_PUBLIC;
 	void enableGnashFallback() DLL_PUBLIC;
 	void needsAVM2(bool n);
@@ -265,7 +244,11 @@ public:
 	//before any other thread gets started
 	SystemState(ParseThread* p, uint32_t fileSize) DLL_PUBLIC;
 	void finalize();
-	~SystemState();
+	/* Stop engines, threads and free classes and objects.
+	 * This call will decRef this object in the end,
+	 * thus destroy() may cause a 'delete this'.
+	 */
+	void destroy() DLL_PUBLIC;
 	
 	//Performance profiling
 	ThreadProfile* allocateProfiler(const RGB& color);
@@ -281,12 +264,14 @@ public:
 	//Application starting time in milliseconds
 	uint64_t startTime;
 
-	//Class map
+	//Class/Template map. They own one reference to each class/template
 	std::map<QName, Class_base*> classes;
+	std::map<QName, Template_base*> templates;
 
 	//Flags for command line options
 	bool useInterpreter;
 	bool useJit;
+	bool exitOnError;
 
 	//Parameters/FlashVars
 	void parseParametersFromFile(const char* f) DLL_PUBLIC;
@@ -329,8 +314,8 @@ public:
 	ObjectEncoding::ENCODING staticByteArrayDefaultObjectEncoding;
 	
 	//enterFrame event management
-	void registerEnterFrameListener(DisplayObject* clip);
-	void unregisterEnterFrameListener(DisplayObject* clip);
+	void registerFrameListener(_R<DisplayObject> clip);
+	void unregisterFrameListener(_R<DisplayObject> clip);
 
 	//tags management
 	void registerTag(Tag* t);
@@ -339,6 +324,9 @@ public:
 	void addToInvalidateQueue(_R<DisplayObject> d);
 	void flushInvalidationQueue();
 
+	//Resize support
+	void resizeCompleted() const;
+
 #ifdef PROFILING_SUPPORT
 	void setProfilingOutput(const tiny_string& t) DLL_PUBLIC;
 	const tiny_string& getProfilingOutput() const;
@@ -350,28 +338,37 @@ public:
 class ParseThread: public IThreadJob
 {
 public:
-	RootMovieClip* root;
 	int version;
 	bool useAVM2;
-	ParseThread(RootMovieClip* r,std::istream& in) DLL_PUBLIC;
+	bool useNetwork;
+	// Parse an object from stream. The type is detected
+	// automatically. After parsing the new object is available
+	// from getParsedObject().
+	ParseThread(std::istream& in, Loader *loader=NULL, tiny_string url="") DLL_PUBLIC;
+	// Parse a clip from stream into root. The stream must be an
+	// SWF file.
+	ParseThread(std::istream& in, RootMovieClip *root) DLL_PUBLIC;
 	~ParseThread();
-	enum FILE_TYPE { NONE=0, SWF, COMPRESSED_SWF, PNG, JPEG, GIF };
 	FILE_TYPE getFileType() const { return fileType; }
+        _NR<DisplayObject> getParsedObject();
+	void setRootMovie(RootMovieClip *root);
+	RootMovieClip *getRootMovie();
+	static FILE_TYPE recognizeFile(uint8_t c1, uint8_t c2, uint8_t c3, uint8_t c4);
 private:
 	std::istream& f;
 	std::streambuf* zlibFilter;
 	std::streambuf* backend;
-	sem_t ended;
-	bool isEnded;
+	Loader *loader;
+	_NR<DisplayObject> parsedObject;
+	Spinlock objectSpinlock;
+	tiny_string url;
+	FILE_TYPE fileType;
 	void execute();
 	void threadAbort();
 	void jobFence() {};
-	/*
-	   parseHeader takes the first four characters as argument
-	*/
-	void parseSWFHeader();
-	void checkType(uint8_t c1, uint8_t c2, uint8_t c3, uint8_t c4);
-	FILE_TYPE fileType;
+	void parseSWFHeader(RootMovieClip *root, UI8 ver);
+	void parseSWF(UI8 ver);
+	void parseBitmap();
 };
 
 };
