@@ -95,6 +95,7 @@ tiny_string tiny_string::substr(uint32_t start, uint32_t end) const
 		ret.createBuffer(subSize);
 	strncpy(ret.buf,buf+start,end-start);
 	ret.buf[end-start]=0;
+	ret.stringSize = subSize;
 	return ret;
 }
 
@@ -133,6 +134,43 @@ tiny_string multiname::normalizedName() const
 	}
 }
 
+void multiname::setName(ASObject* n)
+{
+	if(n->is<Integer>())
+	{
+		name_i=n->as<Integer>()->val;
+		name_type = NAME_INT;
+	}
+	else if(n->is<UInteger>())
+	{
+		name_i=n->as<UInteger>()->val;
+		name_type = NAME_INT;
+	}
+	else if(n->is<Number>())
+	{
+		name_d=n->as<Number>()->val;
+		name_type = NAME_NUMBER;
+	}
+	else if(n->getObjectType()==T_QNAME)
+	{
+		ASQName* qname=static_cast<ASQName*>(n);
+		name_s=qname->local_name;
+		name_type = NAME_STRING;
+	}
+	else if(n->getObjectType()==T_STRING)
+	{
+		ASString* o=static_cast<ASString*>(n);
+		name_s=o->data;
+		name_type = NAME_STRING;
+	}
+	else
+	{
+		n->incRef();
+		name_o=n;
+		name_type = NAME_OBJECT;
+	}
+}
+
 std::ostream& lightspark::operator<<(std::ostream& s, const QName& r)
 {
 	s << r.ns << ':' << r.name;
@@ -223,7 +261,8 @@ std::ostream& lightspark::operator<<(std::ostream& s, const RGB& r)
 MATRIX MATRIX::getInverted() const
 {
 	MATRIX ret;
-	const number_t den=ScaleX*ScaleY+RotateSkew0*RotateSkew1;
+	assert(isInvertible());
+	const number_t den=ScaleX*ScaleY-RotateSkew0*RotateSkew1;
 	ret.ScaleX=ScaleY/den;
 	ret.RotateSkew1=-RotateSkew1/den;
 	ret.RotateSkew0=-RotateSkew0/den;
@@ -233,6 +272,12 @@ MATRIX MATRIX::getInverted() const
 	return ret;
 }
 
+bool MATRIX::isInvertible() const
+{
+	const number_t den=ScaleX*ScaleY-RotateSkew0*RotateSkew1;
+	return (fabs(den) > 1e-6);
+}
+
 void MATRIX::get4DMatrix(float matrix[16]) const
 {
 	memset(matrix,0,sizeof(float)*16);
@@ -249,6 +294,13 @@ void MATRIX::get4DMatrix(float matrix[16]) const
 	matrix[15]=1;
 }
 
+Vector2f MATRIX::multiply2D(const Vector2f& in) const
+{
+	Vector2f out;
+	multiply2D(in.x,in.y,out.x,out.y);
+	return out;
+}
+
 void MATRIX::multiply2D(number_t xin, number_t yin, number_t& xout, number_t& yout) const
 {
 	xout=xin*ScaleX + yin*RotateSkew1 + TranslateX;
@@ -267,7 +319,7 @@ MATRIX MATRIX::multiplyMatrix(const MATRIX& r) const
 	return ret;
 }
 
-const bool MATRIX::operator!=(const MATRIX& r) const
+bool MATRIX::operator!=(const MATRIX& r) const
 {
 	return ScaleX!=r.ScaleX ||
 		RotateSkew1!=r.RotateSkew1 ||
@@ -630,8 +682,8 @@ std::istream& lightspark::operator>>(std::istream& s, FILLSTYLE& v)
 		{
 			try
 			{
-				DictionaryTag* dict=pt->root->dictionaryLookup(bitmapId);
-				v.bitmap=dynamic_cast<Bitmap*>(dict);
+				_R<DictionaryTag> dict=pt->getRootMovie()->dictionaryLookup(bitmapId);
+				v.bitmap=dynamic_cast<Bitmap*>(dict.getPtr());
 				if(v.bitmap==NULL)
 				{
 					LOG(LOG_ERROR,"Invalid bitmap ID " << bitmapId);
@@ -1106,18 +1158,20 @@ ASObject* lightspark::abstract_d(number_t i)
 	return ret;
 }
 
-ASObject* lightspark::abstract_b(bool i)
-{
-	return Class<Boolean>::getInstanceS(i);
-}
-
-ASObject* lightspark::abstract_i(intptr_t i)
+ASObject* lightspark::abstract_i(int32_t i)
 {
 	Integer* ret=getVm()->int_manager->get<Integer>();
 	ret->val=i;
 	return ret;
 }
 
+ASObject* lightspark::abstract_ui(uint32_t i)
+{
+	UInteger* ret=getVm()->uint_manager->get<UInteger>();
+	ret->val=i;
+	return ret;
+}
+
 void lightspark::stringToQName(const tiny_string& tmp, tiny_string& name, tiny_string& ns)
 {
 	//Ok, let's split our string into namespace and name part
@@ -1146,7 +1200,18 @@ void lightspark::stringToQName(const tiny_string& tmp, tiny_string& name, tiny_s
 	ns="";
 }
 
-RunState::RunState():FP(0),next_FP(0),stop_FP(0),explicit_FP(false)
+RunState::RunState():last_FP(-1),FP(0),next_FP(0),stop_FP(0),explicit_FP(false)
 {
 }
 
+tiny_string QName::getQualifiedName() const
+{
+	tiny_string ret;
+	if(ns.len())
+	{
+		ret+=ns;
+		ret+="::";
+	}
+	ret+=name;
+	return ret;
+}
