@@ -21,11 +21,6 @@
 #define SWFTYPES_H
 
 #include "compat.h"
-#ifdef LLVM_28
-#include <llvm/System/DataTypes.h>
-#else
-#include <llvm/Support/DataTypes.h>
-#endif
 #include <iostream>
 #include <fstream>
 #include <vector>
@@ -45,25 +40,47 @@
 #ifdef BIG_ENDIAN
 #include <algorithm>
 #endif
+#include <glibmm/ustring.h>
 
 namespace lightspark
 {
 
-#define ASFUNCTION(name) \
-	static ASObject* name(ASObject* , ASObject* const* args, const unsigned int argslen)
-#define ASFUNCTIONBODY(c,name) \
-	ASObject* c::name(ASObject* obj, ASObject* const* args, const unsigned int argslen)
-
-#define CLASSBUILDABLE(className) \
-	friend class Class<className>; 
-
 enum SWFOBJECT_TYPE { T_OBJECT=0, T_INTEGER=1, T_NUMBER=2, T_FUNCTION=3, T_UNDEFINED=4, T_NULL=5, T_STRING=6, 
-	T_DEFINABLE=7, T_BOOLEAN=8, T_ARRAY=9, T_CLASS=10, T_QNAME=11, T_NAMESPACE=12, T_UINTEGER=13, T_PROXY=14};
+	T_DEFINABLE=7, T_BOOLEAN=8, T_ARRAY=9, T_CLASS=10, T_QNAME=11, T_NAMESPACE=12, T_UINTEGER=13, T_PROXY=14, T_TEMPLATE=15};
 
 enum STACK_TYPE{STACK_NONE=0,STACK_OBJECT,STACK_INT,STACK_UINT,STACK_NUMBER,STACK_BOOLEAN};
+inline std::ostream& operator<<(std::ostream& s, const STACK_TYPE& st)
+{
+	switch(st)
+	{
+	case STACK_NONE:
+		s << "none";
+		break;
+	case STACK_OBJECT:
+		s << "object";
+		break;
+	case STACK_INT:
+		s << "int";
+		break;
+	case STACK_UINT:
+		s << "uint";
+		break;
+	case STACK_NUMBER:
+		s << "number";
+		break;
+	case STACK_BOOLEAN:
+		s << "boolean";
+		break;
+	default:
+		assert(false);
+	}
+	return s;
+}
 
 enum TRISTATE { TFALSE=0, TTRUE, TUNDEFINED };
 
+enum FILE_TYPE { FT_UNKNOWN=0, FT_SWF, FT_COMPRESSED_SWF, FT_PNG, FT_JPEG, FT_GIF };
+
 typedef double number_t;
 
 class ASObject;
@@ -128,6 +145,13 @@ public:
 	}
 	tiny_string(const tiny_string& r):buf(_buf_static),stringSize(r.stringSize),type(STATIC)
 	{
+		//Fast path for static read-only strings
+		if(r.type==READONLY)
+		{
+			type=READONLY;
+			buf=r.buf;
+			return;
+		}
 		if(stringSize > STATIC_SIZE)
 			createBuffer(stringSize);
 		strcpy(buf,r.buf);
@@ -138,6 +162,12 @@ public:
 			createBuffer(stringSize);
 		strcpy(buf,r.c_str());
 	}
+	tiny_string(const Glib::ustring& r):buf(_buf_static),stringSize(r.bytes()+1),type(STATIC)
+	{
+		if(stringSize > STATIC_SIZE)
+			createBuffer(stringSize);
+		strcpy(buf,r.c_str());
+	}
 	~tiny_string()
 	{
 		resetToStatic();
@@ -154,9 +184,18 @@ public:
 	{
 		resetToStatic();
 		stringSize=s.stringSize;
-		if(stringSize > STATIC_SIZE)
-			createBuffer(stringSize);
-		strcpy(buf,s.buf);
+		//Fast path for static read-only strings
+		if(s.type==READONLY)
+		{
+			type=READONLY;
+			buf=s.buf;
+		}
+		else
+		{
+			if(stringSize > STATIC_SIZE)
+				createBuffer(stringSize);
+			strcpy(buf,s.buf);
+		}
 		return *this;
 	}
 	tiny_string& operator=(const std::string& s)
@@ -168,6 +207,15 @@ public:
 		strcpy(buf,s.c_str());
 		return *this;
 	}
+	tiny_string& operator=(const Glib::ustring& s)
+	{
+		resetToStatic();
+		stringSize=s.bytes()+1;
+		if(stringSize > STATIC_SIZE)
+			createBuffer(stringSize);
+		strcpy(buf,s.c_str());
+		return *this;
+	}
 	tiny_string& operator=(const char* s)
 	{
 		makePrivateCopy(s);
@@ -188,6 +236,22 @@ public:
 
 		return strcmp(buf,r.buf)==0;
 	}
+	bool operator==(const Glib::ustring& r) const
+	{
+		//The length is checked as an optimization before checking the contents
+		if(stringSize != r.bytes()+1)
+			return false;
+
+		return strcmp(buf,r.c_str())==0;
+	}
+	bool operator==(const std::string& r) const
+	{
+		//The length is checked as an optimization before checking the contents
+		if(stringSize != r.size()+1)
+			return false;
+
+		return strcmp(buf,r.c_str())==0;
+	}
 	bool operator!=(const tiny_string& r) const
 	{
 		//The length is checked as an optimization before checking the contents
@@ -232,6 +296,7 @@ public:
 		else
 			return ns<r.ns;
 	}
+	tiny_string getQualifiedName() const;
 };
 
 class UI8 
@@ -450,6 +515,9 @@ struct multiname
 	*/
 	tiny_string normalizedName() const;
 	tiny_string qualifiedString() const;
+	/* sets name_type, name_s/name_d based on the object n */
+	void setName(ASObject* n);
+	bool isQName() const { return ns.size() == 1; }
 };
 
 class FLOAT 
@@ -505,9 +573,11 @@ class RGB
 public:
 	RGB(){};
 	RGB(int r,int g, int b):Red(r),Green(g),Blue(b){};
+	RGB(uint color):Red((color>>16)&0xFF),Green((color>>8)&0xFF),Blue(color&0xFF){}
 	UI8 Red;
 	UI8 Green;
 	UI8 Blue;
+	uint32_t toUInt() const { return Blue + (Green<<8) + (Red<<16); }
 };
 
 class RGBA
@@ -831,6 +901,9 @@ public:
 	RECT(int xmin, int xmax, int ymin, int ymax);
 };
 
+template<class T> class Vector2Tmpl;
+typedef Vector2Tmpl<double> Vector2f;
+
 class MATRIX
 {
 	friend std::istream& operator>>(std::istream& stream, MATRIX& v);
@@ -846,9 +919,11 @@ public:
 	MATRIX():ScaleX(1),ScaleY(1),RotateSkew0(0),RotateSkew1(0),TranslateX(0),TranslateY(0){}
 	void get4DMatrix(float matrix[16]) const;
 	void multiply2D(number_t xin, number_t yin, number_t& xout, number_t& yout) const;
+	Vector2f multiply2D(const Vector2f& in) const;
 	MATRIX multiplyMatrix(const MATRIX& r) const;
-	const bool operator!=(const MATRIX& r) const;
+	bool operator!=(const MATRIX& r) const;
 	MATRIX getInverted() const;
+	bool isInvertible() const;
 };
 
 class GRADRECORD
@@ -1292,16 +1367,16 @@ public:
 class RunState
 {
 public:
+	int last_FP;
 	unsigned int FP;
 	unsigned int next_FP;
 	bool stop_FP;
 	bool explicit_FP;
 	RunState();
-	void prepareNextFP();
 };
 
-ASObject* abstract_i(intptr_t i);
-ASObject* abstract_b(bool i);
+ASObject* abstract_i(int32_t i);
+ASObject* abstract_ui(uint32_t i);
 ASObject* abstract_d(number_t i);
 
 void stringToQName(const tiny_string& tmp, tiny_string& name, tiny_string& ns);
